{*

 NOTES:

   Each category of the ASI (e.g., Medical, Legal) has its own scroll box on the
   TabSheet.

   It is designed to allow rapid keyboard data-entry, although mouse data-entry is
   also supported. For keyboard entry, a response will score the current item and
   move automatically to the next item (speed tab). The speed tab can be disengaged
   by the user.

   This form presents three versions of the ASI: Full, Lite & Follow-up. In most
   aspects the DHCP presentation, business rules, and possible responses are used
   here.  There are exceptions. On the DHCP version most items had four possible
   responses: "0. No," "1. Yes," "X. Not Answered," & "N. Not Appliciable" but in
   this version, there are few items that have a "N" as a possible response. See
   page 92 of the ASI Manual and Question by Question Guide, VA Training 1997 for
   busines rules for the "N" response.

   The following questions do not appear on the Lite version but do appear on the
   followup version: M9, E8, E19, E20, E22, D14, D17, D18, D21, D22, D33, L22, L23,
   L30, F9, F10, F11, F27, F33, F35, F36, P14, P15, P16, P17, P18, P19, P20. Thus
   the Lite-Followup is longer than the Lite by 28 items.

   The position of questions, responses, lables, etc. are adjusted at run-time
   for top alignment. The left alignment is set at design time.

   Allowed values for income, items E12, E13, E14, E15, E16, E17, are whole numbers,
   no decimals or "$" sign. Range is 0..9999.

   Since the letters "X" and "N" are required in spin edit functions,
   the TMHA_SpnEdt component is used.

   If this ASI is a partially completed ASI that the clinician (user) is
   finishing then set "Partial" to 'A' for Full ASI,
                                   'L' for Lite
                                   'F' for Followup
                                   ' ' for new ASI

  508 Issues.

    1. The radio groups have been removed, replaced with list boxes. The "rgrp"
       notation is used to indicate list boxes here. This maintains consistency
       with other units involved in saving and building instruments.
    2. Tab key will cycle through all items on a screen, including the comment box.
    3. Items are organized so that only one label or component are on a line.
    4. Some captions have been changed to aid the screen reader.
    5. Highlighting active component and speed-tab feature have been removed.
    6. No hints, F1 help only.

}


unit fMHA_ASI_Entry_508;

interface

uses
  Classes,
  ComCtrls,
  Controls,
  ExtCtrls,
  Forms,
  Graphics,
  Mask,
  Menus,
  Messages,
  RzCmboBx,
  RzEdit,
  RzSplit,
  StdCtrls,
  Windows,
  StExpr,
  ORCtrls,
  Fmcntrls,
  SysUtils,
  Buttons,
  StBase,
  uMHA_SpnEdt,
  RzPanel,
  RzSpnEdt;

type

  TfrmMHA_ASI_Entry_508 = class(TForm)
    MainMenu1: TMainMenu;
    mnuFile: TMenuItem;
    mnuClose: TMenuItem;
    mnuHelp: TMenuItem;
    pgctrlASI: TPageControl;
    TabGeneral: TTabSheet;
    TabMedical: TTabSheet;
    TabEmploy: TTabSheet;
    TabDrug: TTabSheet;
    TabLegal: TTabSheet;
    TabFamily: TTabSheet;
    TabSocial: TTabSheet;
    TabPsych: TTabSheet;
    TabSpirit: TTabSheet;
    TabLeisure: TTabSheet;
    spltEmploy: TRzSplitter;
    spltDrug: TRzSplitter;
    spltLegal: TRzSplitter;
    spltFamily: TRzSplitter;
    spltSocial: TRzSplitter;
    Panel2: TPanel;
    pnlGeneral: TPanel;
    pnlEmploy: TPanel;
    pnlDrug: TPanel;
    pnlLegal: TPanel;
    pnlFamily: TPanel;
    pnlSocial: TPanel;
    Panel1: TPanel;
    spltPsych: TRzSplitter;
    pnlPsych: TPanel;
    spltMedical: TRzSplitter;
    pnlMedical: TPanel;
    pnlVerify: TPanel;
    lblVerify: TLabel;
    memLeisure: TFMMemo;
    memSpirit: TFMMemo;
    mnuASIPopUp: TPopupMenu;
    N1: TMenuItem;
    mnuAbout: TMenuItem;
    mnuContents: TMenuItem;
    mnuPUHelp: TMenuItem;
    mnuOnlineSupport: TMenuItem;
    N2: TMenuItem;
    btnFinished: TBitBtn;
    spltGeneral: TRzSplitter;
    sbGeneral1: TScrollBox;
    GroupBox1: TGroupBox;
    cbxG3: TListBox;
    GroupBox2: TGroupBox;
    dtpG4: TEdit;
    GroupBox3: TGroupBox;
    dtpG5: TEdit;
    GroupBox4: TGroupBox;
    rgrpG9: TListBox;
    GroupBox5: TGroupBox;
    GroupBox6: TGroupBox;
    cbxG11Interviewerx: TORComboBox;
    cbxG11OrderedByx: TORComboBox;
    GroupBox7: TGroupBox;
    rgrpG12: TListBox;
    GroupBox8: TGroupBox;
    edtG14y: TMHA_SpnEdt;
    GroupBox9: TGroupBox;
    edtG14m: TMHA_SpnEdt;
    GroupBox10: TGroupBox;
    rgrpG15: TListBox;
    GroupBox11: TGroupBox;
    rgrpG17: TListBox;
    GroupBox12: TGroupBox;
    rgrpG18: TListBox;
    GroupBox13: TGroupBox;
    rgrpG19: TListBox;
    GroupBox14: TGroupBox;
    edtG19e: TRzEdit;
    GroupBox15: TGroupBox;
    edtG20: TMHA_SpnEdt;
    GroupBox16: TGroupBox;
    memGeneral: TFMMemo;
    sbMedical1: TScrollBox;
    gbM1: TGroupBox;
    edtM1: TMHA_SpnEdt;
    gbM2: TGroupBox;
    edtM2y: TMHA_SpnEdt;
    gbM2a: TGroupBox;
    edtM2m: TMHA_SpnEdt;
    gbM3: TGroupBox;
    rgrpM3: TListBox;
    gbM3S: TGroupBox;
    edtM3e: TRzEdit;
    gbM4: TGroupBox;
    rgrpM4: TListBox;
    gbM5: TGroupBox;
    rgrpM5: TListBox;
    gbM5S: TGroupBox;
    edtM5e: TRzEdit;
    gbM6: TGroupBox;
    edtM6: TMHA_SpnEdt;
    gbM7: TGroupBox;
    rgrpM7: TListBox;
    gbM8: TGroupBox;
    rgrpM8: TListBox;
    gbM9: TGroupBox;
    rgrpM9: TListBox;
    gbM10: TGroupBox;
    rgrpM10: TListBox;
    gbM11: TGroupBox;
    rgrpM11: TListBox;
    gbMcomment: TGroupBox;
    memMedical: TFMMemo;
    sbEmploy1: TScrollBox;
    gbE1y: TGroupBox;
    edtE1y: TMHA_SpnEdt;
    gbE1m: TGroupBox;
    edtE1m: TMHA_SpnEdt;
    gbE2: TGroupBox;
    edtE2: TMHA_SpnEdt;
    gbE3: TGroupBox;
    rgrpE3: TListBox;
    gbE3S: TGroupBox;
    edtE3e: TRzEdit;
    gbE4: TGroupBox;
    rgrpE4: TListBox;
    gbE5: TGroupBox;
    rgrpE5: TListBox;
    gbE6y: TGroupBox;
    edtE6y: TMHA_SpnEdt;
    gbE6m: TGroupBox;
    edtE6m: TMHA_SpnEdt;
    gbE7: TGroupBox;
    rgrpE7: TListBox;
    btnHollingshead: TButton;
    gb7S: TGroupBox;
    edtE7e: TRzEdit;
    gbE8: TGroupBox;
    rgrpE8: TListBox;
    gbE9: TGroupBox;
    rgrpE9: TListBox;
    gbE10: TGroupBox;
    rgrpE10: TListBox;
    gbE11: TGroupBox;
    edtE11: TMHA_SpnEdt;
    gbE12: TGroupBox;
    edtE12: TEdit;
    gbE13: TGroupBox;
    edtE13: TEdit;
    gbE14: TGroupBox;
    edtE14: TEdit;
    gbE15: TGroupBox;
    edtE15: TEdit;
    gbE16: TGroupBox;
    edtE16: TEdit;
    gbE17: TGroupBox;
    edtE17: TEdit;
    gbE18: TGroupBox;
    edtE18: TMHA_SpnEdt;
    gbE19: TGroupBox;
    edtE19: TMHA_SpnEdt;
    gbE20: TGroupBox;
    rgrpE20: TListBox;
    gbE21: TGroupBox;
    rgrpE21: TListBox;
    gbE22: TGroupBox;
    rgrpE22: TListBox;
    gbE23: TGroupBox;
    rgrpE23: TListBox;
    gbE24: TGroupBox;
    rgrpE24: TListBox;
    gbEmployComment: TGroupBox;
    memEmploy: TFMMemo;
    sbDrug1: TScrollBox;
    gbD1d: TGroupBox;
    edtD1d: TMHA_SpnEdt;
    gbD1y: TGroupBox;
    edtD1y: TMHA_SpnEdt;
    gbD1r: TGroupBox;
    gbD2d: TGroupBox;
    edtD2d: TMHA_SpnEdt;
    gbD2y: TGroupBox;
    edtD2y: TMHA_SpnEdt;
    gbD2r: TGroupBox;
    gbD3d: TGroupBox;
    edtD3d: TMHA_SpnEdt;
    gbD3y: TGroupBox;
    edtD3y: TMHA_SpnEdt;
    gbD3r: TGroupBox;
    gbD4d: TGroupBox;
    edtD4d: TMHA_SpnEdt;
    gbD4y: TGroupBox;
    edtD4y: TMHA_SpnEdt;
    gbD4r: TGroupBox;
    gbD5d: TGroupBox;
    edtD5d: TMHA_SpnEdt;
    gbD5y: TGroupBox;
    edtD5y: TMHA_SpnEdt;
    gbD5r: TGroupBox;
    gbD6d: TGroupBox;
    edtD6d: TMHA_SpnEdt;
    gbD6y: TGroupBox;
    edtD6y: TMHA_SpnEdt;
    gbD6r: TGroupBox;
    gbD7d: TGroupBox;
    edtD7d: TMHA_SpnEdt;
    gbD7y: TGroupBox;
    edtD7y: TMHA_SpnEdt;
    gbD7r: TGroupBox;
    gbD8d: TGroupBox;
    edtD8d: TMHA_SpnEdt;
    gbD8y: TGroupBox;
    edtD8y: TMHA_SpnEdt;
    gbD8r: TGroupBox;
    gbD9d: TGroupBox;
    edtD9d: TMHA_SpnEdt;
    gbD9y: TGroupBox;
    edtD9y: TMHA_SpnEdt;
    gbD9r: TGroupBox;
    gbD10d: TGroupBox;
    edtD10d: TMHA_SpnEdt;
    gbD10y: TGroupBox;
    edtD10y: TMHA_SpnEdt;
    gbD10r: TGroupBox;
    gbD11d: TGroupBox;
    edtD11d: TMHA_SpnEdt;
    gbD11y: TGroupBox;
    edtD11y: TMHA_SpnEdt;
    gbD11r: TGroupBox;
    gbD12d: TGroupBox;
    edtD12d: TMHA_SpnEdt;
    gbD12y: TGroupBox;
    edtD12y: TMHA_SpnEdt;
    gbD12r: TGroupBox;
    gbD13d: TGroupBox;
    edtD13d: TMHA_SpnEdt;
    gbD13y: TGroupBox;
    edtD13y: TMHA_SpnEdt;
    gbD14: TGroupBox;
    rgrpD14: TListBox;
    gbD15: TGroupBox;
    edtD15: TMHA_SpnEdt;
    gbD16: TGroupBox;
    edtD16: TMHA_SpnEdt;
    gbD17: TGroupBox;
    edtD17: TMHA_SpnEdt;
    gbD18: TGroupBox;
    edtD18: TMHA_SpnEdt;
    gbD19: TGroupBox;
    edtD19: TMHA_SpnEdt;
    gbD20: TGroupBox;
    edtD20: TMHA_SpnEdt;
    gbD21: TGroupBox;
    edtD21: TMHA_SpnEdt;
    gbD22: TGroupBox;
    edtD22: TMHA_SpnEdt;
    gbD23: TGroupBox;
    edtD23: TMHA_SpnEdt;
    gbD24: TGroupBox;
    EdtD24: TMHA_SpnEdt;
    gbD25: TGroupBox;
    edtD25: TMHA_SpnEdt;
    gbD26: TGroupBox;
    edtD26: TMHA_SpnEdt;
    gbD27: TGroupBox;
    edtD27: TMHA_SpnEdt;
    gbD28: TGroupBox;
    rgrpD28: TListBox;
    gbD29: TGroupBox;
    rgrpD29: TListBox;
    gbD30: TGroupBox;
    rgrpD30: TListBox;
    gbD31: TGroupBox;
    rgrpD31: TListBox;
    gbD32: TGroupBox;
    rgrpD32: TListBox;
    gbD33: TGroupBox;
    rgrpD33: TListBox;
    gbD34: TGroupBox;
    rgrpD34: TListBox;
    gbD35: TGroupBox;
    rgrpD35: TListBox;
    GroupBox17: TGroupBox;
    memDrug: TFMMemo;
    sbLegal1: TScrollBox;
    gbL1: TGroupBox;
    rgrpL1: TListBox;
    gbL2: TGroupBox;
    rgrpL2: TListBox;
    gbL3: TGroupBox;
    edtL3: TMHA_SpnEdt;
    gbL4: TGroupBox;
    edtL4: TMHA_SpnEdt;
    gbL5: TGroupBox;
    edtL5: TMHA_SpnEdt;
    gbL6: TGroupBox;
    edtL6: TMHA_SpnEdt;
    gbL7: TGroupBox;
    edtL7: TMHA_SpnEdt;
    gbL8: TGroupBox;
    edtL8: TMHA_SpnEdt;
    gbL9: TGroupBox;
    edtL9: TMHA_SpnEdt;
    gbL10: TGroupBox;
    edtL10: TMHA_SpnEdt;
    gbL11: TGroupBox;
    edtL11: TMHA_SpnEdt;
    gbL12: TGroupBox;
    edtL12: TMHA_SpnEdt;
    gbL13: TGroupBox;
    edtL13: TMHA_SpnEdt;
    gbL14: TGroupBox;
    edtL14: TMHA_SpnEdt;
    gbL15: TGroupBox;
    edtL15: TMHA_SpnEdt;
    gbL16: TGroupBox;
    edtL16: TMHA_SpnEdt;
    gbL17: TGroupBox;
    edtL17: TMHA_SpnEdt;
    gbL18: TGroupBox;
    edtL18: TMHA_SpnEdt;
    gbL19: TGroupBox;
    edtL19: TMHA_SpnEdt;
    gbL20: TGroupBox;
    edtL20: TMHA_SpnEdt;
    gbL21: TGroupBox;
    edtL21: TMHA_SpnEdt;
    gbL22: TGroupBox;
    edtL22: TMHA_SpnEdt;
    gbL23: TGroupBox;
    rgrpL23: TListBox;
    gbL24: TGroupBox;
    rgrpL24: TListBox;
    gbL25: TGroupBox;
    rgrpL25: TListBox;
    gbL26: TGroupBox;
    edtL26: TMHA_SpnEdt;
    gbL27: TGroupBox;
    edtL27: TMHA_SpnEdt;
    gbL28: TGroupBox;
    rgrpL28: TListBox;
    gbL29: TGroupBox;
    rgrpL29: TListBox;
    gbL30: TGroupBox;
    rgrpL30: TListBox;
    gbL31: TGroupBox;
    rgrpL31: TListBox;
    gbL32: TGroupBox;
    rgrpL32: TListBox;
    GroupBox20: TGroupBox;
    memLegal: TFMMemo;
    sbFamily1: TScrollBox;
    gbH1a: TGroupBox;
    gbH1d: TGroupBox;
    gbH1p: TGroupBox;
    gbH2a: TGroupBox;
    lbH1a: TListBox;
    lbH1d: TListBox;
    lbH1p: TListBox;
    lbH2a: TListBox;
    gbH2d: TGroupBox;
    lbH2d: TListBox;
    gbH2p: TGroupBox;
    lbH2p: TListBox;
    gbH3a: TGroupBox;
    lbH3a: TListBox;
    gbH3d: TGroupBox;
    lbH3d: TListBox;
    gbH3p: TGroupBox;
    lbH3p: TListBox;
    gbH4a: TGroupBox;
    lbH4a: TListBox;
    gbH4d: TGroupBox;
    lbH4d: TListBox;
    gbH4p: TGroupBox;
    lbH4p: TListBox;
    gbH5a: TGroupBox;
    lbH5a: TListBox;
    gbH5d: TGroupBox;
    lbH5d: TListBox;
    gbH5p: TGroupBox;
    lbH5p: TListBox;
    gbH6a: TGroupBox;
    lbH6a: TListBox;
    gbH6d: TGroupBox;
    lbH6d: TListBox;
    gbH6p: TGroupBox;
    lbH6p: TListBox;
    gbH7a: TGroupBox;
    lbH7a: TListBox;
    gbH7d: TGroupBox;
    lbH7d: TListBox;
    gbH7p: TGroupBox;
    lbH7p: TListBox;
    gbH8a: TGroupBox;
    lbH8a: TListBox;
    gbH8d: TGroupBox;
    lbH8d: TListBox;
    gbH8p: TGroupBox;
    lbH8p: TListBox;
    gbH9a: TGroupBox;
    lbH9a: TListBox;
    gbH9d: TGroupBox;
    lbH9d: TListBox;
    gbH9p: TGroupBox;
    lbH9p: TListBox;
    gbH10a: TGroupBox;
    lbH10a: TListBox;
    gbH10d: TGroupBox;
    lbH10d: TListBox;
    gbH10p: TGroupBox;
    lbH10p: TListBox;
    gbH11a: TGroupBox;
    lbH11a: TListBox;
    gbH11d: TGroupBox;
    lbH11d: TListBox;
    gbH11p: TGroupBox;
    lbH11p: TListBox;
    gbH12a: TGroupBox;
    lbH12a: TListBox;
    gbH12d: TGroupBox;
    lbH12d: TListBox;
    gbH12p: TGroupBox;
    lbH12p: TListBox;
    GroupBox21: TGroupBox;
    memFamily: TFMMemo;
    sbSocial1: TScrollBox;
    gbF1: TGroupBox;
    rgrpF1: TListBox;
    gbF2y: TGroupBox;
    edtF2y: TMHA_SpnEdt;
    gbF2m: TGroupBox;
    edtF2m: TMHA_SpnEdt;
    gbF3: TGroupBox;
    rgrpF3: TListBox;
    gbF4: TGroupBox;
    rgrpF4: TListBox;
    gbF5y: TGroupBox;
    edtF5y: TMHA_SpnEdt;
    gbF5m: TGroupBox;
    edtF5m: TMHA_SpnEdt;
    gbF6: TGroupBox;
    rgrpF6: TListBox;
    gbF7: TGroupBox;
    rgrpF7: TListBox;
    gbF8: TGroupBox;
    rgrpF8: TListBox;
    gbF9: TGroupBox;
    rgrpF9: TListBox;
    gbF10: TGroupBox;
    rgrpF10: TListBox;
    gbF11: TGroupBox;
    edtF11: TMHA_SpnEdt;
    gbF12: TGroupBox;
    lbF12: TListBox;
    gbF13: TGroupBox;
    lbF13: TListBox;
    gbF14: TGroupBox;
    lbF14: TListBox;
    gbF15: TGroupBox;
    lbF15: TListBox;
    gbF16: TGroupBox;
    lbF16: TListBox;
    gbF17: TGroupBox;
    lbF17: TListBox;
    gbF18: TGroupBox;
    rgrpF18: TListBox;
    gbF18l: TGroupBox;
    rgrpF18l: TListBox;
    gbF19: TGroupBox;
    rgrpF19: TListBox;
    gbF19l: TGroupBox;
    rgrpF19l: TListBox;
    gbF20: TGroupBox;
    rgrpF20: TListBox;
    gbF20l: TGroupBox;
    rgrpF20l: TListBox;
    gbF21: TGroupBox;
    rgrpF21: TListBox;
    gbF21l: TGroupBox;
    rgrpF21l: TListBox;
    gbF22: TGroupBox;
    rgrpF22: TListBox;
    gbF22l: TGroupBox;
    rgrpF22l: TListBox;
    gbF23: TGroupBox;
    rgrpF23: TListBox;
    gbF23l: TGroupBox;
    rgrpF23l: TListBox;
    gbF24: TGroupBox;
    rgrpF24: TListBox;
    gbF24l: TGroupBox;
    rgrpF24l: TListBox;
    gbF25: TGroupBox;
    rgrpF25: TListBox;
    gbF25l: TGroupBox;
    rgrpF25l: TListBox;
    gbF26: TGroupBox;
    rgrpF26: TListBox;
    gbF26l: TGroupBox;
    rgrpF26l: TListBox;
    gbF27: TGroupBox;
    rgrpF27: TListBox;
    gbF27l: TGroupBox;
    rgrpF27l: TListBox;
    gbF28: TGroupBox;
    rgrpF28: TListBox;
    gbF28l: TGroupBox;
    rgrpF28l: TListBox;
    gbF29: TGroupBox;
    rgrpF29: TListBox;
    gbF29l: TGroupBox;
    rgrpF29l: TListBox;
    gbF30: TGroupBox;
    edtF30: TMHA_SpnEdt;
    gbF31: TGroupBox;
    edtF31: TMHA_SpnEdt;
    gbF32: TGroupBox;
    rgrpF32: TListBox;
    gbF33: TGroupBox;
    rgrpF33: TListBox;
    gbF34: TGroupBox;
    rgrpF34: TListBox;
    gbF35: TGroupBox;
    rgrpF35: TListBox;
    gbF36: TGroupBox;
    rgrpF36: TListBox;
    gbF37: TGroupBox;
    rgrpF37: TListBox;
    gbF38: TGroupBox;
    rgrpF38: TListBox;
    GroupBox23: TGroupBox;
    memSocial: TFMMemo;
    sbPsych1: TScrollBox;
    gbP1a: TGroupBox;
    edtP1a: TMHA_SpnEdt;
    gbP1b: TGroupBox;
    edtP1b: TMHA_SpnEdt;
    gbP2: TGroupBox;
    rgrpP2: TListBox;
    gbP3: TGroupBox;
    rgrpP3: TListBox;
    gbP3l: TGroupBox;
    rgrpP3l: TListBox;
    gbP4: TGroupBox;
    rgrpP4: TListBox;
    gbP4l: TGroupBox;
    rgrpP4l: TListBox;
    gbP5: TGroupBox;
    rgrpP5: TListBox;
    gbP5l: TGroupBox;
    rgrpP5l: TListBox;
    gbP6: TGroupBox;
    rgrpP6: TListBox;
    gbP6l: TGroupBox;
    rgrpP6l: TListBox;
    gbP7: TGroupBox;
    rgrpP7: TListBox;
    gbP7l: TGroupBox;
    rgrpP7l: TListBox;
    gbP8: TGroupBox;
    rgrpP8: TListBox;
    gbP8l: TGroupBox;
    rgrpP8l: TListBox;
    gbP9: TGroupBox;
    rgrpP9: TListBox;
    gbP9l: TGroupBox;
    rgrpP9l: TListBox;
    gbP10: TGroupBox;
    rgrpP10: TListBox;
    gbP10l: TGroupBox;
    rgrpP10l: TListBox;
    gbP11: TGroupBox;
    edtP11: TMHA_SpnEdt;
    gbP12: TGroupBox;
    rgrpP12: TListBox;
    gbP13: TGroupBox;
    rgrpP13: TListBox;
    gbP14: TGroupBox;
    rgrpP14: TListBox;
    gbP15: TGroupBox;
    rgrpP15: TListBox;
    gbP16: TGroupBox;
    rgrpP16: TListBox;
    gbP17: TGroupBox;
    rgrpP17: TListBox;
    gbP18: TGroupBox;
    rgrpP18: TListBox;
    gbP19: TGroupBox;
    rgrpP19: TListBox;
    gbP20: TGroupBox;
    rgrpP20: TListBox;
    gbP21: TGroupBox;
    rgrpP21: TListBox;
    gbP22: TGroupBox;
    rgrpP22: TListBox;
    GroupBox22: TGroupBox;
    memPsych: TFMMemo;
    lbD1r: TListBox;
    lbD2r: TListBox;
    lbD3r: TListBox;
    lbD4r: TListBox;
    lbD5r: TListBox;
    lbD6r: TListBox;
    lbD7r: TListBox;
    lbD8r: TListBox;
    lbD9r: TListBox;
    lbD10r: TListBox;
    lbD11r: TListBox;
    lbD12r: TListBox;
    GroupBox24: TGroupBox;
    GroupBox25: TGroupBox;

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);

    procedure rgrpG19Click(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure rgrpG19Exit(Sender: TObject);
    procedure rgrpM5Click(Sender: TObject);
    procedure rgrpM3Click(Sender: TObject);
    procedure rgrpE3Click(Sender: TObject);
    procedure mnuCloseClick(Sender: TObject);
    procedure btnHollingsheadClick(Sender: TObject);
    procedure edtL22Exit(Sender: TObject);
    procedure rgrpG12Click(Sender: TObject);
    procedure rgrpE4Click(Sender: TObject);
    procedure rgrpD14Click(Sender: TObject);
    procedure rgrpL24Click(Sender: TObject);
    procedure rgrpE8Click(Sender: TObject);
    procedure pgctrlASIChange(Sender: TObject);
    procedure sbSocialEnter(Sender: TObject);
    procedure edtD15Change(Sender: TObject);
    procedure edtD20Change(Sender: TObject);
    procedure edtD19Change(Sender: TObject);
    procedure edtD27Change(Sender: TObject);
    procedure edtD26Change(Sender: TObject);
    procedure edtL21Change(Sender: TObject);
    procedure MemLeisureExit(Sender: TObject);
    procedure MemSpiritExit(Sender: TObject);
    procedure MemPsychExit(Sender: TObject);
    procedure MemSocialExit(Sender: TObject);
    procedure MemFamilyExit(Sender: TObject);
    procedure MemLegalExit(Sender: TObject);
    procedure MemDrugExit(Sender: TObject);
    procedure MemEmployExit(Sender: TObject);
    procedure MemMedicalExit(Sender: TObject);
    procedure edtF11Change(Sender: TObject);
    procedure pnlEndClick(Sender: TObject);
    (*
    procedure spltDrugExit(Sender: TObject);
    procedure spltSocialExit(Sender: TObject);
    procedure spltPsychExit(Sender: TObject);
    procedure splitMedicalExit(Sender: TObject);
    procedure spltLegalExit(Sender: TObject);
    procedure spltFamilyExit(Sender: TObject);
    procedure spltEmployExit(Sender: TObject);
    procedure spltGeneralExit(Sender: TObject);
    *)
    procedure cbxG11InterviewerxNeedData(Sender: TObject; const StartFrom:
      string;
      Direction, InsertAt: Integer);

    procedure edtM1Change(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

    procedure memGeneralExit(Sender: TObject);
    procedure btnFinishedClick(Sender: TObject);
    procedure lblVerifyClick(Sender: TObject);
    procedure CheckLife(Sender: TObject);
    procedure Check9999(Sender: TObject);
    procedure CheckOutL17(Sender: TObject);
    //function CheckDeathDate(DateIn: TDateTime): Boolean;
    procedure FormActivate(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuContentsClick(Sender: TObject);
    procedure mnuOnlineSupportClick(Sender: TObject);
    procedure cbxG11OrderedByxNeedData(Sender: TObject; const StartFrom: string;
      Direction, InsertAt: Integer);
    procedure RecoverCrashData;
    procedure CleanupTxtFiles;
    procedure FormResize(Sender: TObject);
    procedure ChangePT;
    procedure SpiritNextPageEnter(Sender: TObject);
    procedure cbxG3Exit(Sender: TObject);
    procedure dtpG4Exit(Sender: TObject);
    procedure edtG14yExit(Sender: TObject);
  private
    { Private declarations }
    procedure RecoverComments;
    function VerifyData(sender: TObject): Boolean;
    function CheckRange(StringIn: string; Max, Min: integer): Boolean;
    function SkippedItems: string;
    function SaveSign: boolean;
    procedure CrossCheck(WhichOne: string; sender: TObject);
    function ConductCrossChecks(sender: TObject): Boolean;
    procedure CleanupComments;
    function SignAndExit: Boolean;
    procedure FixinToClose(Sender: TObject);
  public
    { Public declarations }

    ProgramIEN, // blank for new records and found in frmMain for old records
      ResponseList: string;

    FASITimedOut,
      FUserTerminates,
      NoFollowupAllowed: Boolean;
    procedure ShowHint(Sender: TObject);
    procedure ASITimedOut;
    procedure RzEditOnExit(Sender: TObject);
    procedure EditOnExit(Sender: TObject);
    procedure SpinEditOnExit(Sender: TObject);
    procedure ListBoxOnExit(Sender: TObject);
    procedure RzComboBoxOnExit(Sender: TObject);
    procedure ORComboBoxOnExit(Sender: TObject);
    procedure DateOnExit(Sender: TObject);
  end;

var
  frmMHA_ASI_Entry_508        : TfrmMHA_ASI_Entry_508;

implementation

{$R *.DFM}

uses

  uMHA_Registry, // Windows registry read\write functions
  uMHA_ASI,
  uMHA_ASI_OPS,
  uMHA_LocalFiles,
  uMHA_ASI_Build_508, // Display the ASI form
  uMHA_ASI_Save_508, // Vista save operations
  fMHA_ASI_Close, // Form for user to select sign, save, ignore, etc.
  fIA_Dialogs, // Message dialog box with variable header
  fMHA_ASI_Sign, // For electronic signature
  uIA_Patient,
  uIA_Staff_OPS,
  uIA_ThisUser,
  uIA_Utils,
  uIA_Const,
  uMHA_Const,
  fMHA_Saved,
  fMHA_About,
  uMHA_DM;

var
  CrashFilePath,
    CrossCheckList            : string;
  lvInitialize,
    CheckSkipItems,
    CrossCheckItems,
    CrossChecksCompleted      : boolean;


  //-------------------------------------------------------------------
  //    Create
  //-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.FormCreate(Sender: TObject);
begin
  lvInitialize := True;
  Position := poMainFormCenter;
  Application.HelpFile := ExtractFilePath(Application.EXEName) +
    'YS_MHA.Hlp';

  ASI_Save_508 := TASI_Save_508.Create; // uMHA3_ASI_Save

  ASI_Build_508 := TASI_Build_508.Create; // uMHA3_ASI_Build_508

  FUserTerminates := False;
  FASITimedOut := false;

  CrashFilePath := LocalFiles.AnswerFilesPath + '\';

  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull then
    gvASI_Type := 1;
  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itLite then
    gvASI_Type := 2;
  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU then
    gvASI_Type := 3;
  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itMV then
    gvASI_Type := 4;

  spltDrug.Percent := 100;
  spltSocial.Percent := 100;
  spltPsych.Percent := 100;
  spltMedical.Percent := 100;
  spltLegal.Percent := 100;
  spltFamily.Percent := 100;
  spltEmploy.Percent := 100;
  spltGeneral.Position := 0;
end;


procedure TfrmMHA_ASI_Entry_508.FormActivate(Sender: TObject);
//var
//  r                           : TRect;
begin
  OpenFormName := ofASI_508;

  with frmMHA_ASI_Entry_508 do
    begin
      // Some label captions are very long. Expand window to max.

      //set form top and height to work area (i.e., desktop size minus task bars)
      //SystemParametersInfo(SPI_GetWorkArea, 0, @r, 0);
      //BoundsRect := r;
      //Top := r.Top;
      //Height := r.Bottom - r.Top;
      Height := 300; // Show one component at a time, otherwise JAWS reads all
      // List boxes on page.
      Width := 700; //r.Right - r.Left;
    end;

  // read windows registry
  MHA_Registry.ReadRegistry; // uMHA_Registry

  // Add these items to the list that the user makes active.
  // This is a list of items that the user has answered during the
  // course of the ASI. This avoids saving default items (user did not
  // actually make a response) on the M-side.
  ResponseList := 'DTPG5 RGRPG8 RGRPG12 ';

  // Save the items without verification when user terminates before completion.
  // Also do not save items that the user has not entered (no defaults allowed).
  FUserTerminates := false;

  CheckSkipItems := False; // toggle for clearing of required items
  CrossCheckItems := false; // toggle for clearing of cross-check items
  CrossChecksCompleted := false; // toggle when cross check done
  ProgramIEN := '';
  CrossCheckList := 'G20,M1,M2,M7,M8,E1y,E12a,E12b,E15,E20,D2d,D2y,D12d,' +
    'D14,D16,D17,D21a,D21b,D22a,D22b,D23,D24a,D24b,D30a,D30b,D31a,D31b,L17,L20,L24,' +
    'L25,L26,L27,F34a,F34b,F35a,F35b,P2,P9a,P9b,P10,P11a,P11b,P13a,P13b,';


  // Setup combo box strings, reset scroll bars, place form on screen, etc.
  ASI_Build_508.BuildBasics;

  ASI_Build_508.BuildForm;

  CleanupComments; // Removes M carriage returns & line feeds


  Caption := 'A S I data entry for patient ' + Patient.Name;

  pgctrlASI.ActivePage := TabGeneral;

  Screen.cursor := crDefault;

  MsgDlgFrm('User Information: To select an item in a list box use the enter key.',
    'User Information: To select an item in a list box use the enter key.',
    mtInformation, [mbOk], 0);

  ActiveControl := GroupBox1;
  lvInitialize := False;
  // ShowModal: call this from the main form, otherwise the SaveData
  // does not work when the form is timed out by Vista.
end;


//-------------------------------------------------------------------
//    Close, Clean-up, Destroy
//-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.FixinToClose(Sender: TObject);
var
  UserResponse                : Char;
  Reply                       : string;
begin
     // Show the Close Options
  try
    frmMHA_ASI_Close := TfrmMHA_ASI_Close.Create(nil);
    UserResponse := frmMHA_ASI_Close.CloseResponse;
    frmMHA_ASI_Close.free;
  except
    on E: Exception do
      begin
        MsgDlgFrm(E.Message, MHA_Caption,
          mtError, [mbOk], 0);
        Exit;
      end;
  end; {-try -}

  if UserResponse = 'E' then // Exit: Close now
    begin
      Close;
      exit;
    end;

  if UserResponse = 'C' then // Return to ASI: do not close
    Exit;

  if UserResponse = 'F' then // Finish later, don't sign
    // This is when the user decides to close the ASI, not the
    // vet. The items are not verified, it is not signed.
    begin
      // don't save ASI on dead vet
      //if not CheckDeathDate(dtpG4.DateTime) then
      //  Exit;
      //if not CheckDeathDate(dtpG5.DateTime) then
      //  Exit;

      FUserTerminates := true;

  // New administration.
      if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS = '' then
        TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS :=
          ASI_OPS.SaveNewInterview(
          Patient.DFN,
          dtpG5.Text, //(dtpG5.Date),
          Piece(cbxG11Interviewerx.Items[cbxG11Interviewerx.ItemIndex], U, 1),
          Piece(cbxG11OrderedByx.Items[cbxG11OrderedByx.ItemIndex], U, 1),
          TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType);

      Reply := ASI_Save_508.SaveASIData(
        TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS,
        Patient.DFN,
        TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerDUZ,
        TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByDUZ,
        DateToStr(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).ASIDate),
        ProgramIEN,
        TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType); // uMHA_ASI_Save

      if Reply <> OK then
        begin
          MsgDlgFrm(Reply, MHA_Caption, mtError, [mbOK], 0);
          exit;
        end;
      Close;
      exit;
    end;

  if UserResponse = 'S' then
    // if G12 is answered "N" then verify, save, & sign,
    // otherwise, simply save and sign.
    begin
      // don't save ASI on dead vet
      //if not CheckDeathDate(dtpG4.DateTime) then
      //  Exit;
      //if not CheckDeathDate(dtpG5.DateTime) then
      //  Exit;

      if rgrpG12.ItemIndex = 3 then // "N" meaning "Completed"
        begin
          // Any blank items?
          if not VerifyData(sender) then
            exit;

          // Cross checks
          if not CrossChecksCompleted then
            begin
              CrossCheckItems := true;

              if not ConductCrossChecks(sender) then
                begin
                  CrossChecksCompleted := false;
                  exit;
                end;
            end;
          if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).CanBeSignedByUser then
          // Save, get signature
            if not SignAndExit then
              Exit;

          Close;
        end
      else // G12 is not "N"
        begin
          if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).CanBeSignedByUser then
          // Save and sign the ASI. No checks conducted.
            if not SignAndExit then
              exit;

          Close;
        end;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.FormClose(Sender: TObject; var Action:
  TCloseAction);
begin

  gvLastASIProgram := cbxG3.ItemIndex;
  //gvSpeedTab := mnuSpeedTab.Checked;
  //gvShowHints := mnuShowHints.Checked;
  //gvDefaultSize := mnuDefaultSize.Checked;

  MHA_Registry.SaveRegistry;

  if gvCPRSShutDown then
    begin
      MsgDlgFrm('CPRS has been closed and this program will close too. Your ' +
        'work has been saved.', MHA_Caption, mtInformation, [mbOk], 0);
      Application.Terminate;
      Exit;
    end;

 (* if gvPtShutDown then
    begin
      MsgDlgFrm('The patient has been changed in CPRS. '+#13+#13+
      'To avoid patient safety issues this ASI will be saved as an incomplete'+
      'and this program will close.',
        MHA_Caption,        mtInformation, [mbOk], 0);
      Application.Terminate;
      Exit;
    end; *)

  CleanupTxtFiles;
end;

procedure TfrmMHA_ASI_Entry_508.FormDestroy(Sender: TObject);
begin
  if ASI_Save_508 <> nil then
    ASI_Save_508.Free;
  if ASI_Build_508 <> nil then
    ASI_Build_508.Free;
end;

procedure TfrmMHA_ASI_Entry_508.CleanupTxtFiles;
begin
  // Remove the MHA_Recover.txt file; crash protection not needed now with normal
  // program termination.
  if FileExists(LocalFiles.ASI_RecoveryFile) then
    DeleteFile(LocalFiles.ASI_RecoveryFile);

  if FileExists(CrashFilePath + 'ASI_General.txt') then
    DeleteFile(CrashFilePath + 'ASI_General.txt');
  if FileExists(CrashFilePath + 'ASI_Medical.txt') then
    DeleteFile(CrashFilePath + 'ASI_Medical.txt');
  if FileExists(CrashFilePath + 'ASI_Employ.txt') then
    DeleteFile(CrashFilePath + 'ASI_Employ.txt');
  if FileExists(CrashFilePath + 'ASI_Drug.txt') then
    DeleteFile(CrashFilePath + 'ASI_Drug.txt');
  if FileExists(CrashFilePath + 'ASI_Legal.txt') then
    DeleteFile(CrashFilePath + 'ASI_Legal.txt');
  if FileExists(CrashFilePath + 'ASI_Family.txt') then
    DeleteFile(CrashFilePath + 'ASI_Family.txt');
  if FileExists(CrashFilePath + 'ASI_Social.txt') then
    DeleteFile(CrashFilePath + 'ASI_Social.txt');
  if FileExists(CrashFilePath + 'ASI_Psych.txt') then
    DeleteFile(CrashFilePath + 'ASI_Psych.txt');
  if FileExists(CrashFilePath + 'ASI_Spirit.txt') then
    DeleteFile(CrashFilePath + 'ASI_Spirit.txt');
  if FileExists(CrashFilePath + 'ASI_Leisure.txt') then
    DeleteFile(CrashFilePath + 'ASI_Leisure.txt');

end;


//-------------------------------------------------------------------
//        Buttons and panels
//-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.btnHollingsheadClick(Sender: TObject);
begin
  Application.HelpContext(1);
end;

procedure TfrmMHA_ASI_Entry_508.btnFinishedClick(Sender: TObject);
begin
  FixinToClose(sender);
end;

procedure TfrmMHA_ASI_Entry_508.pnlEndClick(Sender: TObject);
begin
  FixinToClose(sender);
end;

procedure TfrmMHA_ASI_Entry_508.lblVerifyClick(Sender: TObject);
begin
  lvInitialize := false;
  if CrossCheckItems then
    ConductCrossChecks(sender);
end;


//-------------------------------------------------------------------
//           Crash Recovery
//-------------------------------------------------------------------


// In order to recover from an unexpected crash (power goes off, user
// turns off PC without exiting program, etc.) the user's responses
// are placed in a overwritten text file. This list is deleted upon a normal
// close of the program. The existence of the text file when the program is
// launched indicates that a crash occurred during the last use.

procedure TfrmMHA_ASI_Entry_508.RecoverCrashData;
// To recover the data, first the text file is read to the ASI form (full, lite
// or followup) then the form is saved in the normal way. The Form is not
// displayed during this process.
var
  i                           : integer;
  TempRecordIENS,
    TempPt,
    TempInterviewer,
    TempOrderer,
    TempStr                   : string;
  f                           : TextFile;
  TempInterviewType           : TInterviewType;


  function GetNextItem: string;
  begin
    TempStr := '';
    // Item click are set off by this, resetlvInitialize a must
    lvInitialize := true;
    ReadLn(f, TempStr);
    Result := TempStr;
  end;

begin

  TempStr := '';
  TempRecordIENS := '';
  TempInterviewType := itFull;

  Screen.Cursor := crHourglass;
  try
    // Crash-File setup
    AssignFile(f, LocalFiles.ASI_RecoveryFile);
    Reset(f);

    // First line stores the type of ASI in use, RecordIENS, Patient.DFN, etc.
    ReadLn(f, TempStr);
    TempRecordIENS := Piece(TempStr, '^', 2); // do not put '`' here
    TempPt := Piece(TempStr, '^', 3); // do not put '`' here
    TempInterviewer := Piece(TempStr, '^', 4); // do not put '`' here
    TempOrderer := Piece(TempStr, '^', 5); // do not put '`' here
    ProgramIEN := Piece(TempStr, '^', 6); // do not put '`' here

    if Piece(TempStr, '^', 1) = ASI_FULL then
      begin
        TempInterviewType := itFull;
        ASI_Build_508.Setup_ASIFull_508;
        gvASI_Type := 1;
      end;
    if Piece(TempStr, '^', 1) = ASI_LITE then
      begin
        TempInterviewType := itLite;
        ASI_Build_508.Setup_ASILite_508;
        gvASI_Type := 2;
      end;
    if Piece(TempStr, '^', 1) = ASI_FOLLOWUP then
      begin
        TempInterviewType := itFU;
        ASI_Build_508.Setup_ASIFollowup_508;
        gvASI_Type := 3;
      end;

    //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewTypeStr := Piece(TempStr, U, 1);
    //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS := Piece(TempStr, '^', 2); // do not put '`' here
    //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerDUZ := Piece(TempStr, '^', 4); // do not put '`' here
    //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByDUZ := Piece(TempStr, '^', 5); // do not put '`' here
    //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType :=
    //  ASI_OPS.StrToInterviewType(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewTypeStr);
    //case TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType of
    //  itFull: ASI_SetupFull;
    //  itLite: ASI_SetupLite;
    //  itFU: ASI_SetupFollowup;
    //end;

  except
    on E: Exception do
      begin
        showmessage(e.message);
        exit;
      end;
  end;

  try
    // Run through all the components, place the value from the
    // crash file in the component.
    for i := 0 to ComponentCount - 1 do
      begin
        if Components[i] is TEdit then
          TEdit(Components[i]).Text := GetNextItem;
        if Components[i] is TRzEdit then
          TRzEdit(Components[i]).Text := GetNextItem;
        if Components[i] is TDateTimePicker then
          TDateTimePicker(Components[i]).Date := StrToDate(GetNextItem);
        if Components[i] is TListBox then
          TListBox(Components[i]).ItemIndex := StrToInt(GetNextItem);
        if Components[i] is TRzComboBox then
          TRzComboBox(Components[i]).Text := GetNextItem;
        // The combo box for interviewer G11 and program G3 are saved as IENs
      end;
  except
    on E: Exception do
      begin
        showmessage(e.message);
        exit;
      end;
  end;

  CloseFile(f);

  RecoverComments;

  if (TempRecordIENS = '') or (TempRecordIENS = ' ') then
    TempRecordIENS := NO_ID;

  ASI_Save_508.SaveASIData(
    TempRecordIENS,
    TempPt,
    TempInterviewer,
    TempOrderer,
    DatetoStr(date),
    ProgramIEN,
    TempInterviewType);

  CleanupTxtFiles;
end;


//-------------------------------------------------------------------
//           Menu Operations
//-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.mnuContentsClick(Sender: TObject);
begin
  //Show contents page of help file
  Application.HelpContext(2);
end;

procedure TfrmMHA_ASI_Entry_508.mnuAboutClick(Sender: TObject);
begin
  frmMHA_About := TfrmMHA_About.Create(nil);
  try
    frmMHA_About.ShowModal;
  finally
    frmMHA_About.Free;
  end; //try
end;


procedure TfrmMHA_ASI_Entry_508.mnuCloseClick(Sender: TObject);
begin
  FixinToClose(sender);
end;

procedure TfrmMHA_ASI_Entry_508.ASITimedOut;
begin
  FASITimedOut := true;
  ASI_Save_508.SaveASIData(
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS,
    Patient.DFN,
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerDUZ,
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByDUZ,
    DateToStr(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).ASIDate),
    ProgramIEN,
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType); // uMHA3_ASI_Save
  Close;
end;


//-------------------------------------------------------------------
//       Key operations: Key Press, Down
//-------------------------------------------------------------------

procedure TfrmMHA_ASI_Entry_508.RzEditOnExit(Sender: TObject);
begin
  if lvInitialize then
    exit;
  //if not lvInitialize then
  ASI_Save_508.SaveASIOnTheRun; // uMHA_ASI_Save

  if Sender is TRzEdit then
    with Sender as TRzEdit do
      ResponseList := ResponseList + UpperCase(Name) + ' ';
end;

procedure TfrmMHA_ASI_Entry_508.EditOnExit(Sender: TObject);
begin
  if lvInitialize then
    exit;
 // if not lvInitialize then
  ASI_Save_508.SaveASIOnTheRun; // uMHA_ASI_Save
  if Sender is TEdit then
    with Sender as TEdit do
      ResponseList := ResponseList + UpperCase(Name) + ' ';
end;

procedure TfrmMHA_ASI_Entry_508.ListBoxOnExit(Sender: TObject);
begin

  ASI_Save_508.SaveASIOnTheRun; // uMHA_ASI_Save
  if Sender is TlistBox then
    with Sender as TListBox do
      begin
        ResponseList := ResponseList + UpperCase(Name) + ' ';

        if Name = 'rgrpG19' then
          rgrpG19Exit(Sender);

        if Name = 'rgrpM7' then
          if not CrossCheckItems then
            CrossCheck('M7', sender);

        if Name = 'rgrpM8' then
          if not CrossCheckItems then
            CrossCheck('M8', sender);

        if Name = 'rgrpE20' then
          if not CrossCheckItems then
            CrossCheck('E20', sender);

        if Name = 'rgrpD14' then
          if not CrossCheckItems then
            CrossCheck('D14', sender);

        if Name = 'rgrpD30' then
          if not CrossCheckItems then
            begin
              CrossCheck('D30a', sender);
              CrossCheck('D30b', sender);
            end;

        if Name = 'rgrpD31' then
          if not CrossCheckItems then
            begin
              CrossCheck('D31a', sender);
              CrossCheck('D31b', sender);
            end;

        if Name = 'rgrpL24' then
          if not CrossCheckItems then
            CrossCheck('L24', sender);

        if Name = 'rgrpL25' then
          if not CrossCheckItems then
            CrossCheck('L25', sender);

        if Name = 'rgrpF34' then
          if not CrossCheckItems then
            begin
              CrossCheck('F34a', sender);
              CrossCheck('F34b', sender);
            end;

        if Name = 'rgrpF35' then
          if not CrossCheckItems then
            begin
              CrossCheck('F35a', sender);
              CrossCheck('F35b', sender);
            end;

        if Name = 'rgrpP2' then
          if not CrossCheckItems then
            CrossCheck('P2', sender);

        if Name = 'rgrpP9' then
          if not CrossCheckItems then
            CrossCheck('P9a', sender);

        if Name = 'rgrpP9l' then
          if not CrossCheckItems then
            CrossCheck('P9b', sender);

        if Name = 'rgrpP10l' then
          if not CrossCheckItems then
            CrossCheck('P10', sender);

        if Name = 'rgrpP13' then
          if not CrossCheckItems then
            begin
              CrossCheck('P13a', sender);
              CrossCheck('P13b', sender);
            end;

      end;

end;

procedure TfrmMHA_ASI_Entry_508.SpinEditOnExit(Sender: TObject);
begin
  if lvInitialize then
    exit;
 // if not lvInitialize then
  ASI_Save_508.SaveASIOnTheRun; // uMHA_ASI_Save

  if Sender is TMHA_SpnEdt then
    with Sender as TMHA_SpnEdt do
      begin
        ResponseList := ResponseList + UpperCase(Name) + ' ';

        if Name = 'edtG14y' then
          if ((Text = 'N') or (Text = 'X')) then
            edtG14m.Text := Text;

        if Name = 'edtG20' then
          if not CrossCheckItems then
            begin
              CrossCheck('G20', Sender);
              Exit;
            end;

        if Name = 'edtM1' then
          if not CrossCheckItems then
            CrossCheck('M1', sender);

        if Name = 'edtM2y' then
          if ((Text = 'N') or (Text = 'X')) then
            edtM2m.Text := Text;

        if Name = 'edtM2m' then
          if not CrossCheckItems then
            CrossCheck('M2', sender);

        if Name = 'edtE1y' then
          begin
            if ((Text = 'N') or (Text = 'X')) then
              edtE1m.Text := Text;
            if not CrossCheckItems then
              CrossCheck('E1y', sender);
          end;

        if Name = 'edtE6y' then
          if ((Text = 'N') or (Text = 'X')) then
            edtE6m.Text := Text;

        if Name = 'edtE12' then
          if not CrossCheckItems then
            begin
              CrossCheck('E12a', sender);
              CrossCheck('E12b', sender);
            end;

        if Name = 'edtE15' then
          if not CrossCheckItems then
            CrossCheck('E15', sender);

        if Name = 'edtD2d' then
          if not CrossCheckItems then
            CrossCheck('D2d', sender);

        if Name = 'edtD2y' then
          if not CrossCheckItems then
            CrossCheck('D2y', sender);

        if Name = 'edtD16' then
          if not CrossCheckItems then
            CrossCheck('D16', sender);

        if Name = 'edtD17' then
          if not CrossCheckItems then
            CrossCheck('D17', sender);

        if Name = 'edtD21' then
          if not CrossCheckItems then
            begin
              CrossCheck('D21a', sender);
              CrossCheck('D21b', sender);
            end;

        if Name = 'edtD22' then
          if not CrossCheckItems then
            begin
              CrossCheck('D22a', sender);
              CrossCheck('D22b', sender);
            end;

        if Name = 'edtD23' then
          if not CrossCheckItems then
            begin
              CrossCheck('D23a', sender);
              CrossCheck('D23b', sender);
            end;

        if Name = 'edtD24' then
          if not CrossCheckItems then
            begin
              CrossCheck('D24a', sender);
              CrossCheck('D24b', sender);
            end;

        if Name = 'edtL17' then
          if not CrossCheckItems then
            CrossCheck('L17', sender);

        if Name = 'edtL20' then
          if not CrossCheckItems then
            CrossCheck('L20', sender);

        if Name = 'edtL22' then
          edtL22Exit(Sender);

        if Name = 'edtL26' then
          if not CrossCheckItems then
            CrossCheck('L26', sender);

        if Name = 'edtL27' then
          if not CrossCheckItems then
            CrossCheck('L27', sender);

        if Name = 'edtF2y' then
          if ((Text = 'N') or (Text = 'X')) then
            edtF2m.Text := Text;

        if Name = 'edtF5y' then
          if ((Text = 'N') or (Text = 'X')) then
            edtF5m.Text := Text;

        if Name = 'edtP11' then
          if not CrossCheckItems then
            begin
              CrossCheck('P11a', sender);
              CrossCheck('P11b', sender);
            end;
      end;
end;

procedure TfrmMHA_ASI_Entry_508.RzComboBoxOnExit(Sender: TObject);
begin
  if lvInitialize then
    exit;
  //if not lvInitialize then
  ASI_Save_508.SaveASIOnTheRun; // uMHA_ASI_Save

  if Sender is TRzComboBox then
    with Sender as TRzComboBox do
      ResponseList := ResponseList + UpperCase(Name) + ' ';
end;

procedure TfrmMHA_ASI_Entry_508.ORComboBoxOnExit(Sender: TObject);
begin
  if lvInitialize then
    exit;
 // if not lvInitialize then
  ASI_Save_508.SaveASIOnTheRun; // uMHA_ASI_Save
  if Sender is TORComboBox then
    with Sender as TORComboBox do
      begin
        ResponseList := ResponseList + UpperCase(Name) + ' ';

        if Name = 'cbxG11Interviewerx' then
          begin
            if ItemIndex <> -1 then
              begin
                TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerDUZ :=
                  Piece(Items[ItemIndex], U, 1);
                TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerName :=
                  Piece(Items[ItemIndex], U, 2);
              end;
          end;

        if Name = 'cbxG11OrderedByx' then
          begin
            if ItemIndex <> -1 then
              begin
                TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByDUZ :=
                  Piece(Items[ItemIndex], U, 1);
                TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByName :=
                  Piece(Items[ItemIndex], U, 2);
                TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).CanBeSignedByUser := (ThisUser.DUZ = ASI.OrderedByDUZ);
              end;
          end;
      end;

end;

procedure TfrmMHA_ASI_Entry_508.DateOnExit(Sender: TObject);
begin
  if lvInitialize then
    exit;
 // if not lvInitialize then
  ASI_Save_508.SaveASIOnTheRun; // uMHA_ASI_Save

  if Sender is TDateTimePicker then
    with Sender as TDateTimePicker do
      begin
        ResponseList := ResponseList + UpperCase(Name) + ' ';
        (*
        if Name = 'dtpG4' then
          begin
            if not CheckDeathDate(DateTime) then
              ActiveControl := GroupBox2;
          end;

        if Name = 'dtpG5' then
          begin
            if not CheckDeathDate(DateTime) then
              ActiveControl := GroupBox3;
          end;
         *)
      end;
end;

procedure TfrmMHA_ASI_Entry_508.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if ActiveControl.ClassName = 'TFMMemo' then
    exit;

  // Note: use this here to avoid a beep
  if Key = #13 then {- Go to next field -}
    begin
      Perform(WM_NEXTDLGCTL, 0, 0);
      Key := #0; {- requires "Message" in "uses" -}
      Exit;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ((Shift = [ssALT]) and (Key = vk_F4)) or
    (Key = vk_Escape) then
    begin
      Key := $00;
      FixinToClose(sender);
      Exit;
    end;
end;


//-------------------------------------------------------------------
//      Memo field comments: Saved to file for recovery purposes
//-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.RecoverComments;

begin
  if FileExists(CrashFilePath + 'ASI_General.txt') then
    memGeneral.Lines.LoadFromFile(CrashFilePath + 'ASI_General.txt');
  if FileExists(CrashFilePath + 'ASI_Medical.txt') then
    memMedical.Lines.LoadFromFile(CrashFilePath + 'ASI_Medical.txt');
  if FileExists(CrashFilePath + 'ASI_Employ.txt') then
    memEmploy.Lines.LoadFromFile(CrashFilePath + 'ASI_Employ.txt');
  if FileExists(CrashFilePath + 'ASI_Drug.txt') then
    memDrug.Lines.LoadFromFile(CrashFilePath + 'ASI_Drug.txt');
  if FileExists(CrashFilePath + 'ASI_Legal.txt') then
    memLegal.Lines.LoadFromFile(CrashFilePath + 'ASI_Legal.txt');
  if FileExists(CrashFilePath + 'ASI_Family.txt') then
    memFamily.Lines.LoadFromFile(CrashFilePath + 'ASI_Family.txt');
  if FileExists(CrashFilePath + 'ASI_Social.txt') then
    memSocial.Lines.LoadFromFile(CrashFilePath + 'ASI_Social.txt');
  if FileExists(CrashFilePath + 'ASI_Psych.txt') then
    memPsych.Lines.LoadFromFile(CrashFilePath + 'ASI_Psych.txt');
  if FileExists(CrashFilePath + 'ASI_Spirit.txt') then
    memSpirit.Lines.LoadFromFile(CrashFilePath + 'ASI_Spirit.txt');
  if FileExists(CrashFilePath + 'ASI_Leisure.txt') then
    memLeisure.Lines.LoadFromFile(CrashFilePath + 'ASI_Leisure.txt');

end;

procedure TfrmMHA_ASI_Entry_508.MemLeisureExit(Sender: TObject);
begin
  MemLeisure.Lines.SaveToFile(CrashFilePath + 'ASI_Leisure.txt');

end;

procedure TfrmMHA_ASI_Entry_508.MemSpiritExit(Sender: TObject);
begin
  MemSpirit.Lines.SaveToFile(CrashFilePath + 'ASI_Spirit.txt');
  pgctrlASI.ActivePage := TabLeisure;
  ActiveControl := GroupBox25;
end;

procedure TfrmMHA_ASI_Entry_508.MemPsychExit(Sender: TObject);
begin
  MemPsych.Lines.SaveToFile(CrashFilePath + 'ASI_Psych.txt');
  pgctrlASI.ActivePage := TabSpirit;
  ActiveControl := GroupBox24;
end;

procedure TfrmMHA_ASI_Entry_508.MemSocialExit(Sender: TObject);
begin
  MemSocial.Lines.SaveToFile(CrashFilePath + 'ASI_Social.txt');
  pgctrlASI.ActivePage := TabPsych;
  ActiveControl := gbP1a;
end;

procedure TfrmMHA_ASI_Entry_508.MemFamilyExit(Sender: TObject);
begin
  MemFamily.Lines.SaveToFile(CrashFilePath + 'ASI_Family.txt');
  pgctrlASI.ActivePage := TabSocial;
  ActiveControl := gbF1;
end;

procedure TfrmMHA_ASI_Entry_508.MemLegalExit(Sender: TObject);
begin
  MemLegal.Lines.SaveToFile(CrashFilePath + 'ASI_Legal.txt');
  if gvASI_Type = 1 then // Full ASI
    begin
      pgctrlASI.ActivePage := TabFamily;
      ActiveControl := gbH1a;
    end
  else
    begin // Followup or Lite
      pgctrlASI.ActivePage := TabSocial;
      ActiveControl := gbF1;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.MemDrugExit(Sender: TObject);
begin
  MemDrug.Lines.SaveToFile(CrashFilePath + 'ASI_Drug.txt');
  pgctrlASI.ActivePage := TabLegal;
  if gvASI_Type = 3 then
    ActiveControl := gbL2
  else
    ActiveControl := gbL1;
end;

procedure TfrmMHA_ASI_Entry_508.MemEmployExit(Sender: TObject);
begin
  MemEmploy.Lines.SaveToFile(CrashFilePath + 'ASI_Employ.txt');
  pgctrlASI.ActivePage := TabDrug;
  ActiveControl := gbD1d;
end;

procedure TfrmMHA_ASI_Entry_508.MemMedicalExit(Sender: TObject);
begin
  MemMedical.Lines.SaveToFile(CrashFilePath + 'ASI_Medical.txt');
  pgctrlASI.ActivePage := TabEmploy;
  ActiveControl := gbE1y;
end;

procedure TfrmMHA_ASI_Entry_508.memGeneralExit(Sender: TObject);
begin
  MemGeneral.Lines.SaveToFile(CrashFilePath + 'ASI_General.txt');
  pgctrlASI.ActivePage := TabMedical;
  ActiveControl := gbM1;
end;


//-------------------------------------------------------------------
//            Verify, Save data, Sign it
//-------------------------------------------------------------------


function TfrmMHA_ASI_Entry_508.SkippedItems: string;
const
  RequiredComboBoxs           : string =
  // General
  'cbxG3,cbxG11Interviewerx,cbxG11OrderedByx,' +
    // Drug
  'cbxD1r,cbxD2r,cbxD3r,cbxD4r,cbxD5r,cbxD6r,cbxD7r,cbxD8r,cbxD9r,cbxD10r,cbxD11r,cbxD12r';
  RequiredRadioGroups         : string =
  // General
  'rgrpG9,rgrpG12,rgrpG19,' +
    // Medical
  'rgrpM3,rgrpM4,rgrpM5,rgrpM7,rgrpM8,rgrpM10,rgrpM11,' +
    // Employment
  'rgrpE4,rgrpE5,rgrpE7,rgrpE9,rgrpE10,rgrpE21,rgrpE23,rgrpE24,' +
    // Drug
  'rgrpD28,rgrpD29,rgrpD30,rgrpD31,' +
    // Legal
  'rgrpL1,rgrpL2,rgrpL24,rgrpL28,rgrpL29,rgrpL31,rgrpL32,' +
    // Social
  'rgrpF1,rgrpF3,rgrpF4,rgrpF6,rgrpF7,rgrpF8,rgrpF18,rgrpF18l,rgrpF19,rgrpF19l,rgrpF20,rgrpF20l,rgrpF21,rgrpF21l,' +
    'rgrpF22,rgrpF22l,rgrpF23,rgrpF23l,rgrpF24,rgrpF24l,rgrpF25,rgrpF25l,rgrpF26,rgrpF26l,rgrpF28,rgrpF28l,' +
    'rgrpF29,rgrpF29l,rgrpF32,rgrpF34,rgrpF37,rgrpF38,' +
    // Psych
  'rgrpP2,rgrpP3,rgrpP3l,rgrpP4,rgrpP4l,rgrpP5,rgrpP5l,rgrpP6,rgrpP6l,rgrpP7,rgrpP7l,' +
    'rgrpP8,rgrpP8l,rgrpP9,rgrpP9l,rgrpP10,rgrpP10l,edtP11,rgrpP12,rgrpP13,rgrpP21,rgrpP22,';
  RequiredEdits               : string =
  // General
  'edtG14y,edtG14m,edtG20,' +
    // Medical
  'edtM1,edtM6,' +
    // Employment
  'edtE1y,edtE1m,edtE2,edtE6y,edtE6m,edtE11,edtE12,' +
    'edtE13,edtE14,edtE15,edtE16,edtE17,edtE18,' +
    // Drug
  'edtD1a,edtD1y,edtD4a,edtD4y,edtD7a,edtD7y,edtD10a,edtD10y,' +
    'edtD2a,edtD2y,edtD5a,edtD5y,edtD8a,edtD8y,edtD11a,edtD11y,' +
    'edtD3a,edtD3y,edtD6a,edtD6y,edtD9a,edtD9y,edtD12a,edtD12y,' +
    'edtD13a,edtD13y,edtD19,edtD20,edtD23,edtD24,edtD25,edtD26,edtD27,' +
    // Legal
  'edtL3,edtL4,edtL5,edtL6,edtL7,edtL8,edtL9,edtL10,edtL11,edtL12,edtL13,edtL14,' +
    'edtL15,edtL16,edtL17,edtL18,edtL19,edtL20,edtL21,edtL26,edtL27,' +
    // Social
  'edtF30,edtF31,' +
    // Psych
  'edtP1a,edtP1b,edtP11';

var
  TempStr,
    BlankItem                 : string;
  i, j, k                     : integer;
  GotOne                      : Boolean;

  function RequiredComboBox(Name: string): Boolean;
  begin
    result := false;
    if Pos(Name, RequiredComboBoxs) > 0 then
      result := true;
  end;

  function RequiredRadioGroup(Name: string): Boolean;
  begin
    result := false;
    if Pos(Name, RequiredRadioGroups) > 0 then
      result := true;
  end;

  function RequiredEdit(Name: string): Boolean;
  begin
    result := false;
    if Pos(Name, RequiredEdits) > 0 then
      result := true;
  end;

  procedure MoveToPage(P: Char);
  begin
    case P of
      'G':
        begin
          pgctrlASI.ActivePage := TabGeneral;
        end;
      'M':
        begin
          pgctrlASI.ActivePage := TabMedical;
        end;
      'E':
        begin
          pgctrlASI.ActivePage := TabEmploy;
        end;
      'D':
        begin
          pgctrlASI.ActivePage := TabDrug;
        end;
      'L':
        begin
          pgctrlASI.ActivePage := TabLegal;
        end;
      'F':
        begin
          pgctrlASI.ActivePage := TabSocial;
        end;
      'P':
        begin
          pgctrlASI.ActivePage := TabPsych;
        end;
    end;
  end;

begin
  Result := '';
  if not CheckSkipItems then // Don't continue
    Exit;
  BlankItem := '';
  GotOne := False; // used to place user at required item

  for i := 0 to ComponentCount - 1 do
    begin
      TempStr := '';
      k := 0;
      if Components[i] is TRzComboBox then
        if ((TRzComboBox(Components[i]).Visible) and
          (TRzComboBox(Components[i]).Text = '') and
          (RequiredComboBox(TRzComboBox(Components[i]).Name))) then
          begin
            TempStr := TRzComboBox(Components[i]).Name;
            if TempStr[Length(TempStr)] in ['a'..'z', 'A'..'Z'] then
              k := 1;
            for j := 4 to Length(TempStr) - k do
              BlankItem := BlankItem + TempStr[j];
            BlankItem := BlankItem + ' ';
            if not GotOne then
              begin
                MoveToPage(TempStr[5]);
                //ActiveControl := TRzComboBox(Components[i]);
                GotOne := True;
              end;
          end;
      if Components[i] is TListBox then
        if ((TListBox(Components[i]).Visible) and
          (TListBox(Components[i]).ItemIndex = -1) and
          (RequiredRadioGroup(TListBox(Components[i]).Name))) then
          begin
            TempStr := TListBox(Components[i]).Name;
            if TempStr[Length(TempStr)] in ['a'..'z', 'A'..'Z'] then
              k := 1;
            for j := 5 to Length(TempStr) - k do
              BlankItem := BlankItem + TempStr[j];
            BlankItem := BlankItem + ' ';
            if not GotOne then
              begin
                MoveToPage(TempStr[6]);
                // ActiveControl := TListBox(Components[i]);
                GotOne := True;
              end;
          end;
      if Components[i] is TEdit then
        if ((TEdit(Components[i]).Visible) and
          (TEdit(Components[i]).Text = '') and
          (RequiredEdit(TEdit(Components[i]).Name))) then
          begin
            if TEdit(Components[i]).Name = 'edtG19e' then
              if rgrpG19.ItemIndex <> 5 then
                exit;
            if TEdit(Components[i]).Name = 'edtM3e' then
              if rgrpM3.ItemIndex <> 1 then
                exit;
            if TEdit(Components[i]).Name = 'edtM5e' then
              if rgrpM5.ItemIndex <> 1 then
                exit;

            TempStr := TEdit(Components[i]).Name;
            if TempStr[Length(TempStr)] in ['a'..'z', 'A'..'Z'] then
              k := 1;
            for j := 4 to Length(TempStr) - k do
              BlankItem := BlankItem + TempStr[j];
            BlankItem := BlankItem + ' ';
            if not GotOne then
              begin
                MoveToPage(TempStr[5]);
                //ActiveControl := TEdit(Components[i]);
                GotOne := True;
              end;
          end;
    end;

  if Length(BlankItem) <> 0 then
    begin
      pnlVerify.Height := 39;
      pnlVerify.BevelWidth := 1;
      pnlVerify.BorderStyle := bsNone;
      lblVerify.Caption := 'Required items not answered: ' + BlankItem;
      lblVerify.Color := clRed;
      lblVerify.Font.Color := clWhite;
    end;
  Result := BlankItem;
end;

function TfrmMHA_ASI_Entry_508.VerifyData(sender: TObject): boolean;
begin
  Result := false;
  FUserTerminates := false;
  CheckSkipItems := True;

  // "SkippedItems" is the above function that lists the required items
  // that have not been completed. An empty list means all the items are
  // done. Consistency checks are done at the item level when the user
  // is working on that item.
  if Length(SkippedItems) <> 0 then
    Exit;

  lblVerify.caption := '';
  pnlVerify.Height := 0;
  CheckSkipItems := False;

  Result := True;
end;

//--------------------------------------------------------------------------
//       Range Checks for Spin Edit Items
//--------------------------------------------------------------------------


function TfrmMHA_ASI_Entry_508.CheckRange(StringIn: string; Max, Min: integer):
  Boolean;
var
  n                           : integer;
begin
  Result := True;
  // Allow "X" and "N" to be okay
  if Length(StringIn) = 1 then
    if ((UpperCase(StringIn) = 'X') or (UpperCase(StringIn) = 'N')) then
      Exit;

  // Allow a blank to slip thru
  if Length(StringIn) = 0 then
    Exit;

  Result := False;

  try
    n := StrToInt(StringIn);
  except
    on E: Exception do
      begin
        MsgDlgFrm('This number does not work. ' + e.message,
          MHA_Caption, mtInformation, [mbOK], 0);
        Exit;
      end;
  end; {- try-}

  // Range checks
  if n > Max then
    begin
      MsgDlgFrm('This number is too large. The maximum is ' + IntToStr(Max) +
        '.',
        MHA_Caption, mtInformation, [mbOK], 0);
      Exit;
    end;
  if n < Min then
    begin
      MsgDlgFrm('This number is too small. The minimum is ' + IntToStr(Min) +
        '.',
        MHA_Caption, mtInformation, [mbOK], 0);

      Exit;
    end;

  Result := True;
end;


procedure TfrmMHA_ASI_Entry_508.Check9999(Sender: TObject);
var
  i                           : Integer;
begin
  if lvInitialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TMHA_SpnEdt then
        if TMHA_SpnEdt(Components[i]).Name = ActiveControl.Name then
          begin
            if not CheckRange(TMHA_SpnEdt(Components[i]).text, 9999, 0) then
              TMHA_SpnEdt(Components[i]).Text := '0';
            Break;
          end;
    end;
end;


procedure TfrmMHA_ASI_Entry_508.CheckLife(Sender: TObject);
var
  i                           : Integer;
begin
  if lvInitialize then
    Exit;
  for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TMHA_SpnEdt then
        if TMHA_SpnEdt(Components[i]).Name = ActiveControl.Name then
          begin
            if not CheckRange(TMHA_SpnEdt(Components[i]).text, Patient.Age, 0) then
              TMHA_SpnEdt(Components[i]).Text := '0';
            Break;
          end;
    end;
end;


//--------------------------------------------------------------------------
//              Spin-Edit Items: Exit
//--------------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.CheckOutL17(Sender: TObject);
var
  i                           : Integer;
begin
  for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TEdit then
        if TEdit(Components[i]).Name = ActiveControl.Name then
          begin
            CheckRange(TEdit(Components[i]).text, 99, 0);
            Break;
          end;
    end;

  if ((edtL3.text = '0') and (edtL4.text = '0') and
    (edtL5.text = '0') and (edtL6.text = '0') and
    (edtL7.text = '0') and (edtL8.text = '0') and
    (edtL9.text = '0') and (edtL10.text = '0') and
    (edtL11.text = '0') and (edtL12.text = '0') and
    (edtL13.text = '0') and (edtL14.text = '0') and
    (edtL15.text = '0') and (edtL16.text = '0')) then
    begin
      edtL17.text := 'N';
      edtL17.Enabled := False;
      gbL17.Enabled := false;
    end
  else
    begin
      edtL17.Enabled := true;
      gbL17.Enabled := true;
    end;

end;

procedure TfrmMHA_ASI_Entry_508.edtL22Exit(Sender: TObject);
begin
  if ((edtL21.text = 'N') or (edtL21.text = 'X') or
    (edtL21.text = '') or
    (edtL22.text = 'N') or (edtL22.text = 'X') or
    (edtL22.text = '')) then // do nothing
  else
    begin
      if StrToInt(edtL21.text) < StrToInt(edtL22.text) then
        begin
          MsgDlgFrm('The months of lifelong incarceration is less than the ' +
            'months of the last incarceration. Please correct.',
            MHA_Caption, mtInformation, [mbOK], 0);
          ActiveControl := gbL22;
        end;
    end;
end;


//-------------------------------------------------------------------
//         Radio Group: Click Controls
//-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.rgrpG19Click(Sender: TObject);
begin
  if rgrpG19.ItemIndex = 5 then
    begin
      edtG19e.Enabled := True;
      GroupBox14.Enabled := true;
    end
  else
    begin
      edtG19e.Text := '';
      edtG19e.Enabled := false;
      GroupBox14.Enabled := false;
    end;
  if ((rgrpG19.ItemIndex = 0) or (rgrpG19.ItemIndex = 6 {X}) or
    (rgrpG19.ItemIndex = 7 {N})) then
    begin
      edtG20.Text := 'N';
      edtG20.Enabled := false;
      edtG19e.Text := '';
      edtG19e.Enabled := False;
      GroupBox14.Enabled := false;
      GroupBox15.Enabled := false;
    end;
  if ((rgrpG19.ItemIndex = 1) or (rgrpG19.ItemIndex = 2) or
    (rgrpG19.ItemIndex = 3) or (rgrpG19.ItemIndex = 4) or
    (rgrpG19.ItemIndex = 5)) then
    begin
      edtG20.Enabled := True;
      edtG20.Text := '0';
      //GroupBox14.Enabled := false;
      //edtG19e.Enabled := False;
      GroupBox15.Enabled := true;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.rgrpG12Click(Sender: TObject);
begin
(*
  if lvInitialize then
    exit;
  if rgrpG12.ItemIndex <> 3 then
    begin
      MsgDlgFrm('The many default answers provided on this form will not be honored when ' +
        'an A S I is only partially completed. Only the items that you actually answer will ' +
        'be saved. ',
        MHA_Caption, mtInformation, [mbYes], 0);
    end;
   *)
end;

procedure TfrmMHA_ASI_Entry_508.rgrpM3Click(Sender: TObject);
begin
  if rgrpM3.ItemIndex = 1 then
    begin
      edtM3e.Enabled := True;
      gbM3S.Enabled := true;
    end
  else
    begin
      edtM3e.Text := '';
      edtM3e.Enabled := false;
      gbM3S.Enabled := false;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.rgrpM5Click(Sender: TObject);
begin
  if rgrpM5.ItemIndex = 1 then
    begin
      edtM5e.Enabled := True;
      gbM5S.Enabled := true;
    end
  else
    begin
      edtM5e.Text := '';
      edtM5e.Enabled := false;
      gbM5S.Enabled := false;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.rgrpE3Click(Sender: TObject);
begin
  if rgrpE3.ItemIndex = 1 then
    begin
      edtE3e.Enabled := True;
      gbE3s.Enabled := true;
    end
  else
    begin
      edtE3e.Text := '';
      edtE3e.Enabled := false;
      gbE3s.Enabled := false;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.rgrpE4Click(Sender: TObject);
begin
  if lvInitialize then
    exit;
  if rgrpE4.ItemIndex <> 0 then
    begin
      rgrpE5.Enabled := True;
      gbE5.Enabled := true;
    end;
  if rgrpE4.ItemIndex = 0 then
    begin
      rgrpE5.ItemIndex := 0;
      rgrpE5.Enabled := false;
      gbE5.Enabled := false;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.rgrpE8Click(Sender: TObject);
begin
  if lvInitialize then
    exit;
  if rgrpE8.ItemIndex = 0 then
    begin
      rgrpE9.ItemIndex := 3;
      rgrpE9.Enabled := False;
      gbE9.Enabled := false;
    end
  else
    begin
      rgrpE9.Enabled := True;
      gbE9.Enabled := true;
    end;
end;


procedure TfrmMHA_ASI_Entry_508.rgrpD14Click(Sender: TObject);
begin
  if lvInitialize then
    exit;
  if rgrpD14.ItemIndex = 0 then
    begin
      edtD15.Text := 'N';
      edtD15.Enabled := False;
      gbD15.Enabled := False;
      edtD16.Text := 'N';
      edtD16.Enabled := False;
      gbD16.Enabled := False;
    end
  else
    begin
      edtD15.Enabled := True;
      gbD15.Enabled := true;
      if edtD15.text = 'N' then
        edtD15.Text := '0';
      edtD16.Enabled := True;
      gbD16.Enabled := true;
      if edtD16.text = 'N' then
        edtD16.Text := '0';
    end;
end;

procedure TfrmMHA_ASI_Entry_508.rgrpL24Click(Sender: TObject);
begin
  if lvInitialize then
    exit;
  if rgrpL24.ItemIndex = 1 then
    begin
      if not rgrpL25.Enabled then
        begin
          gbL25.Enabled := true;
          rgrpL25.Enabled := True;
        end;
    end
  else
    begin
      rgrpL25.ItemIndex := 18;
      if rgrpL25.Enabled then
        begin
          gbL25.Enabled := false;
          rgrpL25.Enabled := False;
        end;
    end;
end;


//-------------------------------------------------------------------
//               Page Control
//-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.pgctrlASIChange(Sender: TObject);
begin
  if pgctrlASI.ActivePage = TabGeneral then
    begin
      if not CrossCheckItems then
        ActiveControl := groupbox1;
    end;
  if pgctrlASI.ActivePage = TabMedical then
    begin
      if not lvInitialize then
        if not CrossCheckItems then
          ActiveControl := gbM1;
    end;
  if pgctrlASI.ActivePage = TabEmploy then
    begin
      if not lvInitialize then
        if not CrossCheckItems then
          ActiveControl := gbE1y;
    end;
  if pgctrlASI.ActivePage = TabDrug then
    begin
      sbDrug1.VertScrollBar.Position := 0;
      if not CrossCheckItems then
        ActiveControl := gbD1d;
    end;
  if pgctrlASI.ActivePage = TabLegal then
    begin
      if CrossCheckItems then
        exit;
      if gvASI_Type <> 3 then
        begin
          ActiveControl := gbL1;
        end
      else
        begin
          ActiveControl := gbL2;
        end;
    end;
  if pgctrlASI.ActivePage = TabFamily then
    begin
      if gvASI_Type = 1 then
        ActiveControl := gbH1a;
    end;
  if pgctrlASI.ActivePage = TabSocial then
    begin
      if not CrossCheckItems then
        begin
          ActiveControl := gbF1;
        end;
    end;
  if pgctrlASI.ActivePage = TabPsych then
    begin
      if not lvInitialize then
        if not CrossCheckItems then
          ActiveControl := gbP1a;
    end;
  if pgctrlASI.ActivePage = TabSpirit then
    begin
      ActiveControl := groupbox24;
    end;
  if pgctrlASI.ActivePage = TabLeisure then
    begin
      ActiveControl := groupbox25;
    end;
end;


procedure TfrmMHA_ASI_Entry_508.sbSocialEnter(Sender: TObject);
begin
  if ((edtE11.text = '0') or (edtE11.text = '00') or
    (rgrpE10.ItemIndex = 6)) then
    begin
      rgrpF26.ItemIndex := 3;
      //rgrpF26.Color := clBtnFace;
      rgrpF26.Enabled := false;
      gbF26.Enabled := false;
    end;
end;

//-------------------------------------------------------------------
//  These are dummy items that are used to move to next tab when the
//  user moves from one item to the next.
//-------------------------------------------------------------------

procedure TfrmMHA_ASI_Entry_508.SpiritNextPageEnter(Sender: TObject);
begin

end;


//-------------------------------------------------------------------
//  Comment Fields: Save the size of the comment memo field
//-------------------------------------------------------------------

(*
procedure TfrmMHA_ASI_Entry_508.spltGeneralExit(Sender: TObject);
begin
  gvASIGeneral := spltGeneral.percent;
end;

procedure TfrmMHA_ASI_Entry_508.spltDrugExit(Sender: TObject);
begin
  gvASIDrug := spltDrug.percent;
end;

procedure TfrmMHA_ASI_Entry_508.spltSocialExit(Sender: TObject);
begin
  gvASISocial := spltSocial.percent;
end;

procedure TfrmMHA_ASI_Entry_508.spltPsychExit(Sender: TObject);
begin
  gvASIPsych := spltPsych.percent;
end;

procedure TfrmMHA_ASI_Entry_508.splitMedicalExit(Sender: TObject);
begin
  gvASIMedical := spltMedical.percent;
end;

procedure TfrmMHA_ASI_Entry_508.spltLegalExit(Sender: TObject);
begin
  gvASILegal := spltLegal.percent;
end;

procedure TfrmMHA_ASI_Entry_508.spltFamilyExit(Sender: TObject);
begin
  gvASIFamily := spltFamily.percent;
end;

procedure TfrmMHA_ASI_Entry_508.spltEmployExit(Sender: TObject);
begin
  gvASIEmploy := spltEmploy.percent;
end;  *)


//-------------------------------------------------------------------
//             Spin-Edit Fields: Change
//-------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.edtM1Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  if CheckRange(edtM1.text, 99, 0) then
  else
    Exit;
  if gvASI_Type = 1 then // Full
    begin
      if ((edtM1.Text = '0') or (edtM1.Text = '00')) then
        edtM1.Text := 'N';
      if ((edtM1.Text = 'N') or (edtM1.Text = 'X')) then
        begin
          edtM2m.Text := edtM1.Text;
          edtM2y.Text := edtM1.Text;
          edtM2y.Enabled := false;
          gbM2.Enabled := false;
          edtM2m.Enabled := False;
          gbM2a.Enabled := false;
          ActiveControl := gbM3;
        end
      else
        begin
          edtM2y.Enabled := true;
          gbM2.Enabled := true;
          edtM2m.Enabled := True;
          gbM2a.Enabled := true;
          edtM2y.text := '';
          edtM2m.text := '';
        end;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.edtD15Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  CheckRange(edtD15.text, 99, 0);

  if ((edtD15.Text = '0') or (edtD15.text = '00')) then
    begin
      edtD16.text := 'N';
      edtD16.Enabled := False;
      gbD16.Enabled := false;
    end
  else
    begin
      edtD16.Enabled := True;
      gbD16.Enabled := true;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.edtD20Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  CheckRange(edtD20.text, 99, 0);
  if ((edtD20.text = '00') or (edtD20.text = '0')) then
    begin
      edtD22.text := 'N';
      edtD22.Enabled := False;
      gbD22.Enabled := false;
    end
  else
    begin
      edtD22.Enabled := True;
      gbD22.Enabled := true;
    end
end;

procedure TfrmMHA_ASI_Entry_508.edtD19Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  CheckRange(edtD19.text, 99, 0);
  if ((edtD19.text = '00') or (edtD19.text = '0')) then
    begin
      edtD21.text := 'N';
      edtD21.Enabled := False;
      gbD21.Enabled := false;
    end
  else
    begin
      edtD21.Enabled := True;
      gbD21.Enabled := true;
    end
end;

procedure TfrmMHA_ASI_Entry_508.edtD27Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  CheckRange(edtD27.text, 30, 0);
  if ((edtD27.text = '0') or (edtD27.text = '00')) then
    begin
      rgrpD29.ItemIndex := 0;
      rgrpD29.Enabled := false;
      gbD29.Enabled := false;
      rgrpD31.ItemIndex := 0;
      rgrpD31.Enabled := false;
      gbD31.Enabled := false;
      ActiveControl := gbD27;
    end
  else
    begin
      rgrpD29.Enabled := true;
      gbD29.Enabled := true;
      rgrpD31.Enabled := true;
      gbD31.Enabled := true;
      rgrpD29.ItemIndex := -1;
      rgrpD31.ItemIndex := -1;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.edtD26Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  CheckRange(edtD26.text, 30, 0);
  if ((edtD26.text = '0') or (edtD26.text = '00')) then
    begin
      rgrpD28.ItemIndex := 0;
      rgrpD28.Enabled := false;
      gbD28.Enabled := false;
      rgrpD30.ItemIndex := 0;
      rgrpD30.Enabled := false;
      gbD30.Enabled := false;
      ActiveControl := gbD26;
    end
  else
    begin
      rgrpD28.Enabled := true;
      gbD28.Enabled := true;
      rgrpD30.Enabled := true;
      gbD30.Enabled := true;
      rgrpD28.ItemIndex := -1;
      rgrpD30.ItemIndex := -1;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.edtF11Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  CheckRange(edtF11.text, 9, 0);
  if ((edtF11.text = '0') or (edtF11.text = '00')) then
    begin
      lvInitialize := true;
      rgrpF24.itemindex := 3;
      lvInitialize := false;
      rgrpF24.Enabled := false;
      gbF24.Enabled := false;
    end
  else
    begin
      rgrpF24.Enabled := True;
      gbF24.Enabled := true;
    end;
end;

procedure TfrmMHA_ASI_Entry_508.edtL21Change(Sender: TObject);
begin
  if lvInitialize then
    exit;
  CheckRange(edtL21.text, 99, 0);
  if ((edtL21.text = '0') or (edtL21.text = '00')) then
    begin
      if gvASI_Type <> 2 then
        begin
          edtL22.text := 'N';
          edtL22.Enabled := False;
          gbL22.Enabled := False;
          rgrpL23.ItemIndex := 18; // 'Not applicable'
          rgrpL23.Enabled := false;
          gbL23.Enabled := false;
          ActiveControl := gbL21;
        end;
    end
  else
    begin
      if gvASI_Type <> 2 then
        begin
          edtL22.Enabled := True;
          gbL22.Enabled := true;
          rgrpL23.Enabled := True;
          gbL23.Enabled := true;
        end;
    end;
end;



//---------------------------------------------------------------------
//     Combo Box Items
//--------------------------------------------------------------------



procedure TfrmMHA_ASI_Entry_508.cbxG11InterviewerxNeedData(Sender: TObject; const
  StartFrom: string;
  Direction, InsertAt: Integer);
begin
  inherited;
  (Sender as TORComboBox).ForDataUse(Staff_OPS.LoadStaffSubSet(StartFrom,
    Direction, DataModule1.RPCBroker1));
end;

procedure TfrmMHA_ASI_Entry_508.cbxG11OrderedByxNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;
  (Sender as TORComboBox).ForDataUse(Staff_OPS.LoadStaffSubSet(StartFrom,
    Direction, DataModule1.RPCBroker1));
end;


//---------------------------------------------------------------------
//     Date functions
//--------------------------------------------------------------------


//function TfrmMHA_ASI_Entry_508.CheckDeathDate(DateIn: TDateTime): boolean;
//begin
//  Result := True;
//end;


//---------------------------------------------------------------------
//         Radio Group: Exit
//---------------------------------------------------------------------


procedure TfrmMHA_ASI_Entry_508.rgrpG19Exit(Sender: TObject);
begin
  if ((rgrpG19.ItemIndex = 0) or (rgrpG19.ItemIndex = 6 {X}) or
    (rgrpG19.ItemIndex = 7 {N})) then
    begin
      edtG20.Text := 'N';
      edtG20.Enabled := false;
      GroupBox15.Enabled := false;

      edtG19e.Text := '';
      edtG19e.Enabled := False;
      GroupBox14.Enabled := false;

      //pgctrlASI.ActivePage := TabMedical;
      ActiveControl := groupbox16; // Comment
    end;

  if rgrpG19.ItemIndex = 5 then
    ActiveControl := GroupBox14;

  if ((rgrpG19.ItemIndex = 1) or (rgrpG19.ItemIndex = 2) or
    (rgrpG19.ItemIndex = 3) or (rgrpG19.ItemIndex = 4) or
    (rgrpG19.ItemIndex = 5)) then
    begin
      edtG20.Enabled := True;
      GroupBox15.Enabled := true;
    end;
end;

function TfrmMHA_ASI_Entry_508.ConductCrossChecks(sender: TObject): boolean;
// This will loop thru the items in the CrossCheckList.
// Some may have been checked already while the user inputed data.
begin
  result := false;

  if lvInitialize then
    Exit;

  // Bail out if the list is empty
  if ((CrossCheckItems) and (Length(Trim(CrossCheckList)) = 0)) then
    begin
      result := true;
      CrossCheckItems := false;
      CrossChecksCompleted := True;
      lblVerify.Caption := '';
      pnlVerify.Height := 0;
      // Save, get signature
      //if not SignAndExit then
      //  Exit;
      //Close;
      exit;
    end;

  // Check next item on list (list maybe reduced by prior checks)
  CrossCheck(Piece(CrossCheckList, ',', 1), sender);
end;

procedure TfrmMHA_ASI_Entry_508.CrossCheck(WhichOne: string; sender: TObject);
// There are 48 cross checks. Most users will trip-the-wire as go through
// the ASI, taking each item in order. The exit test for critial items can be defeated
// by non-linear users, so a second test is done before the ASI is saved. Only items
// that have not been reviewed will be tested in the second review.
var
  arrests,
    i                         : integer;
begin
  if WhichOne = '' then
    begin
      CrossCheckItems := false;
      exit;
    end;

  i := Pos(WhichOne, CrossCheckList);
  lvInitialize := true;

  if CrossCheckItems then
    begin
      // Make the Verify Panel look like a button.
      with pnlVerify do
        begin
          Height := 60;
          BevelWidth := 3;
          BorderStyle := bsSingle;
        end;
      lblVerify.Color := clMaroon;
      lblVerify.Font.Color := clWhite;
    end
  else
    begin
      lblVerify.Caption := '';
      pnlVerify.Height := 0;
    end;

  // 1. G20
  if (WhichOne = 'G20') and
    ((rgrpG19.ItemIndex = 1) or (rgrpG19.ItemIndex = 2) or
    (rgrpG19.ItemIndex = 3) or (rgrpG19.ItemIndex = 4) or
    (rgrpG19.ItemIndex = 5)) and ((edtG20.Text = '') or
    (edtG20.text = '0') or (edtG20.text = 'X') or (edtG20.text = 'N')) then
    begin
      // remove this item from the list
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'In question G19 you recorded that the veteran has '
        +
        'been in a controlled environment in the past 30 days, thus question ' +
        'G20 should be greater than 0.'
        + #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('In question G19 you recorded that the veteran has ' +
        'been in a controlled environment in the past 30 days, thus question ' +
        'G20 should be greater than 0.',
        MHA_Caption, mtConfirmation, [mbOK], 0);

      pgctrlASI.ActivePage := TabGeneral;
      edtG20.Text := '0';
      ActiveControl := groupbox15;
      lvInitialize := False;
      exit;
    end;

  // 2. M1
  if (WhichOne = 'M1') and
    ((Length(edtM1.text) > 0) and (edtM1.text[1] in ['0', 'X', 'N'])) and
    (rgrpG19.ItemIndex = 3) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded in the general information section in G19, that ' +
        'the veteran had been hospitialized for medical problems in the past ' +
        '30 days. This hospitialization would usually be coded in question M1. ' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded in the general information section in G19, that '
        +
        'the veteran had been hospitialized for medical problems in the past ' +
        '30 days. This hospitialization would usually be coded in question M1. ' +
        'Do you wish to change M1?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabMedical;
          edtM1.Text := '0';
          ActiveControl := gbM1;
          lvInitialize := False;
          exit;
        end;
    end;

  // 3. M2
  if (WhichOne = 'M2') and (gvASI_Type = 1 { Full}) and
    ((Length(edtM2y.text) > 0) and (edtM2y.text[1] in ['1'..'9'])) and
    ((Length(edtM2m.text) > 0) and (edtM2m.text[1] in ['1'..'9'])) and
    (rgrpG19.ItemIndex = 3) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded in the general information section (G19), that ' +
        'the veteran had been hospitialized this month for medical problems. ' +
        'The correct coding in M2 is usually "0" "0" in this case.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('You recorded in the general information section (G19), that '
        +
        'the veteran had been hospitialized this month for medical problems. ' +
        'The correct coding in M2 is usually "0" "0" in this case. ' +
        'Do you want to recode M2?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabMedical;
          edtM2y.Text := '0';
          edtM2m.Text := '0';
          ActiveControl := gbM2;
          lvInitialize := False;
          exit;
        end;
    end;

  // 4. M7
  if (WhichOne = 'M7') and
    ((length(EdtM6.text) > 0) and (edtM6.text[1] in ['1'..'9'])) and
    (rgrpM7.ItemIndex = 0) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In question M6, you recorded that the veteran experienced ' +
        'some medical problems in the past 30 days. If this were true, then we ' +
        'would expect that the veteran would be at least slightly bothered by these ' +
        'problems.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question M6, you recorded that the veteran experienced '
        +
        'some medical problems in the past 30 days. If this were true, then we ' +
        'would expect that the veteran would be at least slightly bothered by these ' +
        'problems. Do you want to change M7?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabMedical;
          rgrpM7.itemIndex := -1;
          ActiveControl := gbM7;
          lvInitialize := False;
          exit;
        end;
    end;

  // 5. M8
  if (WhichOne = 'M8') and
    ((Length(edtM6.text) > 0) and (edtM6.text[1] in ['0', 'N'])) and
    ((rgrpM7.ItemIndex <> 0) or (rgrpM8.ItemIndex <> 0)) then
    begin
      lblVerify.caption :=
        'In question M6, you recorded that the veteran experienced no ' +
        'medical problems in the past 30 days, since they report being troubled ' +
        'or wanting treatment (M7 or M8), it''s fair to expect that they had some ' +
        'problem days.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('In question M6, you recorded that the veteran experienced no '
        +
        'medical problems in the past 30 days, since they report being troubled ' +
        'or wanting treatment (M7 or M8), it''s fair to expect that they had some ' +
        'problem days. Go back to M6 and identify the number of days the problem ' +
        'has bothered them.',
        MHA_Caption, mtConfirmation, [mbOK],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabMedical;
      edtM6.Text := '0';
      ActiveControl := gbM6;
      lvInitialize := False;
      exit;
    end;

  // 6. E1y
  if (WhichOne = 'E1y') and (gvASI_Type <> 3) {Followup can be zero} and
    ((length(EdtE1y.text) > 0) and (edtE1y.text[1] in ['0'..'9'])) then
    begin
      if (StrToInt(edtE1y.text) < 4) then
        begin
          Delete(CrossCheckList, i, Length(WhichOne) + 1);
          lblVerify.caption :=
            'You are reporting that the veteran has had less than 4 years ' +
            'of education, this is rare. Please review this, did you include home ' +
            'schooling, grade school, etc.?' +
            #13 + #10 + 'Click here to continue cross checks.';
          if
            MsgDlgFrm('You are reporting that the veteran has had less than 4 years '
            +
            'of education, this is rare. Please review this, did you include home ' +
            'schooling, grade school, etc.? Do you want to change E1?',
            MHA_Caption, mtConfirmation, [mbYes, mbNo],
            0) = mrNo then
            begin
              lvInitialize := False;
              if not CrossCheckItems then
                exit;
            end
          else
            begin
              pgctrlASI.ActivePage := TabEmploy;
              edtE1y.Text := '0';
              ActiveControl := gbE1y;
              lvInitialize := False;
              exit;
            end;
        end;
    end;

  // 7. E5 is a forced change. set in code

  // 8. E9 is hard coded

  // 9. E12a
  if (WhichOne = 'E12a') and
    ((Length(edtE11.text) > 0) and (Length(edtE12.text) > 0)) and
    (edtE11.text[1] in ['0', 'N']) and (edtE12.text[1] in ['1'..'9']) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In the last question, E11, you recorded that the veteran ' +
        'was not paid for working at all in the past month. If this is the case, ' +
        'E12 is generally $0.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In the last question, E11, you recorded that the veteran ' +
        'was not paid for working at all in the past month. If this is the case, ' +
        'E12 is generally $0. Do you wish to change E12?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabEmploy;
          edtE12.Text := '0';
          ActiveControl := gbE12;
          lvInitialize := False;
          exit;
        end;
    end;

  // 10. E12b
  if (WhichOne = 'E12b') and
    ((Length(edtE11.text) > 0) and (edtE11.text[1] in ['1'..'9'])) and
    ((Length(edtE12.text) > 0) and (edtE12.text[1] in ['0', 'N'])) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'In the question E11, you recorded that the veteran '
        +
        'was paid for working this month. If this is the case, ' +
        'E12 is generally not $0.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In the question E11, you recorded that the veteran ' +
        'was paid for working this month. If this is the case, ' +
        'E12 is generally not $0. Do you wish to change E12?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabEmploy;
          edtE12.Text := '0';
          ActiveControl := gbE12;
          lvInitialize := False;
          exit;
        end;
    end;

  // 11. E15
  if (WhichOne = 'E15') and
    ((Length(edtE15.text) > 0) and (edtE15.text[1] in ['0', 'N'])) and
    (rgrpM5.ItemIndex = 1) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded in question M5, that the veteran receives a pension ' +
        'for a medical problem. This income is generally recorded in E15 unless ' +
        'they did not receive any cash this month. ' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded in question M5, that the veteran receives a pension '
        +
        'for a medical problem. This income is generally recorded in E15 unless ' +
        'they did not receive any cash this month. Do you wish to change E15?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabEmploy;
          edtE15.Text := '0';
          ActiveControl := gbE15;
          lvInitialize := False;
          exit;
        end;
    end;

  // 12. E20
  if (WhichOne = 'E20') and (gvASI_Type <> 2 {Full & Followup}) and
    ((Length(edtE19.text) > 0) and (edtE19.text[1] in ['1'..'9'])) and
    (rgrpE20.ItemIndex = 0) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In question E19, you recorded that the veteran experienced some employment ' +
        'problems in the past 30 days. If this were true, then we would expect that the ' +
        'veteran would be at least slightly bothered by these problems.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('In question E19, you recorded that the veteran experienced some employment '
        +
        'problems in the past 30 days. If this were true, then we would expect that the ' +
        'veteran would be at least slightly bothered by these problems. ' +
        'Do you wish to change E20?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabEmploy;
          rgrpE20.ItemIndex := -1;
          ActiveControl := gbE20;
          lvInitialize := False;
          exit;
        end;
    end;

  // 13. D2d
  if (WhichOne = 'D2d') and
    ((Length(edtD1d.text) > 0) and (edtD1d.text[1] in ['0'..'9'])) and
    ((length(edtD2d.text) > 0) and (edtD2d.text[1] in ['0'..'9'])) then
    begin
      if StrToInt(edtD1d.text) < StrToInt(edtD2d.text) then
        begin
          lblVerify.caption :=
            'You recorded that the veteran drank "to intoxication" ' +
            '(D2) more days than the total days drinking any alcohol at all ' +
            '(D1). Probe and recode D1. Remember D2 is a sub-set of D1.' +
            #13 + #10 + 'Click here to continue cross checks.';
          MsgDlgFrm('You recorded that the veteran drank "to intoxication" ' +
            '(D2) more days than the total days drinking any alcohol at all ' +
            '(D1). Probe and recode D1. Remember D2 is a sub-set of D1.',
            MHA_Caption, mtConfirmation, [mbOK],
            0);
          Delete(CrossCheckList, i, Length(WhichOne) + 1);
          pgctrlASI.ActivePage := TabDrug;
          edtD1d.Text := '0';
          ActiveControl := gbD1d;
          lvInitialize := False;
          exit;
        end;
    end;

  // 14. D2y
  if (WhichOne = 'D2y') and (gvASI_Type <> 3 {Full & Lite}) and
    ((Length(edtD1y.text) > 0) and (edtD1y.text[1] in ['0'..'9'])) and
    ((Length(edtD2y.text) > 0) and (edtD2y.text[1] in ['0'..'9'])) then
    begin
      if StrToInt(edtD1y.text) < StrToInt(edtD2y.text) then
        begin
          lblVerify.caption :=
            'You recorded that the veteran drank "to intoxication" ' +
            '(D2) more years than the total number of years drinking any alcohol at all ' +
            '(D1). Remember D2 is a sub-set of D1.' +
            #13 + #10 + 'Click here to continue cross checks.';
          MsgDlgFrm('You recorded that the veteran drank "to intoxication" ' +
            '(D2) more years than the total number of years drinking any alcohol at all ' +
            '(D1). Probe and recode D1. Remember D2 is a sub-set of D1.',
            MHA_Caption, mtConfirmation, [mbOK],
            0);
          Delete(CrossCheckList, i, Length(WhichOne) + 1);
          pgctrlASI.ActivePage := TabDrug;
          edtD1y.Text := '0';
          ActiveControl := gbD1y;
          lvInitialize := False;
          exit;
        end;
    end;

  // 15. D12d
  if (WhichOne = 'D12d') and
    ((Length(edtG20.text) > 0) and (edtG20.Text[1] in ['1'..'9'])) and
    ((edtD1d.Text = '30') or
    (edtD2d.Text = '30') or (edtD6d.Text = '30') or (edtD10d.Text = '30') or
    (edtD3d.Text = '30') or (edtD7d.Text = '30') or (edtD11d.Text = '30') or
    (edtD4d.Text = '30') or (edtD8d.Text = '30') or (edtD12d.Text = '30') or
    (edtD5d.Text = '30') or (edtD9d.Text = '30')) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded in the general information section that the veteran ' +
        'had been in a controlled environment in the past month, yet they used ' +
        'either drugs or alcohol every day. Please review this.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded in the general information section that the veteran '
        +
        'had been in a controlled environment in the past month, yet they used ' +
        'either drugs or alcohol every day. Please review this. Do you wish to ' +
        'change any information in the drug/alcohol grid?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          //ActiveControl := edtD1d;
          Delete(CrossCheckList, Pos('1,', CrossCheckList), 2);
          lvInitialize := false;
          exit;
        end;
    end;

  // 16. D14
  if (WhichOne = 'D14') and (gvASI_Type <> 2 {Full & Followup}) and
    ((length(edtD1d.text) > 0) and (edtD1d.text[1] in ['1'..'9'])) and
    ((rgrpD14.ItemIndex = 3) or
    (rgrpD14.ItemIndex = 4) or (rgrpD14.ItemIndex = 5) or (rgrpD14.ItemIndex = 6)
    or
    (rgrpD14.ItemIndex = 7) or (rgrpD14.ItemIndex = 8) or (rgrpD14.ItemIndex = 9)
    or
    (rgrpD14.ItemIndex = 10) or (rgrpD14.ItemIndex = 11) or (rgrpD14.ItemIndex =
    12) or
    (rgrpD14.ItemIndex = 14)) then
    if StrToInt(edtD1d.text) > 15 then
      begin
        Delete(CrossCheckList, i, Length(WhichOne) + 1);
        lblVerify.caption :=
          'You report that the veteran''s problem does not include alcohol, ' +
          'however, the veteran used alcohol at least 15 days in the past month. Please ' +
          'review this and consider option "Alcohol + drug(s)" for question D14.' +
          #13 + #10 + 'Click here to continue cross checks.';
        if
          MsgDlgFrm('You report that the veteran''s problem does not include alcohol, '
          +
          'however, the veteran used alcohol at least 15 days in the past month. Please ' +
          'review this and consider option "Alcohol + drug(s)" for question D14. Do you ' +
          'want to change D14?',
          MHA_Caption, mtConfirmation, [mbYes, mbNo],
          0) = mrNo then
          begin
            lvInitialize := False;
            if not CrossCheckItems then
              exit;
          end
        else
          begin
            pgctrlASI.ActivePage := TabDrug;
            rgrpD14.ItemIndex := -1;
            ActiveControl := gbD14;
            lvInitialize := False;
            exit;
          end;
      end;

  // 17. D16
  if ((WhichOne = 'D16') and (gvASI_Type = 1 {Full}) and
    (edtD16.text = '0') and
    (((Length(edtD1d.text) > 0) and (edtD1d.Text[1] in ['1'..'9'])) or
    ((Length(edtD2d.text) > 0) and (edtD2d.Text[1] in ['1'..'9'])) or
    ((Length(edtD3d.text) > 0) and (edtD3d.Text[1] in ['1'..'9'])) or
    ((Length(edtD4d.text) > 0) and (edtD4d.Text[1] in ['1'..'9'])) or
    ((Length(edtD5d.text) > 0) and (edtD5d.Text[1] in ['1'..'9'])) or
    ((Length(edtD6d.text) > 0) and (edtD6d.Text[1] in ['1'..'9'])) or
    ((Length(edtD7d.text) > 0) and (edtD7d.Text[1] in ['1'..'9'])) or
    ((Length(edtD8d.text) > 0) and (edtD8d.Text[1] in ['1'..'9'])) or
    ((Length(edtD9d.text) > 0) and (edtD9d.Text[1] in ['1'..'9'])) or
    ((Length(edtD10d.text) > 0) and (edtD10d.Text[1] in ['1'..'9'])) or
    ((Length(edtD11d.text) > 0) and (edtD11d.Text[1] in ['1'..'9'])) or
    ((Length(edtD12d.text) > 0) and (edtD12d.Text[1] in ['1'..'9'])))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded that the veteran is "still sober," however ' +
        'drug or alcohol use in the past 30 days is documented in the drug and ' +
        'alcohol grid. Please review this.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('You recorded that the veteran is "still sober," however ' +
        'drug or alcohol use in the past 30 days is documented in the drug and ' +
        'alcohol grid. Please review this. Do you want to change D16?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          edtD16.Text := '0';
          ActiveControl := gbD16;
          lvInitialize := False;
          exit;
        end;
    end;

  // 18. D17
  if (WhichOne = 'D17') and (gvASI_Type <> 2 {Full & Followup}) and
    ((length(edtD17.text) > 0) and (edtD17.Text[1] in ['1'..'9'])) then
    if StrToInt(edtD17.text) > 5 then
      begin
        Delete(CrossCheckList, i, Length(WhichOne) + 1);
        lblVerify.caption :=
          'You recorded more than 5 episodes of DT''s for the veteran. ' +
          'This is extremely rare, please review the definition of DT''s if you ' +
          'are unsure.' +
          #13 + #10 + 'Click here to continue cross checks.';
        if
          MsgDlgFrm('You recorded more than 5 episodes of DT''s for the veteran. '
          +
          'This is extremely rare, please review the definition of DT''s if you ' +
          'are unsure. Do you want to change D17?',
          MHA_Caption, mtConfirmation, [mbYes, mbNo],
          0) = mrNo then
          begin
            lvInitialize := False;
            if not CrossCheckItems then
              exit;
          end
        else
          begin
            pgctrlASI.ActivePage := TabDrug;
            edtD17.Text := '0';
            ActiveControl := gbD17;
            lvInitialize := False;
            exit;
          end;
      end;

  // 19. D21a
  if (WhichOne = 'D21a') and (gvASI_Type <> 2 {Full & Followup}) and
    ((Length(edtD19.text) > 0) and (edtD19.text[1] in ['0', 'N'])) and
    ((Length(edtD21.text) > 0) and (edtD21.text[1] in ['1'..'9'])) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded that the veteran never had any treatment for ' +
        'alcohol abuse, so the number of detox treatments is not applicable. ' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('You recorded that the veteran never had any treatment for '
        +
        'alcohol abuse, so the number of detox treatments is not applicable. ' +
        'Do you want to recode D19, the total number of treatments received?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          edtD19.Text := '0';
          ActiveControl := gbD19;
          lvInitialize := False;
          exit;
        end;
    end;

  // 20.  D21b
  if (WhichOne = 'D21b') and (gvASI_Type <> 2 {Full & Followup}) and
    ((Length(edtD19.text) > 0) and (edtD19.text[1] in ['0'..'9'])) and
    ((Length(edtD21.text) > 0) and (edtD21.text[1] in ['0'..'9'])) then
    if StrToInt(edtD19.text) < StrToInt(edtD21.text) then
      begin
        Delete(CrossCheckList, i, Length(WhichOne) + 1);
        lblVerify.caption :=
          'You recorded that the veteran had more detox treatments than ' +
          'the total number of treatments received for alcohol abuse. Remember ' +
          'D21 is a sub-set of D19.' +
          #13 + #10 + 'Click here to continue cross checks.';
        if
          MsgDlgFrm('You recorded that the veteran had more detox treatments than '
          +
          'the total number of treatments received for alcohol abuse. Remember ' +
          'D21 is a sub-set of D19. Do you want to recode D19?',
          MHA_Caption, mtConfirmation, [mbYes, mbNo],
          0) = mrNo then
          begin
            lvInitialize := False;
            if not CrossCheckItems then
              exit;
          end
        else
          begin
            pgctrlASI.ActivePage := TabDrug;
            edtD19.Text := '0';
            ActiveControl := gbD19;
            lvInitialize := False;
            exit;
          end;
      end;

  // 21.   D22a
  if (WhichOne = 'D22a') and (gvASI_Type <> 2 {Full & Followup}) and
    ((Length(edtD20.text) > 0) and (edtD20.text[1] in ['1'..'9'])) and
    ((Length(edtD22.text) > 0) and (edtD22.text[1] in ['1'..'9'])) then
    if StrToInt(edtD22.text) > StrToInt(edtD20.text) then
      begin
        Delete(CrossCheckList, i, Length(WhichOne) + 1);
        lblVerify.caption :=
          'You recorded that the veteran had more detox treatments than the ' +
          'total number of treatments received for drug abuse. Remember D22 is a sub-set ' +
          'of D20.' +
          #13 + #10 + 'Click here to continue cross checks.';
        if
          MsgDlgFrm('You recorded that the veteran had more detox treatments than the '
          +
          'total number of treatments received for drug abuse. Remember D22 is a sub-set ' +
          'of D20. Do you want to recode D20?',
          MHA_Caption, mtConfirmation, [mbYes, mbNo], 0) = mrNo then
          begin
            lvInitialize := False;
            if not CrossCheckItems then
              exit;
          end
        else
          begin
            pgctrlASI.ActivePage := TabDrug;
            edtD20.Text := '0';
            ActiveControl := gbD20;
            lvInitialize := False;
            exit;
          end;
      end;

  // 22. D22b
  if (WhichOne = 'D22b') and (gvASI_Type <> 2 {Full & Followup}) and
    ((Length(edtD20.text) > 0) and (edtD20.text[1] in ['0', 'N'])) and
    ((Length(edtD22.text) > 0) and (edtD22.text[1] in ['1'..'9'])) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded that the veteran never had any treatments for drug ' +
        'abuse, so the number of detox treatments is not applicable.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded that the veteran never had any treatments for drug '
        +
        'abuse, so the number of detox treatments is not applicable. Do you want ' +
        'to recode D20, the total number of treatments received?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          edtD20.Text := '0';
          ActiveControl := gbD20;
          lvInitialize := False;
          exit;
        end;
    end;

  // 23. D23a
  if (WhichOne = 'D23a') and
    ((Length(edtD1d.text) > 0) and (edtD1d.text[1] in ['0', 'N'])) and
    ((Length(edtD23.text) > 0) and (edtD23.text[1] in ['1'..'9'])) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded no days of drinking in the past 30, but recorded the ' +
        'veteran has spent money on alcohol. This is unlikely. Please review. ' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded no days of drinking in the past 30, but recorded the '
        +
        'veteran has spent money on alcohol. This is unlikely. Please review. Do you ' +
        'want to change D23?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          edtD23.Text := '0';
          ActiveControl := gbD23;
          lvInitialize := False;
          exit;
        end;
    end;

  // 24. D23b
  if (WhichOne = 'D23b') and
    ((Length(edtD1d.text) > 0) and (edtD1d.text[1] in ['1'..'9'])) and
    ((Length(edtD2d.text) > 0) and (edtD2d.text[1] in ['1'..'9'])) and
    ((Length(edtD23.text) > 0) and (edtD23.text[1] in ['0', 'X', 'N'])) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded the veteran as drinking in the past 30 days (D1 and ' +
        'D2), but no money was spent on alcohol. Please review this.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded the veteran as drinking in the past 30 days (D1 and '
        +
        'D2), but no money was spent on alcohol. Please review this, do you wish ' +
        'to change D23?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          edtD23.Text := '0';
          ActiveControl := gbD23;
          lvInitialize := False;
          exit;
        end;
    end;

  // 25. D24a
  if ((WhichOne = 'D24a') and
    ((Length(edtD24.text) > 0) and (edtD24.text[1] in ['1'..'9'])) and
    (((Length(edtD3d.text) > 0) and (edtD3d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD4d.text) > 0) and (edtD4d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD5d.text) > 0) and (edtD5d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD6d.text) > 0) and (edtD6d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD7d.text) > 0) and (edtD7d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD8d.text) > 0) and (edtD8d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD9d.text) > 0) and (edtD9d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD10d.text) > 0) and (edtD10d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD11d.text) > 0) and (edtD11d.text[1] in ['0', 'X', 'N'])) and
    ((Length(edtD12d.text) > 0) and (edtD12d.text[1] in ['0', 'X', 'N'])))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded zero days of drug use in the past 30, but the veteran ' +
        'acknowledges spending money on drugs this month. Please review, and code ' +
        'those drugs used in the past 30 days.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded zero days of drug use in the past 30, but the veteran '
        +
        'acknowledges spending money on drugs this month. Please review, and code ' +
        'those drugs used in the past 30 days. Do you wish to change number of days ' +
        'of drug use in the drug grid?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          lvInitialize := false;
          exit;
        end;
    end;

  // 26. D24b
  if ((WhichOne = 'D24b') and
    ((Length(edtD24.text) > 0) and (edtD24.text[1] in ['0', 'X', 'N'])) and
    (((Length(edtD3d.text) > 0) and (edtD3d.text[1] in ['1'..'9'])) or
    ((Length(edtD4d.text) > 0) and (edtD4d.text[1] in ['1'..'9'])) or
    ((Length(edtD5d.text) > 0) and (edtD5d.text[1] in ['1'..'9'])) or
    ((Length(edtD6d.text) > 0) and (edtD6d.text[1] in ['1'..'9'])) or
    ((Length(edtD7d.text) > 0) and (edtD7d.text[1] in ['1'..'9'])) or
    ((Length(edtD8d.text) > 0) and (edtD8d.text[1] in ['1'..'9'])) or
    ((Length(edtD9d.text) > 0) and (edtD9d.text[1] in ['1'..'9'])) or
    ((Length(edtD10d.text) > 0) and (edtD10d.text[1] in ['1'..'9'])) or
    ((Length(edtD11d.text) > 0) and (edtD11d.text[1] in ['1'..'9'])) or
    ((Length(edtD12d.text) > 0) and (edtD12d.text[1] in ['1'..'9'])))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded some days of drug use in the past 30 days, but no ' +
        'money was spent on drugs. Please review this.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded some days of drug use in the past 30 days, but no '
        +
        'money was spent on drugs. Please review this. Do you want to change D24?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          edtD24.text := '0';
          ActiveControl := gbD24;
          lvInitialize := False;
          exit;
        end;
    end;

  // 27. D30a
  if (WhichOne = 'D30a') and
    ((length(edtD26.text) > 0) and (edtD26.text[1] in ['1'..'9'])) and
    ((rgrpD28.ItemIndex = 0) or (rgrpD30.ItemIndex = 0)) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'In question D26, you recorded that the veteran had '
        +
        'experienced some days with alcohol problems in the past 30 days. If this ' +
        'is true, then we would expect the veteran would be at least slightly ' +
        'bothered or slightly in need of treatment for these problems.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question D26, you recorded that the veteran had ' +
        'experienced some days with alcohol problems in the past 30 days. If this ' +
        'is true, then we would expect the veteran would be at least slightly ' +
        'bothered or slightly in need of treatment for these problems. Do you want ' +
        'to recode D28 and/or D30?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          rgrpD28.ItemIndex := -1;
          //ActiveControl := rgrpD28;
          lvInitialize := false;
          exit;
        end;
    end;

  // 28. D30b
  if (WhichOne = 'D30b') and
    ((length(edtD26.text) > 0) and (edtD26.text[1] in ['0', 'N'])) and
    ((rgrpD28.ItemIndex <> 0) or (rgrpD30.ItemIndex <> 0)) then
    begin
      lblVerify.caption := 'In question D26, you recorded that the veteran had '
        +
        'experienced no alcohol problems in the past 30 days. Since they report ' +
        'being troubled or wanting treatment, it''s fair to expect that they ' +
        'had some problem days.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('In question D26, you recorded that the veteran had ' +
        'experienced no alcohol problems in the past 30 days. Since they report ' +
        'being troubled or wanting treatment, it''s fair to expect that they ' +
        'had some problem days. Go back to D26 and identify the number of days ' +
        'the problem has bothered them.',
        MHA_Caption, mtConfirmation, [mbOK],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabDrug;
      edtD26.text := '0';
      ActiveControl := gbD26;
      lvInitialize := False;
      exit;
    end;

  // 29. D31a
  if (WhichOne = 'D31a') and
    ((Length(edtD27.text) > 0) and (edtD27.text[1] in ['1'..'9'])) and
    ((rgrpD29.ItemIndex = 0) or (rgrpD31.ItemIndex = 0)) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption := 'In question D27, you recorded that the veteran had '
        +
        'experienced some days with drug problems in the past 30 days. If this ' +
        'is true, then we would expect the veteran would be at least slightly ' +
        'bothered or slightly in need of treatment for these problems.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question D27, you recorded that the veteran had ' +
        'experienced some days with drug problems in the past 30 days. If this ' +
        'is true, then we would expect the veteran would be at least slightly ' +
        'bothered or slightly in need of treatment for these problems. Do you want ' +
        'to recode D31?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabDrug;
          rgrpD31.ItemIndex := -1;
          ActiveControl := gbD31;
          lvInitialize := False;
          exit;
        end;
    end;

  // 30. D31b
  if (WhichOne = 'D31b') and
    ((Length(edtD27.text) > 0) and (edtD27.text[1] in ['0', 'N'])) and
    ((rgrpD29.ItemIndex <> 0) or (rgrpD31.ItemIndex <> 0)) then
    begin
      lblVerify.caption := 'In question D27, you recorded that the veteran had '
        +
        'experienced no drug problems in the past 30 days. Since they report ' +
        'being troubled or wanting treatment, it''s fair to expect that they ' +
        'had some problem days.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('In question D27, you recorded that the veteran had ' +
        'experienced no drug problems in the past 30 days. Since they report ' +
        'being troubled or wanting treatment, it''s fair to expect that they ' +
        'had some problem days. Go back to D27 and identify the number of days ' +
        'the problem has bothered them.',
        MHA_Caption, mtConfirmation, [mbOK],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabDrug;
      edtD27.text := '0';
      ActiveControl := gbD27;
      lvInitialize := False;
      exit;
    end;

  // 31. L17
  if (WhichOne = 'L17') then
    begin
      Arrests := 0;
      if ((length(edtL3.text) > 0) and (edtL3.text[1] in ['0'..'9'])) then
        // use zero in range in case the user types "04"
        Arrests := arrests + StrToInt(edtL3.text);
      if ((length(edtL4.text) > 0) and (edtL4.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL4.text);
      if ((length(edtL5.text) > 0) and (edtL5.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL5.text);
      if ((length(edtL6.text) > 0) and (edtL6.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL6.text);
      if ((length(edtL7.text) > 0) and (edtL7.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL7.text);
      if ((length(edtL8.text) > 0) and (edtL8.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL8.text);
      if ((length(edtL9.text) > 0) and (edtL9.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL9.text);
      if ((length(edtL10.text) > 0) and (edtL10.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL10.text);
      if ((length(edtL11.text) > 0) and (edtL11.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL11.text);
      if ((length(edtL12.text) > 0) and (edtL12.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL12.text);
      if ((length(edtL13.text) > 0) and (edtL13.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL13.text);
      if ((length(edtL14.text) > 0) and (edtL14.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL14.text);
      if ((length(edtL15.text) > 0) and (edtL15.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL15.text);
      if ((length(edtL16.text) > 0) and (edtL16.text[1] in ['0'..'9'])) then
        Arrests := arrests + StrToInt(edtL16.text);

      if (Length(edtL17.text) > 0) then
        if edtL17.text[1] in ['0'..'9'] then
          if StrToInt(edtL17.text) > arrests then
            begin
              lblVerify.caption :=
                'You recorded the veteran had more convictions than the total ' +
                'number of times they were arrested and charged (L3 to L16). This is ' +
                'unusual.' +
                #13 + #10 + 'Click here to continue cross checks.';
              MsgDlgFrm('You recorded the veteran had more convictions than the total '
                +
                'number of times they were arrested and charged (L3 to L16). This is ' +
                'unusual. Reconsider the response in L17.',
                MHA_Caption, mtConfirmation, [mbOK],
                0);
              Delete(CrossCheckList, i, Length(WhichOne) + 1);
              pgctrlASI.ActivePage := TabLegal;
              edtL17.text := '0';
              ActiveControl := gbL17;
              lvInitialize := false;
              exit;
            end;
    end;

  // 32. L20
  if (WhichOne = 'L20') and
    ((rgrpL2.ItemIndex = 1) and
    (edtL3.text = '0') and (edtL8.text = '0') and (edtL13.text = '0') and
    (edtL4.text = '0') and (edtL9.text = '0') and (edtL14.text = '0') and
    (edtL5.text = '0') and (edtL10.text = '0') and (edtL15.text = '0') and
    (edtL6.text = '0') and (edtL11.text = '0') and (edtL16.text = '0') and
    (edtL7.text = '0') and (edtL12.text = '0') and (edtL18.text = '0') and
    (edtL19.text = '0') and (edtL20.text = '0')) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In question L2, you indicated the veteran is on probation ' +
        'or parole. However, no arrests or charges are documented in items L3 through L20.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question L2, you indicated the veteran is on probation '
        +
        'or parole. However, no arrests or charges are documented in items L3 through L20.' +
        'Do you want to recode any legal charges?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabLegal;
          lvInitialize := false;
          exit;
        end;
    end;

  // 33. L24
  if (WhichOne = 'L24') and
    ((rgrpL24.ItemIndex = 1) and (edtL3.text = '0') and
    (edtL3.text = '0') and (edtL8.text = '0') and (edtL13.text = '0') and
    (edtL4.text = '0') and (edtL9.text = '0') and (edtL14.text = '0') and
    (edtL5.text = '0') and (edtL10.text = '0') and (edtL15.text = '0') and
    (edtL6.text = '0') and (edtL11.text = '0') and (edtL16.text = '0') and
    (edtL7.text = '0') and (edtL12.text = '0') and (edtL18.text = '0') and
    (edtL19.text = '0') and (edtL20.text = '0')) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded the veteran is awaiting charges, trial or sentence in ' +
        'question L24, but no arrests and/or charges are coded in items L3 through L16 or ' +
        'L18 through L20.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded the veteran is awaiting charges, trial or sentence in '
        +
        'question L24, but no arrests and/or charges are coded in items L3 through L16 or ' +
        'L18 through L20. Do you want to recode any of the charges?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabLegal;
          ActiveControl := gbL3;
          lvInitialize := false;
          exit;
        end;
    end;

  // 34. L25
  if (WhichOne = 'L25') and
    ((rgrpL24.ItemIndex = 0) and (rgrpL25.ItemIndex < 17)) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded the veteran was not awaiting charges, trial or sentence in ' +
        'question L24, yet you coded a charge in L25 (which would be not applicable). ' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded the veteran was not awaiting charges, trial or sentence in '
        +
        'question L24, yet you coded a charge in L25 (which would be not applicable). ' +
        'Do you want to recode L24?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabLegal;
          ActiveControl := gbL24;
          lvInitialize := False;
          exit;
        end;
    end;

  // 35. L26
  if (WhichOne = 'L26') and
    ((rgrpG19.ItemIndex = 1) and
    ((Length(edtL26.text) > 0) and (edtL26.text[1] in ['0', 'N']))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In the general information section, you recorded that the veteran ' +
        'had been in jail in the past 30 days, this is also represented in question ' +
        'L26.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('In the general information section, you recorded that the veteran '
        +
        'had been in jail in the past 30 days, this is also represented in question ' +
        'L26. Do you want to change L26?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabLegal;
          edtL26.text := '0';
          ActiveControl := gbL26;
          lvInitialize := False;
          exit;
        end;
    end;

  // 36. L27
  if (WhichOne = 'L27') and
    (((Length(edtE17.text) > 0) and (edtE17.text[1] in ['1'..'9'])) and
    (edtL27.text = '0')) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In the employment section, you recorded that the veteran had illegal ' +
        'income in the past 30 days, this is usually also documented in question L27.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('In the employment section, you recorded that the veteran had illegal '
        +
        'income in the past 30 days, this is usually also documented in question L27. Do you ' +
        'wish to change your code in L27?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabLegal;
          edtL27.text := '0';
          ActiveControl := gbL27;
          lvInitialize := False;
          exit;
        end;
    end;

  // 37. F34a
  if (WhichOne = 'F34a') and
    (((Length(edtF30.text) > 0) and (edtF30.text[1] in ['1'..'9'])) and
    ((rgrpF32.ItemIndex = 0) or (rgrpF34.ItemIndex = 0))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In question F30, you recorded that the veteran had some ' +
        'family conflicts in the past 30 days. If this is true, then we would expect ' +
        'that the veteran would be at least slightly bothered or slightly in need of ' +
        'treatment.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question F30, you recorded that the veteran had some ' +
        'family conflicts in the past 30 days. If this is true, then we would expect ' +
        'that the veteran would be at least slightly bothered or slightly in need of ' +
        'treatment. Do you wish to recode F32 or F34?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabSocial;
          ActiveControl := gbF32;
          lvInitialize := false;
          exit;
        end;
    end;

  // 38. F34b
  if (WhichOne = 'F34b') and
    (((Length(edtF30.text) > 0) and (edtF30.text[1] in ['0', 'X', 'N'])) and
    (((rgrpF32.ItemIndex > 0) and (rgrpF32.ItemIndex < 5)) or
    ((rgrpF34.ItemIndex > 0) and (rgrpF34.ItemIndex < 5)))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In question F30, you recorded that the veteran had no ' +
        'family conflicts in the past 30 days. Since they report being troubled ' +
        'or wanting treatment, it''s fair to expect that they had some problem days. ' +
        'Do you want to go back to F30 and identify the number of days the problem ' +
        'has bothered them?' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question F30, you recorded that the veteran had no ' +
        'family conflicts in the past 30 days. Since they report being troubled ' +
        'or wanting treatment, it''s fair to expect that they had some problem days. ' +
        'Do you want to go back to F30 and identify the number of days the problem ' +
        'has bothered them?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabSocial;
          edtF30.text := '0';
          ActiveControl := gbF30;
          lvInitialize := False;
          exit;
        end;
    end;

  // 39. F35a
  if (WhichOne = 'F35a') and (gvASI_Type <> 2 {Full & Followup}) and
    (((Length(edtF31.text) > 0) and (edtF31.text[1] in ['1'..'9'])) and
    ((rgrpF33.ItemIndex = 0) and (rgrpF35.ItemIndex = 0))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In question F31, you recorded that the veteran had some ' +
        'conflicts with others in the past 30 days. If this is true, then we would expect ' +
        'that the veteran would be at least slightly bothered or slightly in need of ' +
        'treatment.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question F31, you recorded that the veteran had some ' +
        'conflicts with others in the past 30 days. If this is true, then we would expect ' +
        'that the veteran would be at least slightly bothered or slightly in need of ' +
        'treatment. Do you wish to recode F33 or F35?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabSocial;
          ActiveControl := gbF33;
          lvInitialize := False;
          exit;
        end;
    end;

  // 40. F35b
  if (WhichOne = 'F35b') and (gvASI_Type <> 2 {Full & Followup}) and
    (((Length(edtF31.text) > 0) and (edtF31.text[1] in ['0', 'X', 'N'])) and
    (((rgrpF33.ItemIndex > 0) and (rgrpF33.ItemIndex < 5)) or
    ((rgrpF35.ItemIndex > 0) and (rgrpF35.ItemIndex < 5)))) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'In question F31, you recorded that the veteran had no ' +
        'conflicts with others in the past 30 days. Since they report being troubled ' +
        'or wanting treatment, it''s fair to expect that they had some problem days.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if MsgDlgFrm('In question F31, you recorded that the veteran had no ' +
        'conflicts with others in the past 30 days. Since they report being troubled ' +
        'or wanting treatment, it''s fair to expect that they had some problem days. ' +
        'Do you want to go back to F31 and identify the number of days the problem ' +
        'has bothered them?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabSocial;
          edtF31.text := '0';
          ActiveControl := gbF31;
          lvInitialize := False;
          exit;
        end;
    end;

  // 41. P9b
  if (WhichOne = 'P9b') and
    ((Length(edtP1a.text) > 0) and (edtP1a.text[1] in ['0', 'N', 'X'])) and
    ((Length(edtP1b.text) > 0) and (edtP1b.text[1] in ['0', 'N', 'X'])) and
    ((rgrpP9.ItemIndex = 1) or (rgrpP9l.ItemIndex = 1)) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded in question P1 that the veteran has had no inpatient ' +
        'or outpatient treatment for psychiatric problems, yet they have attempted ' +
        'suicide. Please review treatment they may have received for the suicide ' +
        'attempt.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded in question P1 that the veteran has had no inpatient '
        +
        'or outpatient treatment for psychiatric problems, yet they have attempted ' +
        'suicide. Please review treatment they may have received for the suicide ' +
        'attempt. Do you want to recode P1?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabPsych;
          ActiveControl := gbP1a;
          lvInitialize := False;
          exit;
        end;
    end;

  // 42. P10
  if (WhichOne = 'P10') and
    ((Length(edtP1a.text) > 0) and (edtP1a.text[1] in ['0', 'N', 'X'])) and
    ((Length(edtP1b.text) > 0) and (edtP1b.text[1] in ['0', 'N', 'X'])) and
    ((rgrpP10.ItemIndex = 1) or (rgrpP10l.ItemIndex = 1)) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You recorded in question P1 that the veteran has had no inpatient ' +
        'or outpatient treatment for psychiatric problems, yet they have been prescribed ' +
        'medications for psychiatric problems.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You recorded in question P1 that the veteran has had no inpatient '
        +
        'or outpatient treatment for psychiatric problems, yet they have been prescribed ' +
        'medications for psychiatric problems. Do you want to recode P1?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabPsych;
          ActiveControl := gbP1a;
          lvInitialize := False;
          exit;
        end;
    end;

  // 43. P2
  if (WhichOne = 'P2') and
    ((Length(edtE15.text) > 0) and (edtE15.text[1] in ['0', 'N', 'X'])) and
    (rgrpP2.ItemIndex = 1) then
    begin
      lblVerify.caption :=
        'You recorded that the veteran receives a pension for a psychiatric ' +
        'problem in P2. Unless they did not receive any cash this month, this income ' +
        'is generally recorded in the employment question about pension money received, ' +
        'item E15.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('You recorded that the veteran receives a pension for a psychiatric '
        +
        'problem in P2. Unless they did not receive any cash this month, this income ' +
        'is generally recorded in the employment question about pension money received, ' +
        'item E15. Please change E15 in the Employment section.',
        MHA_Caption, mtConfirmation, [mbOk],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabEmploy;
      ActiveControl := gbE15;
      lvInitialize := False;
      exit;
    end;

  // 44. P11a
  if (WhichOne = 'P11a') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['1'..'9'])) and
    ((rgrpP3.ItemIndex = 0) and (rgrpP4.ItemIndex = 0) and
    (rgrpP5.ItemIndex = 0) and (rgrpP6.ItemIndex = 0) and
    (rgrpP7.ItemIndex = 0) and (rgrpP8.ItemIndex = 0) and
    (rgrpP9.ItemIndex = 0)) then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      lblVerify.caption :=
        'You report that the veteran had problems in the past 30 days, ' +
        'but none are recorded in the psychiatric symptoms list, P3 through P9. ' +
        'Please review P3 through P9.' +
        #13 + #10 + 'Click here to continue cross checks.';
      if
        MsgDlgFrm('You report that the veteran had problems in the past 30 days, '
        +
        'but none are recorded in the psychiatric symptoms list, P3 through P9. ' +
        'Please review P3 through P9, do you want to go back and change the code ' +
        'of any symptoms?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo],
        0) = mrNo then
        begin
          lvInitialize := False;
          if not CrossCheckItems then
            exit;
        end
      else
        begin
          pgctrlASI.ActivePage := TabPsych;
          ActiveControl := gbP3;
          lvInitialize := False;
          exit;
        end;
    end;

  // 45. P11b
  if (WhichOne = 'P11b') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['0', 'X', 'N'])) and
    ((rgrpP3.ItemIndex = 1) or (rgrpP4.ItemIndex = 1) or
    (rgrpP5.ItemIndex = 1) or (rgrpP6.ItemIndex = 1) or
    (rgrpP7.ItemIndex = 1) or (rgrpP8.ItemIndex = 1) or
    (rgrpP9.ItemIndex = 1)) then
    begin
      lblVerify.caption :=
        'You report that the veteran had no problems in the past 30 days, ' +
        'but problems are evident from the psychiatric symptoms list, P3 through P9. ' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('You report that the veteran had no problems in the past 30 days, '
        +
        'but problems are evident from the psychiatric symptoms list, P3 through P9. ' +
        'Please probe about the number of days these symptoms bothered the veteran and  ' +
        'recode P11.',
        MHA_Caption, mtConfirmation, [mbOk],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabPsych;
      ActiveControl := gbP11;
      lvInitialize := False;
      exit;
    end;

  // 47. P13a
  if (WhichOne = 'P13a') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['1'..'9'])) and
    ((rgrpP12.ItemIndex = 0) or (rgrpP13.ItemIndex = 0)) then
    begin
      lblVerify.caption :=
        'You report that the veteran had psychiatric problems in the past ' +
        '30 days in question P11. Given these problems, we would expect that the veteran ' +
        'would be at least slightly bothered or slightly in need of treatment ' +
        'for this recent problem.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('You report that the veteran had psychiatric problems in the past '
        +
        '30 days in question P11. Given these problems, we would expect that the veteran ' +
        'would be at least slightly bothered or slightly in need of treatment ' +
        'for this recent problem. Please recode P12 and/or P13.',
        MHA_Caption, mtConfirmation, [mbOk],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabPsych;
      lvInitialize := False;
      ActiveControl := gbP12;
      exit;
    end;

  // 48. P13b
  if (WhichOne = 'P13b') and
    ((Length(edtP11.text) > 0) and (edtP11.text[1] in ['0', 'N', 'X'])) and
    (((rgrpP12.ItemIndex > 0) and (rgrpP12.ItemIndex < 5)) or
    ((rgrpP13.ItemIndex > 0) and (rgrpP13.ItemIndex < 5))) then
    begin
      lblVerify.caption :=
        'You report that the veteran had no psychiatric problems in the past ' +
        '30 days in question P11. If they report being troubled or wanting treatment, ' +
        'it''s fair to expect that they had some problem days.' +
        #13 + #10 + 'Click here to continue cross checks.';
      MsgDlgFrm('You report that the veteran had no psychiatric problems in the past '
        +
        '30 days in question P11. If they report being troubled or wanting treatment, ' +
        'it''s fair to expect that they had some problem days. Please identiry the number of ' +
        'days the symptoms have bothered the veteran.',
        MHA_Caption, mtConfirmation, [mbOk],
        0);
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      pgctrlASI.ActivePage := TabPsych;
      ActiveControl := gbP11;
      lvInitialize := False;
      exit;
    end;

  lvInitialize := False;
  if not CrossCheckItems then
    Exit;

  // Done with Cross checks
  if Length(Trim(CrossCheckList)) = 0 then
    begin
      ConductCrossChecks(sender); // need to clear its result
      exit;
    end;

  // Item was okay, go to next cross check item if needed
  if i > 0 then
    begin
      Delete(CrossCheckList, i, Length(WhichOne) + 1);
      if CrossCheckItems then
        ConductCrossChecks(sender);
    end;
end;

function TfrmMHA_ASI_Entry_508.SaveSign: boolean;
var
  Reply,
    Sig,
    MySig                     : string;
label
  skip;

  function GetElectronicSignature: string;
  begin
    SignatureForASI(Font.Size,
      'Sign the Addiction Severity Index',
      'Electronic Signature',
      MySig);
    if MySig = '' then
      begin
        result := 'Cancel';
        exit;
      end;

    if ASI_OPS.Sign(MySig, TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS) then
      result := 'Done'
    else
      result := 'Error';
  end;

begin
  result := false;

  if (dtpG5.Text = '') then
    exit;

  // New administration.
  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS = '' then
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS :=
      ASI_OPS.SaveNewInterview(
      Patient.DFN,
      dtpG5.Text, //(dtpG5.Date),
      Piece(cbxG11Interviewerx.Items[cbxG11Interviewerx.ItemIndex], U, 1),
      Piece(cbxG11OrderedByx.Items[cbxG11OrderedByx.ItemIndex], U, 1),
      TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType);

  // Attempt to save ASI
  Reply := ASI_Save_508.SaveASIData(
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS,
    Patient.DFN,
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerDUZ,
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByDUZ,
    DateToStr(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).ASIDate),
    ProgramIEN,
    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType); // uMHA_ASI_Save_508

  if Reply <> OK then
    begin
      MsgDlgFrm(Reply, MHA_Caption, mtError, [mbOK], 0);
      exit;
    end;

  // Ensure that "Ordered By" is the person signing this ASI.
  if not TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).CanBeSignedByUser then
    begin
      //MsgDlgFrm('The only person allowed to sign this A S I is "Ordered By".',
      //  'Electronic Signature', mtInformation, [mbOK], 0);
      Result := true;
      exit;
    end;

  // User signs ASI
  repeat
    Sig := GetElectronicSignature;
    if sig = 'Cancel' then
      exit;
    if Sig = 'Done' then
      goto Skip;
    MsgDlgFrm('This signature was not recognized. Capitalization may be required. '
      +
      'Please try again.',
      'Electronic Signature', mtInformation, [mbOK], 0);
  until sig = 'Done';
  skip:

  if ((Sig = 'Done') and (rgrpG12.ItemIndex = 3)) then
    begin
      if
        MsgDlgFrm('Would you like to save a narrative progress note for this A S I?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo], 0)
        = mrYes then

        if not ASI_OPS.SaveProgressNote(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS) then
          MsgDlgFrm('The A S I progress note was not written to VistA database.',
            MHA_Caption, mtError, [mbOK], 0);

    end;

  result := true;
end;

function TfrmMHA_ASI_Entry_508.SignAndExit: boolean;
begin
  result := false;

  if not SaveSign then
    begin
   // Reset verify parameters
      CheckSkipItems := False;
      CrossCheckItems := False;
      CrossCheckList := 'G20,M1,M2,M7,M8,E1y,E12a,E12b,E15,E20,D2d,D2y,D12d,' +
        'D14,D16,D17,D21a,D21b,D22a,D22b,D23,D24a,D24b,D30a,D30b,D31a,D31b,L17,L20,L24,' +
        'L25,L26,L27,F34a,F34b,F35a,F35b,P2,P9a,P9b,P10,P11a,P11b,P13a,P13b,';
      exit;
    end;

  // Show user the saved message.
  gvNameOfInstrument := 'A S I';

  try
    frmMHA_Saved := TfrmMHA_Saved.Create(Application);
    sleep(2000); // milliseconds
    frmMHA_Saved.Free;
  except
    on E: Exception do
      begin
        MsgDlgFrm(E.Message, 'Form Save',
          mtError, [mbOk], 0);
        exit;
      end;
  end; {-try -}
  result := true;
end;

procedure TfrmMHA_ASI_Entry_508.ShowHint(Sender: TObject);
begin
 //
end;


procedure TfrmMHA_ASI_Entry_508.CleanupComments;
// The M side stores all carriage returns causing excessive word wrapping.
begin
  // Mark the double carriage returns. This keeps the paragraphs intact.
  memDrug.Text := StringReplace(memDrug.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memEmploy.Text := StringReplace(memEmploy.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memFamily.Text := StringReplace(memFamily.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memGeneral.Text := StringReplace(memGeneral.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memLegal.Text := StringReplace(memLegal.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memLeisure.Text := StringReplace(memLeisure.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memMedical.Text := StringReplace(memMedical.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memPsych.Text := StringReplace(memPsych.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memSocial.Text := StringReplace(memSocial.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  memSpirit.Text := StringReplace(memSpirit.Text, #$D#$A#$D#$A, #240,
    [rfReplaceAll, rfIgnoreCase]);
  // Remove all carriage returns
  memDrug.Text := StringReplace(memDrug.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memEmploy.Text := StringReplace(memEmploy.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memFamily.Text := StringReplace(memFamily.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memGeneral.Text := StringReplace(memGeneral.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memLegal.Text := StringReplace(memLegal.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memLeisure.Text := StringReplace(memLeisure.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memMedical.Text := StringReplace(memMedical.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memPsych.Text := StringReplace(memPsych.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memSocial.Text := StringReplace(memSocial.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  memSpirit.Text := StringReplace(memSpirit.Text, #$D#$A, '',
    [rfReplaceAll, rfIgnoreCase]);
  // Replace the paragraph markers
  memDrug.Text := StringReplace(memDrug.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memEmploy.Text := StringReplace(memEmploy.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memFamily.Text := StringReplace(memFamily.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memGeneral.Text := StringReplace(memGeneral.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memLegal.Text := StringReplace(memLegal.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memLeisure.Text := StringReplace(memLeisure.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memMedical.Text := StringReplace(memMedical.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memPsych.Text := StringReplace(memPsych.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memSocial.Text := StringReplace(memSocial.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
  memSpirit.Text := StringReplace(memSpirit.Text, #240, #$D#$A#$D#$A,
    [rfReplaceAll, rfIgnoreCase]);
end;

procedure TfrmMHA_ASI_Entry_508.mnuOnlineSupportClick(Sender: TObject);
begin
  VisitURL(MHA_ONLINE_SUPPORT_URL);
end;

procedure TfrmMHA_ASI_Entry_508.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  //gvLeft := Left;
  //gvTop := Top;
  //gvHeight := Height;
  //gvWidth := Width;
end;

procedure TfrmMHA_ASI_Entry_508.ChangePT;
begin
  MsgDlgFrm('The patient in C P R S has been changed and does not match ' +
    'the patient in use here.' + #13 + #13 +
    'You should close this A S I immediately.',
    MHA_CAPTION, mtWarning, [mbOk], 0);

end;

procedure TfrmMHA_ASI_Entry_508.cbxG3Exit(Sender: TObject);
begin
  ListBoxOnExit(Sender);
end;

procedure TfrmMHA_ASI_Entry_508.dtpG4Exit(Sender: TObject);
begin
  EditOnExit(Sender);
end;

procedure TfrmMHA_ASI_Entry_508.edtG14yExit(Sender: TObject);
begin
  SpinEditOnExit(Sender);
end;

initialization
  RegisterClasses([TMHA_SpnEdt]);

end.

