unit fMHA_SpecialResultsWizard;
{*
This form will allow user to select instruments to display on a single form.
Only instruments that have scale(s) can be used here.

*}


interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ComCtrls,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  StdCtrls,
  Buttons,
  VirtualTrees,
  ExtCtrls,
  RzLabel,
  RzPanel,
  Menus;

type
  TfrmMHA_SpecialResultsWizard = class(TForm)
    pnlHeader: TRzPanel;
    lblHeader: TRzLabel;
    imgBackup: TImage;
    pnlSpacer0: TPanel;
    pnlSpacer1: TPanel;
    grpAvailable: TGroupBox;
    treSelect: TVirtualStringTree;
    pnlSpacer4: TPanel;
    grpInstructions: TGroupBox;
    lblInstructions: TLabel;
    pnlRight: TPanel;
    pnlSpacer3: TPanel;
    btnUp: TBitBtn;
    btnDown: TBitBtn;
    btnRemove: TBitBtn;
    pnlButton: TPanel;
    btnSave: TJvXPButton;
    grpPickedInstruments: TGroupBox;
    lstChosen: TListView;
    mnuMain: TMainMenu;
    mnuFile: TMenuItem;
    mnuExit: TMenuItem;
    mnuHelp: TMenuItem;
    mnuInstrumentDescription: TMenuItem;
    N3: TMenuItem;
    mnuOnlineSupport: TMenuItem;
    N2: TMenuItem;
    mnuAbout: TMenuItem;
    PopupMenu1: TPopupMenu;
    mnuDescription: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure ConfigureForm;
    procedure DisplayAvailableInstruments;
    procedure DisplayHSInstruments;
    procedure UpdateButtons;
    procedure SaveHSInstrumentsToRegistry;

    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure lstChosenEnter(Sender: TObject);
    procedure lstChosenDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lstChosenDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lstChosenMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure treSelectCheckThisNode(ThisNode: string);
    procedure treSelectUnCheckThisNode(ThisNode: string);
    procedure imgBackupClick(Sender: TObject);
    procedure treSelectDblClick(Sender: TObject);
    procedure treSelectChecked(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure treSelectClick(Sender: TObject);
    procedure treSelectEnter(Sender: TObject);
    procedure treSelectGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure treSelectInitChildren(Sender: TBaseVirtualTree;
      Node: PVirtualNode; var ChildCount: Cardinal);
    procedure treSelectInitNode(Sender: TBaseVirtualTree; ParentNode,
      Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);

    procedure lstChosenRedundant;
    procedure RemoveDeselectedInstrument(AInstrument: string);
    function treSelectGetChildCount(NodeIn: string): Integer;
    function treSelectGetItems(PNode: string; ChildNumber: integer): string;
    function treSelectGetHasChildren(NodeName: string): Boolean;
    procedure btnSaveClick(Sender: TObject);
    procedure lstChosenDblClick(Sender: TObject);
    procedure lstChosenSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure lstChosenMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure mnuInstrumentDescriptionClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMHA_SpecialResultsWizard : TfrmMHA_SpecialResultsWizard;

implementation

{$R *.dfm}

uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uMHA_Registry,
  uIA_ThisUser,
  fIA_Dialogs,
  uIA_Instrument,
  uMHA_Instrument_OPS,
  uIA_AvailableInstrument,
  fMHA_About,
  uIA_GUIControl,
  fMHA_InstrumentDescription,
  uMHA_SpecialResults_OPS,
  fMHA_Progress,
  uMHA_DM;

type
  // Node data record
  PSelectData = ^TSelectData;

  TSelectData = record
    Caption: WideString;
    Style: Char; // B,I,R Battery, Incomplete, Regular
    ImageIndex: Integer;
  end;

const
  lcColor                     : TColor = clInfoBk;
  lcGraphOnly                 : string = ' (display graph only)';
  lcGraphAndQA                : string = ' (display graph and Q/A)';

var
// "lv" Local Variable
  lvNodeHeight,
    lvRootNode,
    lvChildren                : Integer;
  treSelectHS                 : TStringList;
  lvInitialize                : Boolean;

procedure TfrmMHA_SpecialResultsWizard.FormCreate(Sender: TObject);
begin
  lvInitialize := True;
  // Keep Position out of ConfigureForm when ConfigureForm can be called
  // more than once with the form.
  Position := poDesigned;

  treSelect.Images := nil;
  treSelectHS := TStringList.Create;

  ConfigureForm;

  DisplayAvailableInstruments;
  DisplayHSInstruments;
  ShowModal;
end;

procedure TfrmMHA_SpecialResultsWizard.FormShow(Sender: TObject);
begin
  Application.ProcessMessages;
  lvInitialize := False;
  Screen.Cursor := crDefault;
  ActiveControl := treSelect;
end;

procedure TfrmMHA_SpecialResultsWizard.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  treSelectHS.Free;
end;

procedure TfrmMHA_SpecialResultsWizard.ConfigureForm;
var
  i                           : Integer;
begin
  // Add a unique color
  for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TPanel then
        TPanel(Components[i]).Color := lcColor;
      if Components[i] is TLabel then
        TLabel(Components[i]).Color := lcColor;
      if Components[i] is TGroupBox then
        TGroupBox(Components[i]).Color := lcColor;
    end;

  // GUI Settings
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;

      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Width := GetWidth(gvWidth);
      Height := GetHeight(gvHeight);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlButton.Height := 3 * ButtonHeight div 2;
      btnSave.Font.Assign(gvFont);
      btnSave.Left := 0;
      btnSave.Top := ButtonHeight div 4;
      btnSave.Height := ButtonHeight;
      btnSave.Width := ButtonWidthNormal;
      pnlHeader.Height := pnlHeaderHeight;
      lblHeader.Font.Size := lblHeaderFontSize;

      grpInstructions.Width := (5 * ButtonWidthNormal) div 2;
      grpAvailable.Width := grpInstructions.Width;
      lblInstructions.Font.Assign(gvFont);
      lblInstructions.Font.Color := clNavy;
      grpInstructions.Font.Color := clNavy;
      lvNodeHeight := 3 * TextHeight('Yy', gvFont) div 2;
    end;

  lstChosen.Clear;
  UpdateButtons;
  mnuInstrumentDescription.Enabled := False;

  Caption := MHA_Caption;
  grpAvailable.Caption := ' &Available Instruments:  ';
  grpPickedInstruments.Caption := ' &Instruments in Special Results: ';

  btnUp.Caption := '';
  btnDown.Caption := '';
  btnRemove.Caption := '';

end;

procedure TfrmMHA_SpecialResultsWizard.DisplayHSInstruments;
var
  i                           : Integer;
  TempStr1,
    TempStr                   : string;
begin
  MHA_Registry.ReadRegistry;
  lvInitialize := True;

  for i := 1 to NumberOfPieces(gvSpecialResultsList, RECORD_END) do
    begin
      TempStr := Piece(gvSpecialResultsList, RECORD_END, i);

      TempStr1 := Piece(TempStr, U, 1);
      if TempStr1 <> '' then
        begin
          treSelectCheckThisNode(TempStr1);

          if Piece(TempStr, U, 2) = 'G' then
            TempStr1 := TempStr1 + lcGraphOnly
          else
            TempStr1 := TempStr1 + lcGraphAndQA;

          lstChosen.Items.Add.Caption := TempStr1;
        end;
    end;

  lvInitialize := False;
end;

//------------------------------------------------------------------------------
//              Populate Tree Views
//------------------------------------------------------------------------------

procedure TfrmMHA_SpecialResultsWizard.DisplayAvailableInstruments;
begin

  treSelect.Colors.FocusedSelectionColor := clHighlight;
  treSelect.Colors.UnfocusedSelectionColor := clBtnFace;
  treSelect.Font.Assign(gvFont);
  treSelect.Font.Color := clBlack;
  treSelect.DefaultNodeHeight := lvNodeHeight;
  treSelect.Clear;

  treSelectHS.Clear;
  lvRootNode := 0; // The first node in lvtreSelectBatt;

  treSelectHS.Add('Root=' + SpecialResults_OPS.LoadSpecialResultsInstruments);

  treSelect.RootNodeCount := NumberOfPieces(treSelectHS[0], U) - 1;
  treSelect.NodeDataSize := SizeOf(TSelectData);
end;

procedure TfrmMHA_SpecialResultsWizard.UpdateButtons;
begin
  // Allow for saving of no instruments: otherwise the user cannot turn-off
  // Special Results Tab.
  //if lstChosen.Items.Count <= 0 then
    //btnSave.Enabled := False
  //else
    //btnSave.Enabled := True;
  if lstChosen.Items.Count > 1 then
    begin
      btnUp.Enabled := true;
      btnDown.Enabled := true;
    end
  else
    begin
      btnUp.Enabled := False;
      btnDown.Enabled := False;
    end;

  if lstChosen.SelCount > 0 then
    btnRemove.Enabled := true
  else
    btnRemove.Enabled := False;
end;


procedure TfrmMHA_SpecialResultsWizard.treSelectCheckThisNode(ThisNode: string);
// This is used to check a given instrument when the battery is selected.
var
  Node                        : PVirtualNode;
  Data                        : PSelectData;
  i                           : Integer;
begin
  // Do the first node
  Node := treSelect.GetFirst;
  Data := treSelect.GetNodeData(Node);
  if Data^.Caption = ThisNode then
    treSelect.CheckState[Node] := csCheckedNormal;

  // Now do the remaining
  for i := 1 to treSelect.RootNodeCount - 1 do
    begin
      Node := treSelect.GetNextSibling(Node);
      Data := treSelect.GetNodeData(Node);
      if Data^.Caption = ThisNode then
        treSelect.CheckState[Node] := csCheckedNormal;
    end;
  treSelect.Refresh;
end;

procedure TfrmMHA_SpecialResultsWizard.treSelectUnCheckThisNode(ThisNode: string);
// This is used to uncheck a given instrument when it is dropped from the
// list of selected instruments
var
  Node                        : PVirtualNode;
  Data                        : PSelectData;
  i                           : Integer;
begin
  Node := treSelect.GetFirst;
  for i := 1 to treSelect.RootNodeCount - 1 do
    begin
      Node := treSelect.GetNextSibling(Node);
      Data := treSelect.GetNodeData(Node);
      if Data^.Caption = ThisNode then
        treSelect.CheckState[Node] := csUnCheckedNormal;
    end;
  treSelect.Refresh;
end;


procedure TfrmMHA_SpecialResultsWizard.btnRemoveClick(Sender: TObject);
var
  i                           : Integer;
  tempCaption                 : string;
begin
  mnuInstrumentDescription.Enabled := False;
  tempCaption := lstChosen.Selected.Caption;

  i := lstChosen.Items.IndexOf(lstChosen.Selected);
  lstChosen.Items.Delete(i);

  treSelectUnCheckThisNode(TempCaption);

  UpdateButtons;
end;


procedure TfrmMHA_SpecialResultsWizard.btnUpClick(Sender: TObject);
var
  i                           : Integer;
  tempstr                     : string;
begin
  mnuInstrumentDescription.Enabled := False;
  for i := 1 to lstChosen.Items.Count - 1 do
    if lstChosen.Items[i].Selected then
      begin
        tempstr := lstChosen.Items[i - 1].Caption;
        lstChosen.Items[i - 1].Caption := lstChosen.Items[i].Caption;
        lstChosen.Items[i].Caption := tempstr;
        break;
      end;
  ActiveControl := lstChosen;
  if i < 2 then
    lstChosen.Items[0].Selected := True
  else
    lstChosen.Items[i - 1].Selected := True;
end;


procedure TfrmMHA_SpecialResultsWizard.btnDownClick(Sender: TObject);
var
  i                           : Integer;
  tempstr                     : string;
begin
  mnuInstrumentDescription.Enabled := False;
  for i := 0 to lstChosen.Items.Count - 2 do
    if lstChosen.Items[i].Selected then
      begin
        tempstr := lstChosen.Items[i + 1].Caption;
        lstChosen.Items[i + 1].Caption := lstChosen.Items[i].Caption;
        lstChosen.Items[i].Caption := tempstr;
        break;
      end;
  ActiveControl := lstChosen;
  if i > lstChosen.Items.Count - 2 then
    Exit;
  lstChosen.Items[i + 1].Selected := True;
end;

//------------------------------------------------------------------------------
//            Drag and Drop
//------------------------------------------------------------------------------

procedure TfrmMHA_SpecialResultsWizard.lstChosenDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  OldIndex, NewIndex          : integer;
  InsertItem                  : TListItem;
  Old                         : string;
begin
  Old := lstChosen.Selected.Caption;
  OldIndex := lstChosen.Items.IndexOf(lstChosen.Selected);
  NewIndex := lstChosen.Items.IndexOf(lstChosen.GetItemAt(X, Y));
  if NewIndex = -1 then
    NewIndex := lstChosen.Items.Count - 1;
  with Source as TListView do
    begin
      Items.Delete(OldIndex);
      InsertItem := Items.Insert(NewIndex);
      InsertItem.Caption := Old;
    end;
end;


procedure TfrmMHA_SpecialResultsWizard.lstChosenDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if Source is TListView then
    Accept := True;
end;


procedure TfrmMHA_SpecialResultsWizard.lstChosenMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    with Sender as TListView do
      BeginDrag(False);
end;


procedure TfrmMHA_SpecialResultsWizard.lstChosenEnter(Sender: TObject);
begin
  lblInstructions.Caption := #13 + '    This is the order in which the instruments ' +
    'will be presented. Use the arrows or drag && drop to change the order.' + CRLF + CRLF +
    'Double click on instrument to toggle the presentation of questions and answers';
  lblInstructions.Update;
  mnuInstrumentDescription.Enabled := False;
end;


procedure TfrmMHA_SpecialResultsWizard.imgBackupClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_SpecialResultsWizard.treSelectDblClick(Sender: TObject);
begin
  treSelectChecked(treSelect, treSelect.GetFirstSelected);
end;

procedure TfrmMHA_SpecialResultsWizard.lstChosenRedundant;
var
  i, j                        : Integer;
  TempStr,
    TempStr1                  : string;

  function RemoveParen(StringIn: string): string;
  var
    x                         : Integer;
    TempSt                    : string;
  begin
    TempSt := StringIn;
    x := Pos(lcGraphOnly, TempSt);
    if x > 0 then
      Delete(TempSt, x, Length(TempSt) - x + 1)
    else
      begin
        x := Pos(lcGraphAndQA, TempSt);
        if x > 0 then
          Delete(TempSt, x, Length(TempSt) - x + 1);
      end;
    Result := TempSt;
  end;

begin
  UpdateButtons;

  for i := 0 to lstChosen.Items.Count - 1 do
    begin
      if i = lstChosen.Items.Count - 1 then
        Exit;
      for j := i + 1 to lstChosen.Items.Count - 1 do
        begin
          TempStr := RemoveParen(lstChosen.Items[i].Caption);
          TempStr1 := RemoveParen(lstChosen.Items[j].Caption);
          if TempStr = TempStr1 then
            begin
              lstChosen.Items[j].Delete;
              Break;
            end;
        end;
    end;
end;

procedure TfrmMHA_SpecialResultsWizard.RemoveDeselectedInstrument(AInstrument: string);
var
  i                           : integer;
begin
  for i := 0 to lstChosen.Items.Count - 1 do
    if POS(AInstrument, lstChosen.Items[i].Caption) > 0 then
      begin
        lstChosen.Items[i].Delete;
        Break;
      end;

  UpdateButtons;
end;

procedure TfrmMHA_SpecialResultsWizard.treSelectChecked(
  Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  Data                        : PSelectData;
begin
  if lvInitialize then
    Exit;
  Data := Sender.GetNodeData(Node);
  if Node.CheckState = csCheckedNormal then
    begin
      lstChosen.Items.Add.Caption := Trim(Data.Caption) + lcGraphOnly;
      // Remove redundant items
      lstChosenRedundant;
     // Node.States
    end
  else
    begin
      RemoveDeselectedInstrument(Data.Caption);
    end;

end;

procedure TfrmMHA_SpecialResultsWizard.treSelectClick(Sender: TObject);
begin
  mnuInstrumentDescription.Enabled := True;
end;

procedure TfrmMHA_SpecialResultsWizard.treSelectEnter(Sender: TObject);
begin
 // Update the instructions panel.
  lblInstructions.Caption := CRLF +
    '  Select the instrument by checking its box with the mouse, or ' +
    'use the arrow keys and space-bar to select.';
end;

procedure TfrmMHA_SpecialResultsWizard.treSelectGetText(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType; var CellText: WideString);
var
  Data                        : PSelectData;
begin
  Data := Sender.GetNodeData(Node);
  CellText := Data.Caption;
end;

function TfrmMHA_SpecialResultsWizard.treSelectGetChildCount(NodeIn: string): integer;
var
  i                           : integer;
begin
  Result := 0;

  for i := 0 to treSelectHS.Count - 1 do
    begin
      if NodeIn = Piece(treSelectHS[i], '=', 1) then
        begin
          Result := NumberOfPieces(treSelectHS[i], U) - 1;
          break;
        end;
    end;
end;

procedure TfrmMHA_SpecialResultsWizard.treSelectInitChildren(
  Sender: TBaseVirtualTree; Node: PVirtualNode; var ChildCount: Cardinal);
var
  Data                        : PSelectData;
begin
  lvChildren := 0;
  Data := Sender.GetNodeData(Node);
  ChildCount := treSelectGetChildCount(Data^.Caption);

end;

function TfrmMHA_SpecialResultsWizard.treSelectGetItems(PNode: string; ChildNumber: integer): string;
var
  i, j                        : integer;
  TempStr                     : string;
begin

  Result := '';
  for i := 0 to treSelectHS.Count - 1 do
    begin
      if PNode = Piece(treSelectHS[i], '=', 1) then
        begin
          TempStr := Piece(treSelectHS[i], U, ChildNumber + 1);
          // First child has root in its name.
          j := Pos('=', TempStr);
          if j > 0 then
            Result := Copy(TempStr, j + 1, Length(TempStr))
          else
            Result := TempStr;
        end;
    end;
end;

function TfrmMHA_SpecialResultsWizard.treSelectGetHasChildren(NodeName: string): Boolean;
var
  i                           : integer;
begin
  Result := False;
  for i := 0 to treSelectHS.Count - 1 do
    if NodeName = Piece(treSelectHS[i], '=', 1) then
      begin
        Result := True;
        break;
      end;
end;


procedure TfrmMHA_SpecialResultsWizard.treSelectInitNode(
  Sender: TBaseVirtualTree; ParentNode, Node: PVirtualNode;
  var InitialStates: TVirtualNodeInitStates);

var
  DataP,
    Data                      : PSelectData;
begin
  Data := Sender.GetNodeData(Node);

  if ParentNode = nil then // Parent nodes
    begin
      Data^.Caption := treSelectGetItems('Root', lvRootNode);

      if treSelectGetHasChildren(Data^.Caption) then
        begin
          Data^.Style := 'B'; // Battery
          InitialStates := [ivsHasChildren];
        end
      else
        begin
          InitialStates := [];
          if Pos('(', Data^.Caption) > 0 then
            Data^.Style := 'I' //Incomplete
          else
            Data^.Style := 'R'; //Regular
        end;
      Inc(lvRootNode);
    end
  else
    begin
      DataP := Sender.GetNodeData(ParentNode);
      Data^.Caption := treSelectGetItems(DataP^.Caption, lvChildren);

      if treSelectGetHasChildren(Data^.Caption) then
        InitialStates := [ivsHasChildren]
      else
        begin
          InitialStates := [];
          Data^.Style := 'R'; //Regular
        end;
      Inc(lvChildren);
    end;

  Node.NodeHeight := lvNodeHeight;
  Node.CheckType := ctCheckBox;
  Sender.CheckState[Node] := csUnCheckedNormal;
end;

procedure TfrmMHA_SpecialResultsWizard.SaveHSInstrumentsToRegistry;
var
  i, x                        : Integer;
  TempStr                     : string;
begin
  gvSpecialResultsList := '';

  if lstChosen.Items.Count = 0 then
    begin
      MHA_Registry.SaveRegistry;
      Exit;
    end;

  for i := 0 to lstChosen.Items.Count - 1 do
    begin
      TempStr := lstChosen.Items[i].Caption;
      x := Pos(lcGraphOnly, lstChosen.Items[i].Caption);
      if x > 0 then
        begin
          Delete(TempStr, x, Length(TempStr) - x + 1);
          gvSpecialResultsList := gvSpecialResultsList + TempStr +
            U + 'G' + RECORD_END;
        end;

      x := Pos(lcGraphAndQA, lstChosen.Items[i].Caption);
      if x > 0 then
        begin
          Delete(TempStr, x, Length(TempStr) - x + 1);
          gvSpecialResultsList := gvSpecialResultsList + TempStr +
            U + 'B' + RECORD_END;
        end;
    end; // for i

  MHA_Registry.SaveRegistry;

end;

procedure TfrmMHA_SpecialResultsWizard.btnSaveClick(Sender: TObject);
begin
  SaveHSInstrumentsToRegistry;
  Close;
end;

procedure TfrmMHA_SpecialResultsWizard.lstChosenDblClick(Sender: TObject);
var
  x                           : Integer;
  TempStr                     : string;
begin
  TempStr := lstChosen.ItemFocused.Caption;
  x := Pos(lcGraphOnly, TempStr);

  if x > 0 then
    begin
      Delete(TempStr, x, Length(TempStr) - x + 1);
      TempStr := TempStr + lcGraphAndQA;
    end
  else
    begin
      x := Pos(lcGraphAndQA, TempStr);
      Delete(TempStr, x, Length(TempStr) - x + 1);
      TempStr := TempStr + lcGraphOnly;
    end;
// a little time is needed....
  sleep(200); // milliseconds
  lstChosen.ItemFocused.Caption := TempStr;

end;

procedure TfrmMHA_SpecialResultsWizard.lstChosenSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  UpdateButtons;
end;

procedure TfrmMHA_SpecialResultsWizard.lstChosenMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  lstChosenEnter(Sender);
end;

procedure TfrmMHA_SpecialResultsWizard.mnuInstrumentDescriptionClick(
  Sender: TObject);
var
  Data                        : PSelectData;
  aInstrument                 : TInstrument;
begin
  Data := treSelect.GetNodeData(treSelect.GetFirstSelected);

  aInstrument := TInstrument.Create;
  aInstrument.InstrumentName := Data^.Caption;

  if Instrument_OPS.LoadDescriptiveProperties(aInstrument) then
    begin
      try
        frmMHA_InstrumentDescription := TfrmMHA_InstrumentDescription.Create(nil);
        frmMHA_InstrumentDescription.ShowDescription(aInstrument);
        frmMHA_InstrumentDescription.ShowModal;
      finally
        frmMHA_InstrumentDescription.Free;
        aInstrument.Free;
      end;
    end //if instrument descriptive properties loaded
  else
    aInstrument.Free;
end;

procedure TfrmMHA_SpecialResultsWizard.mnuAboutClick(Sender: TObject);
begin
  frmMHA_About := TfrmMHA_About.Create(nil);
  try
    frmMHA_About.ShowModal;
  finally
    frmMHA_About.Free;
  end; //try
end;

procedure TfrmMHA_SpecialResultsWizard.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

procedure TfrmMHA_SpecialResultsWizard.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_SpecialResultsWizard.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
 if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;
end;

end.

