unit uMHA_ASI_List;

interface

uses
  Classes,
  Controls,
  SysUtils;

type
  TASI_List = class
  private
    FInterviewList: TStringList;
    FNumberOfInterviews: Integer;
    FLastASIDate: string;
    FSignedInterviewsExist: BOOLEAN;
    FPreviousInterviewsExist: BOOLEAN;

    procedure Clear;
    procedure SetLastASIDate;
    function SetSignedInterviewsExist: BOOLEAN;

  protected
  public
    property NumberOfInterviews: Integer read FNumberOfInterviews write FNumberOfInterviews;
    property LastASIDate: string read FLastASIDate write FLastASIDate;
    property SignedInterviewsExist: BOOLEAN read FSignedInterviewsExist write FSignedInterviewsExist;
    property PreviousInterviewsExist: BOOLEAN read FPreviousInterviewsExist write FPreviousInterviewsExist;

    destructor Destroy; override;
    constructor Create;
    function Load(ADFN: string): BOOLEAN;
    procedure LoadASIByIENS(AIENS: string);
    procedure LoadASIByIndex(AIndex: integer);

  published
  end; { TASI_List }

var
  ASI_List: TASI_List;

implementation

uses
  uMHA_Const,
  uMHA_ASI,
  uMHA_Utils,
  uMHA_ASI_IO;

destructor TASI_List.Destroy;
begin
  if ASI <> nil then
    ASI.Free;
  if FInterviewList <> nil then
    FInterviewList.Free;
  inherited Destroy;
end;

constructor TASI_List.Create;
begin
  inherited Create;
  FInterviewList := TStringList.Create;
  ASI := TASI.Create;
end;

function TASI_List.Load(ADFN: string): BOOLEAN;
begin
  Clear;

  PiecesToList(ASIReadList(ADFN), RECORD_END, FInterviewList);

  if (FInterviewList.Count = 0)
    or (FInterviewList[0] = BAD_DATA) then
    begin
      Result := False;
      FInterviewList.Clear;
    end
  else
    Result := True;

  FNumberOfInterviews := FInterviewList.Count;
  FPreviousInterviewsExist := (FNumberOfInterviews > 0);

  if FPreviousInterviewsExist then
    begin
      SetLastASIDate;
      FSignedInterviewsExist := SetSignedInterviewsExist;
    end;
end;

procedure TASI_List.LoadASIByIENS(AIENS: string);
var
  tmpValue, ADate, AInterviewType, AG12, AIsSigned,
    AOrderedBy, AOrderedByDUZ: string;
begin
  tmpValue := FInterviewList.Values[AIENS];

  ADate := Piece(tmpValue, U, 1);
  AInterviewType := Piece(tmpValue, U, 2);
  AG12 := Piece(tmpValue, U, 3);
  AIsSigned := Piece(tmpValue, U, 4);
  AOrderedBy := Piece(tmpValue, U, 5);
  AOrderedByDUZ := Piece(tmpValue, U, 6);

  ASI.Load(AIENS, ADate, AInterviewType, AG12, AIsSigned, AOrderedBy, AOrderedByDUZ);

end;


procedure TASI_List.LoadASIByIndex(AIndex: integer);
begin
  LoadASIByIENS(FInterviewList.Names[AIndex]);
end;

function TASI_List.SetSignedInterviewsExist: BOOLEAN;
var
  i: integer;
begin
  Result := False;

  for i := 0 to FNumberOfInterviews - 1 do
    if (Piece(FInterviewList[i], U, 4) = 'YES') then
      begin
        Result := True;
        Break;
      end;

end;

procedure TASI_List.SetLastASIDate;
var
  i: integer;
  tmpDate, LastDate: TDate;
  tmpDateStr: string;
begin

  //look for most recent interview
  LastDate := 0; //12/30/1899 12:00 am
  for i := 0 to FInterviewList.Count - 1 do
    begin
      tmpDateStr := Piece(Piece(FInterviewList[i], '=', 2), U, 1);

      if (tmpDateStr = '') then
        tmpDate := StrToDateTime('01/01/1900')
      else
        tmpDate := StrToDateTime(tmpDateStr);

      if (tmpDate > LastDate)
        {and (tmpDate < ASI.Date)}then
        LastDate := tmpDate;
    end; //for i...

  FLastASIDate := DateToStr(LastDate);

end;

procedure TASI_List.Clear;
begin
  FInterviewList.Clear;
  FNumberOfInterviews := 0;
  FLastASIDate := DateToStr(Date);
  FSignedInterviewsExist := False;
  FPreviousInterviewsExist := False;
end;

end.

