{
 NOTES: Datebase on the M-side is:

 601.77  Battery File:         (.01) Battery ID ^(1) Battery Name
 601.78  Battery Users File:   (.01) Batt Users ID ^(1) UserID ^(2) Battery ID
 601.781 Battery Content File: (.01) Battery Content ID ^(1) Battery ID ^(2) Sequence
     ^(3) InstrumentName

}

unit uMHA_Battery_IO;

interface

uses
  Trpcb,
  fIA_dialogs,
  Sysutils,
  Classes,
  uMHA_Battery;

function BatteryReadList(Broker: TRPCBroker): string;
function BatteryWrite(aBattery: TBattery;
  Broker: TRPCBroker): string;
function BatteryDelete(aBatteryID: string;
  Broker: TRPCBroker): string;
function BatteryRename(NewBatteryName, aBatteryID: string;
  Broker: TRPCBroker): string;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uMHA_Battery_OPS,
  uMHA_LocalFiles,
  uIA_ThisUser,
  uIA_Utils;

function BatteryReadList(Broker: TRPCBroker): string;
// Resulting list has this format for each line:
// Battery ID ^ BatteryName ^ Sequence ^ InstrumentID ^ Instrument Name ~
var
  TempID,
    LastID,
    TempStr                   : string;
  i                           : integer;
  aBattery                    : TBattery;
begin
  result := OK;
  LastID := '';

  with Broker do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ BATTC.');
      RemoteProcedure := 'YTQ BATTC';
      try
        Call;

      except
        on E: EBrokerError do
          begin
            result := 'Broker Error: ' + E.message;
            exit;
          end;
        on E: EAccessViolation do
          begin
            result := 'Access Violation: ' + E.message;
            exit;
          end;
      end; // try

      if (Results[0] <> DATA) then
        begin
          Result := 'Bad data';
          //* LocalFiles.UpdateAPILog('Call to YTQ BATTC: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
          Exit;
        end; //bad data

      //* LocalFiles.UpdateAPILog('Call to YTQ BATTC: Successful');

      // no data
      if Results.Count = 0 then
        begin
          result := '0';
          Exit;
        end;

      for i := 1 to Results.Count - 1 do
        begin
          TempStr := Results[i];
          TempID := Piece(TempStr, U, 1); // Battery ID

          if TempID = LastID then
            begin
              aBattery.InstrumentsInBattery.Add(
                Piece(TempStr, U, 3) + U + // Sequence
                Piece(TempStr, U, 4) + U + // InstrumentID
                Piece(TempStr, U, 5)); // Instrument Name
            end;

          if (TempID <> LastID) then
            begin
              if LastID <> '' then
                Battery_OPS.ListOfBatteries.Add(aBattery);
              aBattery := TBattery.Create;
              aBattery.Clear;

              aBattery.BatteryID := Piece(TempStr, U, 1);
              aBattery.BatteryName := Piece(TempStr, U, 2);
              LastID := aBattery.BatteryID;
              aBattery.InstrumentsInBattery.Add(
                Piece(TempStr, U, 3) + U + // Sequence
                Piece(TempStr, U, 4) + U + // InstrumentID
                Piece(TempStr, U, 5)); // Instrument Name
            end;

           // Add the last battery to the list.
          if i = Results.Count - 1 then
            Battery_OPS.ListOfBatteries.Add(aBattery);
        end;
    end; //with DataModule1.RPCBroker1
end;


function BatteryWrite(aBattery: TBattery;
  Broker: TRPCBroker): string;
// Write the Battery Info in three tables:
// 601.77  Battery File:         (.01) Battery ID ^(1) Battery Name
// 601.78  Battery Users File:   (.01) Batt Users ID ^(1) UserID ^(2) Battery ID
// 601.781 Battery Content File: (.01) Battery Content ID ^(1) Battery ID ^(2) Sequence
//     ^(3) InstrumentName
//
// A change to an existing battery requires that the original batteryID be removed
// and a new ID be established.
var
  i                           : integer;
  TempStr,
    TempBatteryID             : string;

begin
  result := BAD_DATA;
  TempBatteryID := aBattery.BatteryID;

  // Remove the Battery from the list of batteries. It will be replaced with a new
  // ID that will be linked to the changed list of instruments.
  if TempBatteryID <> '' then
    begin
      TempStr := BatteryDelete(TempBatteryID, Broker);
      if TempStr <> OK then
        begin
          result := 'Error record: ' + TempBatteryID + '. ' + TempStr;
          exit;
        end;
    end;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: FILEN=601.77 ' +
      //*   '.01=.01^NEW^1 ' + ' 1=' + aBattery.BatteryName);
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_BATTERIES; // MH Battery File 601.77
      Param[0].Mult['".01"'] := '.01^NEW^1';
      Param[0].Mult['"1"'] := '1^' + aBattery.BatteryName;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            result := 'Broker Error: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
        on E: EAccessViolation do
          begin
            result := 'Access Violation: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
      end; // try

      if Results[0] <> DATA then
        begin
          result := 'File 601.77 ' + results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ' + results[1]);
          exit;
        end;
      //* else
        //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Successful');

      TempBatteryID := '';
      TempBatteryID := Piece(Results[1], U, 2);
      if TempBatteryID = '' then
        begin
          result := 'File 601.77 BatteryID invalid.';
          Exit;
        end;

    //------------------------------

      for i := 0 to aBattery.InstrumentsInBattery.Count - 1 do
        begin
          ClearParameters := True;
          ClearResults := True;
          CreateContext('YS BROKER1');
          //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: FILEN=601.78 ' +
          //*   '.01=.01^NEW^1' +
          //*   ' 1= ' + TempBatteryID +
          //*   ' 2=' + inttostr(i + 1) +
          //*   ' 3=' + aBattery.InstrumentsInBattery[i]);
          RemoteProcedure := 'YTQ SAVE';

          Param[0].Mult['"FILEN"'] := FILE_BATTERY_CONTENTS;
          // MH Battery Contents File 601.78
          Param[0].Mult['".01"'] := '.01^NEW^1'; // BatteryContentID
          Param[0].Mult['"1"'] := '1^' + '`' + TempBatteryID; // BatteryID
          Param[0].Mult['"2"'] := '2^' + inttostr(i + 1); // Sequence
          Param[0].Mult['"3"'] := '3^' + aBattery.InstrumentsInBattery[i];
          // InstrumentName
          Param[0].PType := List;
          try
            Call;

          except
            on E: EBrokerError do
              begin
                result := 'Broker Error: ' + E.message;
                //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
                //* LocalFiles.UpdateAPILog('   ' + result);
                exit;
              end;
            on E: EAccessViolation do
              begin
                result := 'Access Violation: ' + E.message;
                //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
                //* LocalFiles.UpdateAPILog('   ' + result);
                exit;
              end;
          end; // try

          if Results[0] <> DATA then
            begin
              result := 'File 601.78 ' + results[1];
              //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ' + results[1]);
              exit;
            end;
          //* else
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Successful');
        end; // for i

      //------------------------------
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: FILEN=601.781 ' +
      //*   '.01=.01^NEW^1' +
      //*   ' 1=' + User.DUZ +
      //*   ' 2=' + TempBatteryID);
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_BATTERY_USERS;
          // MH Battery Users File 601.781
      Param[0].Mult['".01"'] := '.01^NEW^1'; // BatteryUserID
      Param[0].Mult['"1"'] := '1^' + '`' + User.DUZ;
      Param[0].Mult['"2"'] := '2^' + '`' + TempBatteryID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            result := 'Broker Error: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
        on E: EAccessViolation do
          begin
            result := 'Access Violation: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
      end; // try

      if Results[0] <> DATA then
        begin
          result := 'File 601.781 ' + results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ' + results[1]);
          exit;
        end;
      //* else
        //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Successful');

    end; //DataModule1.RPCBroker1

  Result := OK;
end;

function BatteryRename(NewBatteryName, aBatteryID: string;
  Broker: TRPCBroker): string;
begin
  result := OK;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: FILEN=601.77 ' +
      //*   'IEN=' + aBatteryID +
      //*   ' 1=' + NewBatteryName);
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_BATTERIES; // MH Battery File 601.77
      Param[0].Mult['"IEN"'] := aBatteryID;
      Param[0].Mult['"1"'] := '1^' + NewBatteryName;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            result := 'Broker Error: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
        on E: EAccessViolation do
          begin
            result := 'Access Violation: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
      end; // try

      if Results[0] <> DATA then
        begin
          result := 'File 601.77 ' + results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ' + results[1]);
          exit;
        end;
      //* else
        //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Successful');
    end;
end;


function BatteryDelete(aBatteryID: string;
  Broker: TRPCBroker): string;
// The battery information is stored in three files on the M-side:
//   601.77  has the BatteryID, BatteryName
//   601.78  has the BatteryContentID, BatteryID, Sequence, InstrumentID
//   601.781 has the BatteryUserID, User, BatteryID
begin
  Result := 'OK';

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ BATTDEL: ' +
      //*   'IEN=' + aBatteryID);
      RemoteProcedure := 'YTQ BATTDEL';
      Param[0].Mult['"IEN"'] := aBatteryID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            result := 'Broker Error: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ BATTDEL: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
        on E: EAccessViolation do
          begin
            result := 'Access Violation: ' + E.message;
            //* LocalFiles.UpdateAPILog('Call to YTQ BATTDEL: Error ');
            //* LocalFiles.UpdateAPILog('   ' + result);
            exit;
          end;
      end; // try

      if not (Results[0] = DATA) then
        begin
          Result := 'BATTDEL: ' + Results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ BATTDEL: Error ' + results[1]);
          Exit;
        end; //bad data
      //* else
        //* LocalFiles.UpdateAPILog('Call to YTQ BATTDEL: Successful');
    end;
end;

end.

