unit uMHA_GAF_OPS;
// Business level unit to separate form from IO calls

interface

uses
  Classes,
  Contnrs;

type

  TGAF_OPS = class
  private
    FListOfGAFs: TObjectList;
    FRatingIsDue : Boolean;
  protected
  public
    constructor Create;
    destructor Destroy; override;

    function SaveNew(aDFN, aDUZ, aDate, aGAFScore: string): string;
    procedure MarkInError(aIENS, AUserName: string);
    function Delete(aIENS: string): string;
    function SaveEdit(aIENS: string; aRating: integer): boolean;
    function GetPreviousGAFRatings(aDFN: string): string;
    function RatingsCanBeGraphed: Boolean;

    property ListOfGAFs: TObjectList read FListOfGAFs write FListOfGAFs;
    property RatingIsDue: Boolean read FRatingIsDue Write FRatingIsDue;
  published
  end; { TGAF_OPS}

var
  GAF_OPS                     : TGAF_OPS;

implementation

uses
  uMHA_GAF,
  uMHA_GAF_IO,
  uMHA_DM;

constructor TGAF_OPS.Create;
begin
  inherited Create;
  FListOfGAFs := TObjectList.Create;
end;

destructor TGAF_OPS.Destroy;
begin
  if ListOfGAFs <> nil then
    ListOfGAFs.Free;
  inherited Destroy;
end;


function TGAF_OPS.RatingsCanBeGraphed: boolean;
 //  To be able to graph ratings, there must be at least 2 ratings not
 //  marked in error
var
  InError, i                  : integer;
begin
  Result := False;

  if (FListOfGAFs.Count < 2) then
    Exit;

  InError := 0;

  for i := 0 to FListOfGAFs.Count - 1 do
    if TGAF(FListOfGAFs[i]).EnteredInError then
      Inc(InError);

  if (FListOfGAFs.Count - InError > 1) then
    Result := True;

end;

function TGAF_OPS.SaveNew(aDFN, aDUZ, aDate, aGAFScore: string): string;
begin
  result := WriteNewGAFRating(aDFN, aDUZ, aDate, aGAFScore, DataModule1.RPCBroker1);
end;

procedure TGAF_OPS.MarkInError(aIENS, AUserName: string);
begin
  WriteGAFEnteredInError(aIENS, AUserName, DataModule1.RPCBroker1);
end;

function TGAF_OPS.Delete(aIENS: string): string;
begin
  Result := DeleteGAF(aIENS, DataModule1.RPCBroker1);
end;

function TGAF_OPS.SaveEdit(aIENS: string; aRating: integer): boolean;
begin
  Result := WriteEditedGAFRating(aIENS, aRating, DataModule1.RPCBroker1);
end;

function TGAF_OPS.GetPreviousGAFRatings(aDFN: string): string;
begin
  Result := ReadPreviousGAFRatings(aDFN, DataModule1.RPCBroker1);
end;

end.

