unit uMHA_InstrumentResults_IO;

interface

uses
  SysUtils,
  Trpcb;

// Gets scores, regardless of whether they are stored in the new or old files
function ReadScores(AnswersAreInLegacyFile: BOOLEAN; AAdministrationIENS,
  AInstrumentName, ADFN, AFMDate: string; Broker: TRPCBroker): string;

// This works for both old and new administrations
// It returns a list of previous administrations, not scores or scales
function ReadCompleteResults(aDFN: string; Broker: TRPCBroker): string;

function ReadScaleGroupsAndScales(WhichInstrument: string; Broker: TRPCBroker): string;

implementation

uses
  uIA_Const,
  fIA_dialogs,
  uIA_Utils,
  uIA_ThisUser,
  uIA_Scale,
  uIA_ScaleGroup,
  uIA_Scales_IO,
  uMHA_LocalFiles,
  uMHA_AResult,
  uMHA_Result_OPS;


function ReadCompleteResults(aDFN: string; Broker: TRPCBroker): string;
//   1         2                  3           4             5    6   7 8 9 10 11   12 1314
//100764^A Report-Script Test^3050317.13525^3050317.155349^6627^6627^N^11^^N^100065^^N^2~
//100750^A Pretty Good Test^3050314.063229^3050314.083402^6627^6627^N^11^^N^100038^^N^12~

//1. Administration ID
//2. Instrument Name
//3. Date Given
//4. Date Saved
//5. Orderer
//6. Admin.By
//7. Signed
//8. # Answers
//9. Exempt/ R_Privl
//10. Is Legacy
//11. INSTRUMENT id 601.71
//12. Test IENS 601  legacy value
//13. WriteFullText
//14. Location IEN
//15. DaysToRestart

var
  i                           : integer;
  TempStr                     : string;
  TempIR                      : TAResult;

  function StrToIRKind(AKind: string): TIRKind;
  begin
    Result := irNew; //this will be the default

    if (AKind = 'ASI') then
      Result := irASI;
    if (AKind = 'Y') then
      Result := irLegacy;
  end;

begin
  Result := OK;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: DFN=' + aDFN);
      RemoteProcedure := 'YTQ ADMINS BOTH';
      Param[0].Mult['"DFN"'] := ADFN;
      Param[0].Mult['"COMPLETE"'] := 'Y';
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results[0] <> DATA then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Error ' + Results[1]);
          Result := Results[1];
          Exit;
        end;

      //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Successful');
      Result_OPS.ListOfResults.Clear;

        // Ignore the first two results
      for i := 2 to Results.Count - 1 do
        begin
          TempStr := Results[i] + U;

          if NumberOfPieces(TempStr, U) > 10 then // This weeds out the GAF
            begin
              TempIR := TAResult.Create;

              TempIR.AdministrationIENS := Piece(TempStr, U, 1);
              TempIR.InstrumentName := Piece(TempStr, U, 2);

              TempIR.DateGiven := FMDateStrToDateStr(Piece(TempStr, U, 3));

              TempIR.DateSaved := FMDateStrToDateStr(Piece(TempStr, U, 4));
              TempIR.OrderedBy := Piece(TempStr, U, 5);
              TempIR.AdminBy := Piece(TempStr, U, 6);

              //TempIR.Signed := StrToBoolean(Piece(TempStr, U, 7));
              //TempIR.NumberAnswered := Piece(TempStr, U, 8);
              TempIR.Privledge := Piece(TempStr, U, 9);
              TempIR.Kind := StrToIRKind(Piece(TempStr, U, 10));

              if (TempIR.Kind = irLegacy) then
                TempIR.TestIENS := StrToIntDef(Piece(TempStr, U, 12), -1)
              else
                TempIR.TestIENS := StrToIntDef(Piece(TempStr, U, 11), -1);

              TempIR.WriteFullText := StrToBool(Piece(TempStr, U, 13));
              TempIR.Location := Piece(TempStr, U, 14);
              TempIR.UserHasPermissionToView := False; // default

              if TempIR.Privledge = '' then
                TempIR.UserHasPermissionToView := True
              else
                if Pos(TempIR.Privledge, ThisUser.AccessKeys) > 0 then
                  TempIR.UserHasPermissionToView := True;

              TempIR.AnswersAreInLegacyFile :=
                (StrToIntDef(TempIR.AdministrationIENS, -1) = -1);

              Result_OPS.ListOfResults.Add(TempIR);

              TempStr := '';
            end;
        end;
    end; //with DataModule1
end;

function ReadScores(AnswersAreInLegacyFile: BOOLEAN; AAdministrationIENS,
  AInstrumentName, ADFN, AFMDate: string; Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  Result := '';

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ SCORE ADMIN';

      if AnswersAreInLegacyFile then
        begin
          Param[0].Mult['"CODE"'] := AInstrumentName;
          Param[0].Mult['"DFN"'] := ADFN;
          Param[0].Mult['"ADATE"'] := AFMDate;
          //* LocalFiles.UpdateAPILog('Call to YTQ SCORE ADMIN: ' +
          //*   ' CODE=' + AInstrumentName +
          //*   ' DFN=' + ADFN +
          //*   ' ADATE=' + AFMDate);
        end
      else
        begin
          if (AAdministrationIENS = '') then
            exit
          else
            begin
              //* LocalFiles.UpdateAPILog('Call to YTQ SCORE ADMIN: ' +
              //*   ' AD=' + AAdministrationIENS);
              Param[0].Mult['"AD"'] := AAdministrationIENS;
            end;
        end;

      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SCORE ADMIN: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SCORE ADMIN: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      // This will return '[ERROR]' when there are no scales, or scores, for this
      // instrument: Not really an error.
      if (Results[0] <> DATA) then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ SCORE ADMIN: Error ' + Results[0]);
          Result := '';
        end
      else
        begin
          // Format is: ScaleName=ScaleScore
          for i := 1 to Results.Count - 1 do
            Result := Result + Results[i] + RECORD_END;

          Result := DropTrailer(Result);
          //* LocalFiles.UpdateAPILog('Call to YTQ SCORE ADMIN: Successful');
        end;
    end;
end;


function ReadScaleGroupsAndScales(WhichInstrument: string; Broker: TRPCBroker): string;
var
  NameStr,
    TempStr, SubStr, SubSubStr: string;
  i                           : Integer;
  AScale                      : TScale;
  AGroup                      : TScaleGroup;

begin
  Result := OK;

  Result_OPS.ListOfScaleGroups.Clear;
  Result_OPS.ListOfScales.Clear;

  TempStr := GetScalesFromVista(WhichInstrument, Broker);

  if TempStr = '' then
    Exit;

  if TempStr = BAD_DATA then
    begin
      Result := BAD_DATA;
      Exit;
    end;

  for i := 1 to NumberOfPieces(TempStr, RECORD_END) do
    begin
      SubStr := Piece(TempStr, RECORD_END, i);
      NameStr := Piece(SubStr, EQ, 1);
      if Pos('Scale', NameStr) > 0 then
        begin
          AScale := TScale.Create;
          SubSubStr := Piece(SubStr, EQ, 2);
          AScale.IEN := Piece(SubSubStr, U, 1);
          AScale.GroupIEN := Piece(SubSubStr, U, 2);
          AScale.Sequence := StrToIntDef(Piece(SubSubStr, U, 3), -1);
          AScale.ScaleName := Piece(SubSubStr, U, 4);
          AScale.XLabel := Piece(SubSubStr, U, 5);

          Result_OPS.ListOfScales.Add(AScale);
        end;

      if Pos('Group', NameStr) > 0 then
        begin
    //  GroupX= [1] SCALEGROUP ID^
    //          [2] INSTRUMENT NAME^
    //          [3] SCALEGROUP NAME^
    //          [4] GROUP SEQUENCE^
    //          [5-11] ORDINATE TITLE^MIN^INCREMENT^MAX^GRID1^GRID2^GRID3

          AGroup := TScaleGroup.Create;
          SubSubStr := Piece(SubStr, EQ, 2);
          AGroup.IEN := Piece(SubSubStr, U, 1);
          // 2nd piece is instrument name
          AGroup.GroupName := Piece(SubSubStr, U, 3);
          AGroup.Sequence := StrToIntDef(Piece(SubSubStr, U, 4), -1);
          AGroup.OrdinateTitle := Piece(SubSubStr, U, 5);
          AGroup.OrdinateMin := StrToIntDef(Piece(SubSubStr, U, 6), -1);
          AGroup.OrdinateIncrement := StrToIntDef(Piece(SubSubStr, U, 7), -1);
          AGroup.OrdinateMax := StrToIntDef(Piece(SubSubStr, U, 8), -1);
          AGroup.Grid1Ordinate := StrToIntDef(Piece(SubSubStr, U, 9), -1);
          AGroup.Grid2Ordinate := StrToIntDef(Piece(SubSubStr, U, 10), -1);
          AGroup.Grid3Ordinate := StrToIntDef(Piece(SubSubStr, U, 11), -1);

          Result_OPS.ListOfScaleGroups.Add(AGroup);
        end;
    end;
end;

end.

