unit uMHA_Instrument_IO;
{* functions unique to Instrument object in MHA. Many are shared in
   uIA_Instrument_IO.

*}

interface

uses
  Trpcb,
  SysUtils,
  classes,
  uIA_Instrument;

function ReadListOfAvailableInstruments(UserKeys: string;
  Broker: TRPCBroker): string;

  function ReadInstrumentDetailsVista(WhichInstrument: string;
  Broker: TRPCBroker): string;

implementation

uses
  uIA_Const,
  uMHA_Const,
  fIA_dialogs,
  uIA_Utils,
  uIA_Question,
  uIA_AvailableInstrument,
  uMHA_Instrument_OPS,
  uMHA_LocalFiles;

function ReadInstrumentDetailsVista(WhichInstrument: string;
  Broker: TRPCBroker): string;
  
// Yields string with DaysToRestart: integer (Range is -1 to 99
//  where -1 is no limit, 0 is not allowed) ^
// GenerateProgNote: Boolean ^
// ProgNoteTitle: string ^
// ConsultTitle: string

// Result of YTQ TSLIST1 is"
// [DATA]
// IEN=5
// NAME=AUDC
// PRINT TITLE=Alcohol Use Disorders Identification Test Consumption
// VERSION=
// AUTHOR=Bradley; McDonell; Bush; Kivlahan; Diehr; Fihn
// PUBLISHER=Alcoholism:Clinical & Experimental Research
// PUBLICATION DATE=1998
// REFERENCE=Alcoholism: Clinical & Experimental Research. 22(8):1842, November 1998
// A PRIVILEGE=
// R PRIVILEGE=
// OPERATIONAL=Yes
// HAS BEEN OPERATIONAL=Yes
// REQUIRES LICENSE=No
// PURPOSE=Alcohol Screening.  3 items from WHO AUDIT test dealing with consumption of alcohol.
// NORM SAMPLE=
// TARGET POPULATION=
// ENTERED BY=A FINKELSTEIN
// ENTRY DATE=FEB 26, 2004
// LAST EDITED BY=FINKELSTEIN,ALLAN
// LAST EDIT DATE=3090409
// IS NATIONAL TEST=Yes
// LICENSE CURRENT=No
// COPYRIGHT TEXT=
// REQUIRES SIGNATURE=No
// IS LEGACY=No
// SUBMIT TO NATIONAL DB=Yes
// IS COPYRIGHTED=No
// WRITE FULL TEXT=Yes
// DAYS TO RESTART=2
// GENERATE PNOTE=Yes
// TIU TITLE=MENTAL HEALTH DIAGNOSTIC STUDY NOTE
// Consult Title=MENTAL HELATH CONSULT NOTE

begin
  Result := BAD_DATA;

  with Broker do
    begin
      //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST1: CODE=' + WhichInstrument);

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ TSLIST1';
      Param[0].Mult['"CODE"'] := WhichInstrument;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST1: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST1: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = DATA) then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST1: Successful');
          Result := Piece(Results[29], EQ, 2) + U +
            Piece(Results[30], EQ, 2) + U +
            Piece(Results[31], EQ, 2) + U +
            Piece(Results[32], EQ, 2);
        end;
      //* else
        //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST1: Error ' + results[0]);
    end;
end;


function ReadListOfAvailableInstruments(UserKeys: string;
  Broker: TRPCBroker): string;

  function ParseSrName(AName: string): string;
  begin
    AName := Piece(AName, '.', 1);
    Result := Piece(AName, FILE_NAME_DELIMITER, 1) + '=' + Piece(AName,
      FILE_NAME_DELIMITER, 2) + '^YES';
  end;

var
  TempStr, TempStr1, TempStr2,
    Temp_A_Privilege          : string;
  i                           : integer;
  sr                          : TSearchRec;
  aAvail                      : TAvailableInstrument;
begin
  Result := OK;
  Instrument_OPS.ListOfInstruments.Clear;

  case DataBaseMode of
    dbVista:
      begin
        with Broker do
          begin
            ClearParameters := True;
            ClearResults := True;
            CreateContext('YS BROKER1');
            //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST');
            RemoteProcedure := 'YTQ TSLIST';
            try
              Call;

            except
              on E: EBrokerError do
                begin
                  //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST: Error ');
                  //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                  exit;
                end;
              on E: EAccessViolation do
                begin
                  //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST: Error ');
                  //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                  exit;
                end;
            end; // try

            if Results[0] <> DATA then
              begin
               // Result := BAD_DATA;
                Result := 'ERROR ' + Results[1];
                //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST: Error ' + Results[1]);
                Exit;
              end; //if bad data

            for i := 1 to Results.Count - 1 do
              begin
                aAvail := TAvailableInstrument.Create;
                with aAvail do
                  begin
                    TempStr := Results[i];
                    AvailInstrumentName := Piece(TempStr, EQ, 1);
                    TempStr := Piece(TempStr, EQ, 2);
                    LastEditDate := Piece(TempStr, U, 1);
                    IsOperational := 'U'; // Default
                    TempStr1 := Piece(TempStr, U, 2); // Could be Y, N or U
                    if Length(TempStr1) <> 0 then
                      if TempStr1[1] in ['Y', 'N', 'U'] then
                        IsOperational := TempStr1[1];
                    RequiresLicense := StrToBool(Piece(TempStr, U, 3));
                    LicenseIsCurrent := StrToBool(Piece(TempStr, U, 4));
                    IsLegacy := StrToBool(Piece(TempStr, U, 5));
                    AvailInstrumentID := Piece(TempStr, U, 6);
                    R_Privilege := Piece(TempStr, U, 7);
                    IsNationalInstrument := StrToBool(Piece(TempStr, U, 8));
                    HasBeenOperational := StrToBool(Piece(TempStr, U, 9));
                    Temp_A_Privilege := Piece(TempStr, U, 10);

                    if RequiresLicense then
                      CopyrightStatusIsOK := LicenseIsCurrent
                    else
                      CopyrightStatusIsOK := True;

                    CanBeAdministered := (IsOperational = 'Y') and CopyrightStatusIsOK;

                    if aAvail.AvailInstrumentName = 'WAS' then
                      begin
                        gvUpdateSeries := R_Privilege;
                        R_Privilege := '';
                      end;

                    // Check for permission to administer
                    if Temp_A_Privilege <> '' then
                      if Pos(Temp_A_Privilege, UserKeys) = 0 then
                        CanBeAdministered := False;
                  end; // with aAvail

                if aAvail.CanBeAdministered then
                  Instrument_OPS.ListOfInstruments.Add(aAvail);
              end;

            //* LocalFiles.UpdateAPILog('Call to YTQ TSLIST: Successful');
          end; //with DataModule1.RPCBroker1
      end; //dbVista

    dbOffLine:
      begin
        if (FindFirst(LocalFiles.InstrumentFilesPath + '\*.' + RESOURCE_FILE_EXT, faAnyFile, sr)
          = 0) then
          begin
            TempStr := ParseSrName(sr.Name) + RECORD_END;
            while (FindNext(sr) = 0) do
              TempStr := TempStr + ParseSrName(sr.Name) + RECORD_END;
            FindClose(sr);
          end
        else
          begin
            FindClose(sr);
            Result := BAD_DATA;
            Exit;
          end; //if bad data

        if Length(TempStr) > 1 then
        // Remove last delimiter
          Delete(TempStr, Length(TempStr), 1)
        else
          Exit;

        for i := 1 to NumberOfPieces(TempStr, RECORD_END) do
          begin
            aAvail := TAvailableInstrument.Create;
            with aAvail do
              begin
                TempStr1 := Piece(TempStr, RECORD_END, i);
                AvailInstrumentName := Piece(TempStr1, EQ, 1);
                TempStr2 := Piece(TempStr, EQ, 2);
                LastEditDate := Piece(TempStr2, U, 1);
                IsOperational := 'U'; // Default
                CanBeAdministered := True;
              end; // with aAvail

            Instrument_OPS.ListOfInstruments.Add(aAvail);
          end;
      end; //dbOffLine
  end; //case DataBaseMode.....
end;


end.

