unit uMHA_OffLine_OPS;

interface

uses
  SysUtils,
  Windows,
  fIA_dialogs,
  Contnrs,
  Classes;

type
  TOffLine_OPS = class
  private
    FListOfOffLineFiles: TObjectList;
    FListOfOffLinePatients: TObjectList;
    FNextOffLineDFN: Integer;
  protected
  public
    constructor Create;
    destructor Destroy; override;

    procedure LoadOffLineFiles;
    procedure LoadOffLinePatients;
    procedure SaveOffLinePatients;

    property ListOfOffLineFiles: TObjectList read FListOfOffLineFiles write FListOfOffLineFiles;
    property ListOfOffLinePatients: TObjectList read FListOfOffLinePatients write FListOfOffLinePatients;
    property NextOffLineDFN: Integer read FNextOffLineDFN write FNextOffLineDFN;
  published
  end; { TOffLine_OPS }

var
  OffLine_OPS                 : TOffLine_OPS;

implementation

uses
  uIA_Utils,
  uIA_Const,
  uMHA_LocalFiles,
  uMHA_Administration,
  uIA_Instrument,
  uIA_Question,
  uIA_Answer,
  uIA_Patient,
  uMHA_OffLineFile,
  uMHA_Administration_OPS,
  uMHA_UploadResults_IO,
  uMHA_OffLinePatient_IO;


constructor TOffLine_OPS.Create;
begin
  inherited Create;
  FListOfOffLineFiles := TObjectList.Create;
  FListOfOffLinePatients := TObjectList.Create;
end;


destructor TOffLine_OPS.Destroy;
begin
  ListOfOffLineFiles.Free;
  ListOfOffLinePatients.Free;
  inherited Destroy;
end;


procedure TOffLine_OPS.LoadOffLineFiles;
begin
  FListOfOffLineFiles.Clear;
  ReadOffLineAdministrations;
end;

procedure TOffLine_OPS.LoadOffLinePatients;
var
  TempStr,
    TempStr1                  : string;
  i,
    TempDFN,
    MaxDFN                    : Integer;
  aPatient                    : TPatient;
begin
  FListOfOffLinePatients.Clear;
  FNextOffLineDFN := -1;
  MaxDFN := 0;
  TempStr := OffLinePatientsRead;

  if (TempStr = BAD_DATA) or (TempStr = '') then
    Exit;

  for i := 1 to NumberOfPieces(TempStr, RECORD_END) do
    begin
      aPatient := TPatient.Create;
      with aPatient do
        begin
          TempStr1 := Piece(TempStr, RECORD_END, i);
          Name := Piece(TempStr1, EQ, 1);
          TempStr1 := Piece(TempStr1, EQ, 2);
          SSN := Piece(TempStr1, U, 1);
          DOB := Piece(TempStr1, U, 2);
          Sex := Piece(TempStr1, U, 3);
          DFN := Piece(TempStr1, U, 4);
          TempDFN := StrToIntDef(DFN, 0);

          if Name <> '' then
            begin
              FListOfOffLinePatients.Add(aPatient);
              if TempDFN > MaxDFN then
                MaxDFN := TempDFN;
            end; // Name
        end; // with patient
    end; // for i
  FNextOffLineDFN := MaxDFN + 1;
end;


procedure TOffLine_OPS.SaveOffLinePatients;
var
  i                           : Integer;
  TempStr                     : string;
begin
  if FListOfOffLinePatients.Count = 0 then
    Exit;

  for i := 0 to FListOfOffLinePatients.Count - 1 do
    begin
      with TPatient(FListOfOffLinePatients[i]) do
        begin
          TempStr := TempStr +
            Name + EQ +
            SSN + U +
            DOB + U +
            Sex + U +
            DFN + RECORD_END;
        end;
    end;

  OffLinePatientsWrite(TempStr);
end;

initialization
  OffLine_OPS := TOffLine_OPS.Create;

finalization
  OffLine_OPS.Free;

end.

