unit uMHA_UploadResults_IO;

interface

uses
  SysUtils,
  fIA_dialogs,
  TRPCb,
  DateUtils,
  Classes;

procedure ReadOffLineAdministrations;
function PatientSearch(ASearchString: string; MaximumResultSet: Word;
  Broker: TRPCBroker): string;
procedure DeleteLocalAdministrationAndAnswerFile(aDFN,
  aDateTimeGiven, aInstrumentName, AnAdministrationID: string);

implementation

uses
  Diaccess,
  uMHA_LocalFiles,
  uMHA_OffLineFile,
  uMHA_UploadResults_OPS,
  uIA_Utils,
  uIA_Const;


procedure ReadOffLineAdministrations;
// Populate ListOfOffLineFiles. These are both
// completed and incompleted administrations (within the allowable period
// to be completed).
var
  sr                          : TSearchRec;
  TempList                    : TStringList;
  RestartInt                  : Integer;
  TempIsComplete              : Boolean;
  TempDate                    : TDateTime;
  TempOffLine                 : TOffLineFile;

  procedure AddThisFile(aFullFileName: string);
  begin
    TempList.Clear;
    TempList.LoadFromFile(aFullFileName);
    TempIsComplete := StrToBool(TempList.Values['IsComplete']);

    if TempIsComplete then
      begin
        TempOffLine := TOffLineFile.Create;

        with TempOffLine do
          begin
            NameOfFile := aFullFileName;
            OffLinePtID := Piece(ExtractFileName(NameOfFile), FILE_NAME_DELIMITER, 1);
            OffLineAdminID := TempList.Values['AdministrationID']; // local use only
            InstrumentName := TempList.Values['InstrumentName'];
            DateGiven := TempList.Values['DateGiven'];
            DateSaved := TempList.Values['DateSaved'];
            Location := TempList.Values['Location'];
            Signed := StrToBool(TempList.Values['Signed']);
            IsComplete := StrToBool(TempList.Values['IsComplete']);
            NumberOfQuestionsAnswered := StrToIntDef(TempList.Values['NumberOfQuestionsAnswered'], 0);
            DaysToRestart := StrToIntDef(TempList.Values['DaysToRestart'], 2);
          end;
        UploadResults_OPS.ListOfOffLineFiles.Add(TempOffLine);
      end

    else
      begin
        // To avoid bad data (e.g., Victor Rabbit in L1
        // DateGiven format is 09/18/2007 06:36:53 AM
        if TempList.Values['DateGiven'] <> '' then
          begin
          // We only want the complete instruments for this user and the
          // fresh incompletes.  The duration will be fine tuned below.

            TempDate := StrToDateTime(TempList.Values['DateGiven']);

            RestartInt := StrToIntDef(TempList.Values['DaysToRestart'], 2);
            if RestartInt = 2 then
              begin
                // Adjust the number for the weekend;
                if DayOfWeek(TempDate) = 6 {Friday} then
                  RestartInt := RestartInt + 2;
                if DayOfWeek(TempDate) = 7 {Saturday} then
                  RestartInt := RestartInt + 1;
              end;

            if (DaysBetween(Now, TempDate) < RestartInt) then
              begin
                TempOffLine := TOffLineFile.Create;

                with TempOffLine do
                  begin
                    NameOfFile := aFullFileName;
                    OffLinePtID := Piece(ExtractFileName(NameOfFile), FILE_NAME_DELIMITER, 1);
                    OffLineAdminID := TempList.Values['AdministrationID']; // local use only
                    InstrumentName := TempList.Values['InstrumentName'];
                    DateGiven := TempList.Values['DateGiven'];
                    DateSaved := TempList.Values['DateSaved'];
                    Location := TempList.Values['Location'];
                    Signed := StrToBool(TempList.Values['Signed']);
                    IsComplete := StrToBool(TempList.Values['IsComplete']);
                    NumberOfQuestionsAnswered := StrToIntDef(TempList.Values['NumberOfQuestionsAnswered'], 0);
                    DaysToRestart := StrToIntDef(TempList.Values['DaysToRestart'], 2);
                  end;
                UploadResults_OPS.ListOfOffLineFiles.Add(TempOffLine);
              end
            else
              // Remove incomplete off-line administration file that is too old.
              LocalFiles.CleanupDatedOfflineFiles(aFullFileName);
          end; // TempDate
      end; // Incomplete
  end;


begin

  if SysUtils.FindFirst(LocalFiles.AnswerFilesPath + '\' + '*.' + ADMIN_FILE_EXT, faAnyFile, sr) = 0 then
    begin

      TempList := TStringList.Create;
      AddThisFile(LocalFiles.AnswerFilesPath + '\' + sr.Name);
    end
  else
    begin
      SysUtils.FindClose(sr);
      Exit;
    end;

  while (SysUtils.FindNext(sr) = 0) do
    AddThisFile(LocalFiles.AnswerFilesPath + '\' + sr.Name);

  SysUtils.FindClose(sr);
  TempList.Free;
end;

function PatientSearch(ASearchString: string; MaximumResultSet: Word;
  Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ PATIENT LOOKUP';
      Param[0].Mult['"VALUE"'] := ASearchString;
      Param[0].Mult['"NUMBER"'] := IntToStr(MaximumResultSet);
      Param[0].PType := List;

      Call;

      if (Results.Count > 99) then
        begin
          result := 'TO MANY RESULTS';
          exit;
        end;

    // First piece is COUNT of returned records.
    //'[DATA]^1'#$D#$A'238^MOON,STRUCK^2/14/1972^123-45-6789^Sex'#$D#$A
      if (Results.Count = 0) then
        Result := BAD_DATA
      else
        begin
          for i := 1 to Results.Count - 1 do
            Result := Result + Results[i] + RECORD_END;
          Result := DropTrailer(Result);
        end;
    end;

end;

procedure DeleteLocalAdministrationAndAnswerFile(aDFN,
  aDateTimeGiven, aInstrumentName, AnAdministrationID: string);
begin
  if FileExists(LocalFiles.AnswerFileName(aDFN, AnAdministrationID)) then
    DeleteFile(LocalFiles.AnswerFilename(aDFN, AnAdministrationID));

  if FileExists(LocalFiles.BuildAdministrationFileName(aDFN,
    aDateTimeGiven, aInstrumentName)) then
    DeleteFile(LocalFiles.BuildAdministrationFileName(aDFN,
      aDateTimeGiven, aInstrumentName));
end;

end.

