unit uMHA_AUX_AUDC;

interface

uses
  SysUtils,
  fIA_dialogs,
  uMHA_AUX_Const,
  uMHA_AUX_Utils;

function ProgressNote_AUDC(
  PatientSex,
  ScoreString: string): string;

function ScoreAnswers_AUDC(
  AnswerString: string): string;

implementation


function ScoreAnswers_AUDC(
  AnswerString: string): string;

var
  Score,
    Count, i                  : Integer;

  TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330^234~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1330' is ChoiceID (not used here)
  // '234' is AnswerID (not used here)

  // AUDC scoring is straight forward for items 1 & 3. For question 2,
  // both the 1st and 2nd question are scored zero.

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'AUDC' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  Score := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // ignore (Blank) answers
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = '0') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          //
        end // Skipped questions
      else
        // Tally the raw sum
        if i = 2 then // Second Question
          Score := Score + StrToIntDef(TempAnswer, 0) - 1
        else
          Score := Score + StrToIntDef(TempAnswer, 0);
    end; // for i

  Result := 'Score^^' + IntToStr(Score) + RECORD_END;
end;


function ProgressNote_AUDC(
  PatientSex,
  ScoreString: string): string;
// The header, footer and questions and answers are handled else where; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  AUDC_Score                  : string;
begin
  Result := '';

  if (ScoreString = '') then
    Exit;


  AUDC_Score := Piece(ScoreString, U, 3);
  Delete(AUDC_Score, Length(AUDC_Score), 1);

  if (PatientSex = 'F') then
    begin
      // patch 105, changed >3 to >=3 llh
      if (StrToIntDef(AUDC_Score, 0) >= 3) then
        Result := Result + Indent +
          'Score: ' + AUDC_Score + ' points, which is a positive result. ' + #13
      else
        Result := Result + Indent +
          'Score: ' + AUDC_Score + ' points, which is a negative result.' + #13;
    end
  else // Male
    begin
      // patch 105, changed > 4 to >= 4 llh
      if (StrToIntDef(AUDC_Score, 0) >= 4) then
        Result := Result + Indent +
          'Score: ' + AUDC_Score + ' points, which is a positive result.' + #13
      else
        Result := Result + Indent +
          'Score: ' + AUDC_Score + ' points, which is a negative result.' + #13;
    end;
end;

end.

