unit uMHA_AUX_BAI;
// Beck Anxiety Inventory
// 21 Questions, four possible answers:
//   None, Mild, Moderate and Severe

interface

uses
  SysUtils,
  fIA_dialogs,
  uMHA_AUX_Const,
  uMHA_AUX_Utils;

function ProgressNote_BAI(
  ScoreString: string): string;

function ScoreAnswers_BAI(
  AnswerString: string): string;

implementation


function ScoreAnswers_BAI(
  AnswerString: string): string;

var
  Score, Ans,
    Skipped,
    Count, i                  : Integer;

  TempResult,
    TempAnswer,
    ResultString              : string;

begin
  // AnswerString Format:
  //      83=1^1330^234~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1330' is ChoiceID (not used here)
  // '234' is AnswerID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'BAI' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  Skipped := 0;
  ResultString := '';

  if Count < 1 then
    Exit;

  Score := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // ignore (Blank) answers
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          Inc(Skipped);
          ResultString := ResultString + IntToStr(i) + '. Not answered.' + U;
        end // Skipped questions
      else
        begin
          Ans := StrToIntDef(TempAnswer, 0);
          Score := Score + Ans;
          if Ans = 0 then
            ResultString := ResultString + IntToStr(i) + '. Not at all (0 point)' + U;
          if Ans = 1 then
            ResultString := ResultString + IntToStr(i) + '. Mildly (1 point)' + U;
          if Ans = 2 then
            ResultString := ResultString + IntToStr(i) + '. Moderately (2 points)' + U;
          if Ans = 3 then
            ResultString := ResultString + IntToStr(i) + '. Severely (3 points)' + U;
        end;
    end; // for i

  if (Skipped > 5) then
    Score := -1;

  Result := 'Beck Anxiety Inventory^^' + IntToStr(Score) + U + ResultString + RECORD_END;

end;


function ProgressNote_BAI(
  ScoreString: string): string;
// The header, footer and questions and answers are handled else where; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = ' ';
var
  BAI_Score                   : string;
  Count, i,
    Score                     : integer;
begin
  Result := '';

  if (ScoreString = '') then
    Exit;

  BAI_Score := Piece(ScoreString, U, 3);

  Score := StrToIntDef(BAI_Score, -1);

  if (Score < 22) then
    Result := Result + Indent +
      'BAI Score: ' + BAI_Score + ' indicates low anxiety. The overall range is 0 to 63 with low anxiety between 0 - 21.' + #13;

  if (Score = -1) then
    begin
      Result := '';
      Result := Result + Indent +
        'BAI score could not be determined.';
    end;

  if (Score < 36) and (Score > 21) then
    Result := Result + Indent +
      'BAI Score: ' + BAI_Score + ' indicates moderate anxiety. The overall range is 0 to 63 with moderate anxiety between 22 - 35.' + #13;

  if (Score > 35) then
    Result := Result + Indent +
      'BAI Score: ' + BAI_Score + ' indicates severe anxiety. The overall range is 0 to 63 with severe anxiety above 36.' + #13;

  Result := Result + #13 + Indent +
    'Questions and answers ' + #13 + #13;

  Count := NumberOfPieces(ScoreString, U);

  for i := 4 to Count do
    begin
      if Piece(ScoreString, U, i) <> RECORD_END then
        Result := Result + Indent + Piece(ScoreString, U, i) + #13;
    end;

end;

end.

