unit uMHA_AUX_BHS;
// Beck Hopelessness Scale
// 20 True False questions


interface

uses
  SysUtils,
  fIA_dialogs,
  uMHA_AUX_Const,
  uMHA_AUX_Utils;

function ProgressNote_BHS(
  ScoreString: string): string;

function ScoreAnswers_BHS(
  AnswerString: string): string;

implementation


function ScoreAnswers_BHS(
  AnswerString: string): string;

var
  Score, Ans,
    Future, Motivation, Expectation,
    Skipped,
    Count, i                  : Integer;

  TempResult,
    TempAnswer,
    ResultString              : string;

begin
  // AnswerString Format:
  //      83=1^1330^234~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1330' is ChoiceID (not used here)
  // '234' is AnswerID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'BHS' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  Skipped := 0;
  Future := 0;
  Motivation := 0;
  Expectation := 0;
  ResultString := '';

  if Count < 1 then
    Exit;

  Score := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // ignore (Blank) answers
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          Inc(Skipped);
          ResultString := ResultString + IntToStr(i) + '. Not answered.' + U;
        end // Skipped questions
      else
        begin
          Ans := StrToIntDef(TempAnswer, 0);

          // Sub-scale: Feeling of Future
          if (i = 1) or (i = 6) or (i = 13) or (i = 15) or
            (i = 19) then
            if Ans = 1 then // False
              Future := Future + 1;

          // Sub-scale: Loss of Motivation
          if (i = 2) or (i = 9) or (i = 11) or (i = 12) or
            (i = 16) or (i = 17) or (i = 20) then
            if Ans = 0 then // True
              Motivation := Motivation + 1;

          if (i = 3) then
            if Ans = 1 then // False
              Motivation := Motivation + 1;

          // Sub-scale: Future Expectation
          if (i = 4) or (i = 7) or (i = 14) or (i = 18) then
            if Ans = 0 then // True
              Expectation := Expectation + 1;
          if (i = 8) then
            if Ans = 1 then // False
              Expectation := Expectation + 1;

          // Total
          if (i = 1) or (i = 3) or (i = 5) or (i = 6) or
            (i = 8) or (i = 10) or (i = 13) or (i = 15) or
            (i = 19) then
            begin
              if Ans = 0 then // True
                ResultString := ResultString + IntToStr(i) + '. True (0 point)' + U;
              if Ans = 1 then // False
                begin
                  Score := Score + 1;
                  ResultString := ResultString + IntToStr(i) + '. False (1 point)' + U;
                end;
            end;

          if (i = 2) or (i = 4) or (i = 7) or (i = 9) or
            (i = 11) or (i = 12) or (i = 14) or (i = 16) or
            (i = 17) or (i = 18) or (i = 20) then
            begin
              if Ans = 0 then // True
                begin
                  Score := Score + 1;
                  ResultString := ResultString + IntToStr(i) + '. True (1 point)' + U;
                end;
              if Ans = 1 then // False
                ResultString := ResultString + IntToStr(i) + '. False (0 point)' + U;
            end;
        end;
    end; // for i

  if (Skipped > 5) then
    Score := -1;

//original RS code
{  Result := 'Beck Hopelessness Scale^^' + IntToStr(Score) + U +
    'Feelings of Future^^' + IntToStr(Future) + U +
    'Loss of Motivation^^' + IntToStr(Motivation) + U +
    'Future Expectations^^' + IntToStr(Expectation) + U +
    ResultString + RECORD_END;
}

  // modified to get scales to work
  Result := 'Beck Hopelessness Scale^^' + IntToStr(Score) + RECORD_END +
            'Feelings of Future^^' + IntToStr(Future) + RECORD_END +
            'Loss of Motivation^^' + IntToStr(Motivation) + RECORD_END +
            'Future Expectations^^' + IntToStr(Expectation) + RECORD_END + U +
            ResultString + RECORD_END;
end;


function ProgressNote_BHS(
  ScoreString: string): string;
// The header, footer and questions and answers are handled else where; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = ' ';
var
  BHS_Score,
    Answers                   : string;
  Count, i,
    Score                     : integer;
begin
  Result := '';

  if (ScoreString = '') then
    Exit;

  //RS code
  //BHS_Score := Piece(ScoreString, U, 3);
  //Score := StrToIntDef(BHS_Score, -1);

  //patch 105 code to make scales work llh - changed RS's code above
  BHS_Score := Piece(Piece(ScoreString, RECORD_END, 1), U, 3);
  Score := StrToIntDef(BHS_Score, -1);

  if (Score < 4) then
    Result := Result + Indent +
      'BHS Score: ' + BHS_Score + ' indicates none or minimal hopelessness. The overall range is 0 to 20 with low hopelessness between 0 - 3.' + #13+ #13;

  if (Score = -1) then
    begin
      Result := '';
      Result := Result + Indent +
        'BHS score could not be determined.'+ #13 + #13;
    end;

  if (Score < 9) and (Score > 3) then
    Result := Result + Indent +
      'BHS Score: ' + BHS_Score + ' indicates mild hopelessness. The overall range is 0 to 20 with mild hopelessness between 4 - 8.' + #13+ #13;

  if (Score < 15) and (Score > 8) then
    Result := Result + Indent +
      'BHS Score: ' + BHS_Score + ' indicates moderate hopelessness. Evaluate suicide risk.  The overall range is 0 to 20 with moderate hopelessness between 9 - 14.' + #13+ #13;

  if (Score > 14) then
    Result := Result + Indent +
      'BHS Score: ' + BHS_Score + ' indicates severe hopelessness. Definite suicide risk. The overall range is 0 to 20 with severe hopelessness above 15.' + #13 + #13;

  Result := Result + Indent +
    //'Feelings of Future: ' + Piece(ScoreString, U, 6) + #13;
    'Feelings of Future: ' +  Piece(Piece(ScoreString, RECORD_END, 2), U, 3) + #13;

  Result := Result + Indent +
    //'Loss of Motivation: ' + Piece(ScoreString, U, 9) + #13;
    'Loss of Motivation: ' +  Piece(Piece(ScoreString, RECORD_END, 3), U, 3) + #13;
  Result := Result + Indent +
    //'Future Expectations: ' + Piece(ScoreString, U, 12) + #13;
    'Future Expectations: ' + Piece(Piece(ScoreString, RECORD_END, 4), U, 3) + #13;

  Result := Result + #13 + Indent +
    'Questions and answers ' + #13 + #13;

  Count := NumberOfPieces(Piece(ScoreString, RECORD_END, 5), U);
  Answers := Piece(ScoreString, RECORD_END, 5) ;

  for i := 2 to Count do
    begin
      if Piece(Answers, U, i) <> '' then
        Result := Result + Indent + Piece(Answers, U, i) + #13;
    end;

end;

end.

