unit uMHA_AUX_FAST;

interface

uses
  SysUtils;

function ScoreAnswers_FAST(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_FAST(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The FAST has an overall score, called a stage, which is the highest level of
  impairment the user checks.
*}

function ScoreAnswers_FAST(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  Count, i, Stage
                              : Integer;

  TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'FAST' then
  begin
     Result := 'YES';
     exit;
  end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  Stage := 1;

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // Disregard all answers except those with item index of 1
      if (TempAnswer = '1') then
        // Find the highest stage
        begin
          if (i = 1) then
            Stage := 1;
          if (i = 2) then
            Stage := 2;
          if (i = 3) and (Stage = 2) then
            Stage := 3;
          if (i = 4) and (Stage = 3) then
            Stage := 4;
          if (i = 5) and (Stage = 4) then
            Stage := 5;

          if (i = 6) and (Stage = 5) then //6a
            Stage := 6;
          if (i = 7) and (Stage = 6) then //6b
            Stage := 7;
          if (i = 8) and (Stage = 7) then //6c
            Stage := 8;
          if (i = 9) and (Stage = 8) then //6d
            Stage := 9;
          if (i = 10) and (Stage = 9) then //6e
            Stage := 10;

          if (i = 11) and (Stage = 10) then //7a
            Stage := 11;
          if (i = 12) and (Stage = 11) then //7b
            Stage := 12;
          if (i = 13) and (Stage = 12) then //7c
            Stage := 13;
          if (i = 14) and (Stage = 13) then //7d
            Stage := 14;
          if (i = 15) and (Stage = 14) then //7e
            Stage := 15;
          if (i = 16) and (Stage = 15) then //7f
            Stage := 16;
        end; // if 1 item index
    end; // for i

  if (Stage > 5) and (Stage < 11) then
    Stage := 6;
  if (Stage > 10) and (Stage < 17) then
    Stage := 7;

    // Note: "Stage" is the name of the scale and it must be used here.
    // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'Stage^^' + Inttostr(Stage) + RECORD_END;
end;


function ProgressNote_FAST(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result :=  Piece(Piece(ScoreString, RECORD_END, 1), U, 3);

end;


end.

