unit uMHA_AUX_QOLI;

interface

uses
  SysUtils;

Type
  TDissatisfaction = array[1..16] of string;

function ScoreAnswers_QOLI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_QOLI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

function sortSatisfaction(scoreString: string) : TDissatisfaction;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


function padScore(StringIn: string): string;
begin
  result := '';
  if StringIn = '99' then exit ;

  if StringIn = '-6' then result :=      '    *'
  else if StringIn = '-4' then result := '        *'
  else if StringIn = '-3' then result := '            *'
  else if StringIn = '-2' then result := '                *'
  else if StringIn = '-1' then result := '                    *'
  else if StringIn = '0' then result :=  '                        *'
  else if StringIn = '1' then result :=  '                            *'
  else if StringIn = '2' then result :=  '                                *'
  else if StringIn = '3' then result :=  '                                    *'
  else if StringIn = '4' then result :=  '                                        *'
  else if StringIn = '6' then result :=  '                                            *'
  else
    result := '' ;

  // original
  {
  if StringIn = '-6' then result :=      '   *'
  else if StringIn = '-4' then result := '        *'
  else if StringIn = '-3' then result := '             *'
  else if StringIn = '-2' then result := '                  *'
  else if StringIn = '-1' then result := '                       *'
  else if StringIn = '0' then result :=  '                            *'
  else if StringIn = '1' then result :=  '                                 *'
  else if StringIn = '2' then result :=  '                                      *'
  else if StringIn = '3' then result :=  '                                           *'
  else if StringIn = '4' then result :=  '                                                *'
  else if StringIn = '6' then result :=  '                                                     *'
  else
    result := '' ;
  }
end;

function PadSpace(StringIn: string; Position: integer): string;
begin
  result := result + StringIn;
  while Length(result) < Position do
    result := ' ' + result ;
end;

function AddSpace(StringIn: string; Position: integer): string;
begin
  result := result + StringIn;
  while Length(result) < Position do
    result := result + ' ';
end;

function GetTScore(raw: extended; wghtSat0: integer; wghtSat99: integer): integer;
// T-score range 0-77
begin
  result := -1;

  if (wghtSat0 = 16) then
  begin
    result := -99;
    exit;
  end;

  if (wghtSat0 = 15) and (wghtSat99 = 1) then
  begin
    result := -99 ;
    exit;
  end;

  if (raw < -3.8) then result := 0 ;
  if (raw = -3.7) then result := 1;
  if (raw = -3.6) then result := 2;
  if (raw = -3.5) then result := 3;
  if (raw = -3.4) then result := 4;
  if (raw = -3.3) then result := 4;
  if (raw = -3.2) then result := 5;
  if (raw = -3.1) then result := 6;
  if (raw = -3.0) then result := 7;
  if (raw = -2.9) then result := 1;
  if (raw = -2.8) then result := 2;
  if (raw = -2.7) then result := 3;
  if (raw = -2.6) then result := 4;
  if (raw = -2.5) then result := 4;
  if (raw = -2.4) then result := 5;
  if (raw = -2.3) then result := 6;
  if (raw = -2.2) then result := 7;
  if (raw = -2.9) then result := 8;
  if (raw = -2.8) then result := 8;
  if (raw = -2.7) then result := 9;
  if (raw = -2.6) then result := 10;
  if (raw = -2.5) then result := 11;
  if (raw = -2.4) then result := 11;
  if (raw = -2.3) then result := 12;
  if (raw = -2.2) then result := 13;
  if (raw = -2.1) then result := 14;
  if (raw = -2.0) then result := 15;
  if (raw = -1.9) then result := 15;
  if (raw = -1.8) then result := 16;
  if (raw = -1.7) then result := 17;
  if (raw = -1.6) then result := 18;
  if (raw = -1.5) then result := 18;
  if (raw = -1.4) then result := 19;
  if (raw = -1.3) then result := 20;
  if (raw = -1.2) then result := 21;
  if (raw = -1.1) then result := 22;
  if (raw = -1.0) then result := 22;
  if (raw = -0.9) then result := 23;
  if (raw = -0.8) then result := 24;
  if (raw = -0.7) then result := 25;
  if (raw = -0.6) then result := 25;
  if (raw = -0.5) then result := 26;
  if (raw = -0.4) then result := 27;
  if (raw = -0.3) then result := 28;
  if (raw = -0.2) then result := 29;
  if (raw = -0.1) then result := 29;
  if (raw = 0) then result := 30;
  if (raw = 0.1) then result := 31;
  if (raw = 0.2) then result := 32;
  if (raw = 0.3) then result := 32;
  if (raw = 0.4) then result := 33;
  if (raw = 0.5) then result := 34;
  if (raw = 0.6) then result := 35;
  if (raw = 0.7) then result := 36;
  if (raw = 0.8) then result := 36;
  if (raw = 0.9) then result := 37;
  if (raw = 1.0) then result := 38;
  if (raw = 1.1) then result := 39;
  if (raw = 1.2) then result := 39;
  if (raw = 1.3) then result := 40;
  if (raw = 1.4) then result := 41;
  if (raw = 1.5) then result := 42;
  if (raw = 1.6) then result := 43;
  if (raw = 1.7) then result := 43;
  if (raw = 1.8) then result := 44;
  if (raw = 1.9) then result := 45;
  if (raw = 2.0) then result := 46;
  if (raw = 2.1) then result := 46;
  if (raw = 2.2) then result := 47;
  if (raw = 2.3) then result := 48;
  if (raw = 2.4) then result := 49;
  if (raw = 2.5) then result := 50;
  if (raw = 2.6) then result := 50;
  if (raw = 2.7) then result := 51;
  if (raw = 2.8) then result := 52;
  if (raw = 2.9) then result := 53;
  if (raw = 3.0) then result := 53;
  if (raw = 3.1) then result := 54;
  if (raw = 3.2) then result := 55;
  if (raw = 3.3) then result := 56;
  if (raw = 3.4) then result := 57;
  if (raw = 3.5) then result := 57;
  if (raw = 3.6) then result := 58;
  if (raw = 3.7) then result := 59;
  if (raw = 3.8) then result := 60;
  if (raw = 3.9) then result := 61;
  if (raw = 4.0) then result := 61;
  if (raw = 4.1) then result := 62;
  if (raw = 4.2) then result := 63;
  if (raw = 4.3) then result := 64;
  if (raw = 4.4) then result := 64;
  if (raw = 4.5) then result := 65;
  if (raw = 4.6) then result := 66;
  if (raw = 4.7) then result := 67;
  if (raw = 4.8) then result := 68;
  if (raw = 4.9) then result := 68;
  if (raw = 5.0) then result := 69;
  if (raw = 5.1) then result := 70;
  if (raw = 5.2) then result := 71;
  if (raw = 5.3) then result := 71;
  if (raw = 5.4) then result := 72;
  if (raw = 5.5) then result := 73;
  if (raw = 5.6) then result := 74;
  if (raw = 5.7) then result := 75;
  if (raw = 5.8) then result := 75;
  if (raw = 5.9) then result := 76;
  if (raw > 5.9) then result := 77;
end;

function GetPScore(raw: extended; wghtSat0: integer; wghtSat99: integer): integer;
// P-score range 1-99
// invalid score = -99
begin
  result := -1;

  if (wghtSat0 = 16) then
  begin
    result := -99;
    exit;
  end;

  if (wghtSat0 = 15) and (wghtSat99 = 1) then
  begin
    result := -99 ;
    exit;
  end;

  if (raw < -1.2) then result := 1;
  if (raw = -1.1) then result := 2;
  if (raw = -1.0) then result := 2;
  if (raw = -0.9) then result := 2;
  if (raw = -0.8) then result := 2;
  if (raw = -0.7) then result := 2;
  if (raw = -0.6) then result := 2;
  if (raw = -0.5) then result := 2;
  if (raw = -0.4) then result := 3;
  if (raw = -0.3) then result := 3;
  if (raw = -0.2) then result := 3;
  if (raw = -0.1) then result := 4;
  if (raw =  0) then result := 4;
  if (raw = 0.1) then result := 5;
  if (raw = 0.2) then result := 5;
  if (raw = 0.3) then result := 5;
  if (raw = 0.4) then result := 6;
  if (raw = 0.5) then result := 6;
  if (raw = 0.6) then result := 7;
  if (raw = 0.7) then result := 9;
  if (raw = 0.8) then result := 10;
  if (raw = 0.9) then result := 11;
  if (raw = 1.0) then result := 12;
  if (raw = 1.1) then result := 15;
  if (raw = 1.2) then result := 16;
  if (raw = 1.3) then result := 17;
  if (raw = 1.4) then result := 18;
  if (raw = 1.5) then result := 19;
  if (raw = 1.6) then result := 21;
  if (raw = 1.7) then result := 23;
  if (raw = 1.8) then result := 26;
  if (raw = 1.9) then result := 28;
  if (raw = 2.0) then result := 30;
  if (raw = 2.1) then result := 34;
  if (raw = 2.2) then result := 36;
  if (raw = 2.3) then result := 39;
  if (raw = 2.4) then result := 42;
  if (raw = 2.5) then result := 46;
  if (raw = 2.6) then result := 49;
  if (raw = 2.7) then result := 51;
  if (raw = 2.8) then result := 55;
  if (raw = 2.9) then result := 59;
  if (raw = 3.0) then result := 61;
  if (raw = 3.1) then result := 66;
  if (raw = 3.2) then result := 67;
  if (raw = 3.3) then result := 71;
  if (raw = 3.4) then result := 75;
  if (raw = 3.5) then result := 78;
  if (raw = 3.6) then result := 81;
  if (raw = 3.7) then result := 84;
  if (raw = 3.8) then result := 86;
  if (raw = 3.9) then result := 89;
  if (raw = 4.0) then result := 90;
  if (raw = 4.1) then result := 92;
  if (raw = 4.2) then result := 93;
  if (raw = 4.3) then result := 95;
  if (raw = 4.4) then result := 96;
  if (raw = 4.5) then result := 97;
  if (raw = 4.6) then result := 97;
  if (raw = 4.7) then result := 98;
  if (raw = 4.8) then result := 98;
  if (raw > 4.8) then result := 99;
end;

function ScoreAnswers_QOLI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  Count, i, validScore,
    wghtSat0,wghtSat99         : Integer;

  rawScore                     : double;

  raw   : extended ;

  TempResult,TempAnswer,
    omittedItems,
    responses                  : string;

   tempScale: array[1..32] of Integer;
   itemScore: array[1..32] of string;
   scale:  array[1..16] of Integer ;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'QOLI' then
  begin
     Result := 'YES';
     exit;
  end;

  validScore := 0 ;
  rawScore := 0 ;
  wghtSat0 := 0 ;
  wghtSat99 := 0 ;
  omittedItems := '' ;
  responses := ''  ;

  for i := 1 to 16 do scale[i] := 0 ;

  Result := '';
  Count := NumberOfPieces(AnswerString, '~');

  if Count < 1 then
    Exit;

  for i := 1 to Count -1 do
  begin
    TempResult := Piece(AnswerString, '~', i);
    TempAnswer := Piece(Piece(TempResult, '^', 1), '=', 2);

    // must convert entered response of 1 (itemindex = 0) to 1 for display
    case strtoint(tempAnswer) of
    0: itemScore[i] := '1';
    1: itemScore[i] := '2';
    2: itemScore[i] := '3';
    3: itemScore[i] := '4';
    4: itemScore[i] := '5';
    5: itemScore[i] := '6';
    else
      itemScore[i] := '' ;
    end;


    // no conversion yet, itemIndex will be non negative, if unanswered
    if (strToInt(TempAnswer) = 1155) then
    begin
      tempScale[i] := -99 ;
      omittedItems := omittedItems + inttostr(i) + ','  ;
    end
    else
    begin
      tempScale[i] := strToInt(TempAnswer) ;
    end;

    // need to have a 2nd question before can do any calculations
    if i = 1 then
    begin
      responses := responses + inttostr(i) + ':' + itemscore[i] + '|' ;
      continue ;
    end;

    // getting even numbered questions, satisfaction rating, needs conversion
    // importance rating does not need any conversion.
    if (i mod 2) = 0  then
    begin
      case tempScale[i] of
      0: tempScale[i] := -3;
      1: tempScale[i] := -2;
      2: tempScale[i] := -1;
      3: tempScale[i] := 1;
      4: tempScale[i] := 2;
      5:  tempScale[i] := 3;
      else
        tempScale[i] := tempScale[i] ;
      end;

      // compute the scale, -99 = unendorsed
      if (i = 26) and (tempScale[i-1] = 0) and (tempScale[i] = -99) then
        begin
          scale[i div 2] := 0  ;
        end
      else if (tempScale[i-1] = -99) or (tempScale[i] = -99) then
        begin
          scale[i div 2] := -99     ;
        end
      else
        begin
          scale[i div 2] := tempScale[i-1] * tempScale[i] ;
        end;

      // compute the raw score and determine if a valid importance item (odd # question)
      if scale[i div 2] <> -99 then
      begin
        rawScore := rawScore + scale[i div 2] ;
        if tempScale[i-1] > 0 then validScore := validScore + 1 ;
      end;
      // used to compute invalid percentile score
      if scale[i div 2] = 0 then wghtSat0 := wghtSat0 + 1  ;
      if scale[i div 2] = -99 then wghtSat99 := wghtSat99 + 1;

    end;  // end (i mod 2) block

    //collect value of all individual questions with the value entered
    responses := responses + inttostr(i) + ':' + itemscore[i] + '|' ;

  end; // end loop thru AnswerString

  // compute rawScore
  if validScore <> 0 then rawScore := ((rawScore / validScore) +  0.05)
    else rawScore := 0.05 ;

  tempAnswer := floattostr(rawScore);
  TempResult := Piece(tempAnswer,'.',1) + '.' + Copy(Piece(tempAnswer,'.',2),0,1) ;
  raw := strtofloat(TempResult);

  Result := 'Health' + U + U + IntToStr(scale[1]) {Raw Score} + RECORD_END +
            'Self-Esteem'+ U + U + IntToStr(scale[2]) {Raw Score} + RECORD_END +
            'Goals-and-Values'+ U + U + IntToStr(scale[3]) {Raw Score} + RECORD_END +
            'Money'+ U + U + IntToStr(scale[4]) {Raw Score} + RECORD_END +
            'Work'+ U + U + IntToStr(scale[5]) {Raw Score} + RECORD_END +
            'Play'+ U + U + IntToStr(scale[6]) {Raw Score} + RECORD_END +
            'Learning'+ U + U + IntToStr(scale[7]) {Raw Score} + RECORD_END +
            'Creativity'+ U + U + IntToStr(scale[8]) {Raw Score} + RECORD_END +
            'Helping'+ U + U + IntToStr(scale[9]) {Raw Score} + RECORD_END +
            'Love'+ U + U + IntToStr(scale[10]) {Raw Score} + RECORD_END +
            'Friends'+ U + U + IntToStr(scale[11]) {Raw Score} + RECORD_END +
            'Children'+ U + U + IntToStr(scale[12]) {Raw Score} + RECORD_END +
            'Relatives'+ U + U + IntToStr(scale[13]) {Raw Score} + RECORD_END +
            'Home'+ U + U + IntToStr(scale[14]) {Raw Score} + RECORD_END +
            'Neighborhood'+ U + U + IntToStr(scale[15]) {Raw Score} + RECORD_END +
            'Community'+ U + U + IntToStr(scale[16]) {Raw Score} + RECORD_END +
            'QOLI_R' + U + TempResult {QOLI raw score} + RECORD_END +
            'QOLI_T' + U + inttostr(GetTScore(raw,wghtSat0,wghtSat99)) {QOLI T-score} + RECORD_END +
            'QOLI_P' + U + inttostr(GetPScore(raw,wghtSat0,wghtSat99)) {QOLI P-score} + RECORD_END +
            'INV_SCALES' + U + inttostr((16-validScore)) {#invalid} + RECORD_END +
            'Omitted' + U + omittedItems + RECORD_END +   {piece 21}
            'responses' + U + responses {string with question responses} + RECORD_END ;
end;


function ProgressNote_QOLI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

const
  Indent                             : string = '   ';
var
  i,cnt                             : Integer;

  TempStr,tScore,tScoreBar,
    omitted, scale, items,
    templine,endScale               : string;

  //areaofsat :   array[1..16] of string;
  dissat :    TDissatisfaction ;

begin
  Result := '';
  TempStr := '' ;

  if (ScoreString = '') then Exit;

  omitted := Piece(Piece(ScoreString, RECORD_END, 21),U,2) ;
  delete(omitted, length(omitted), 1);

  items := Piece(Piece(ScoreString, RECORD_END, 22), U, 2) ;


  // call function to sort the array
  dissat := sortSatisfaction(scoreString) ;

  // to be removed
  {tScore :='' ;
  for i := 1 to 22 do
    tScore := tScore + Piece(ScoreString, RECORD_END, i) + #13;
  showmessage(tScore) ;
  // end remove code
  }

  tScoreBar := '' ;
  tScore := Piece(Piece(ScoreString, RECORD_END, 18), U, 2) ;
  for i := 0 to (strtoInt(tScore) -1) do tScoreBar := tScoreBar + '*' ;

  Result := #13#13 + Indent + 'INTRODUCTION' + #13 ;
  Result := Result + #13 + Indent +'The Quality of Life Inventory (QOLI) provides a score that ' +
    'indicates a person' + #39 + 's overall ' + #13 + Indent + 'satisfaction with ' + 'life. People' + #39 +
    's life satisfaction is based on how well their needs, ' + #13 + Indent + 'goals, and wishes are being met in ' +
    'important areas of life.' + #13#13;

  Result := Result + Indent + '(Raw Score: ' + Piece(Piece(ScoreString, RECORD_END, 17), U, 2) + ')' + #13;
  Result := Result + Indent + ' T Score: ' + tScore + #13 ;
  Result := Result + Indent + '(%ile Score: ' + Piece(Piece(ScoreString, RECORD_END, 19), U, 2) + ')' + #13#13;

  Result := Result + #13#13#13 + Indent + 'Overall Quality of Life' ;
  Result := Result + #13#13 + Indent + '               VERY LOW             LOW        AVERAGE            HIGH' ;
  Result := Result + #13 + Indent + '-----------------------------------------------------------------------------' ;
  Result := Result + #13 + Indent + tScoreBar ;
  Result := Result + #13 + Indent + '-----------------------------------------------------------------------------' ;
  Result := Result + #13 + Indent + '|                                   |      |               |                |' ;
  Result := Result + #13 + Indent + '0                                  37       43             58               77' ;

  Result := Result + #13#13#13 + Indent + 'Weighted Satisfaction Profile' + #13;
  Result := Result + #13 + Indent + '                    DISSATISFACTION              SATISFACTION' ;
  Result := Result + #13 + Indent + '                   -6  -4  -3  -2  -1   0   1   2   3   4   6' ;
  Result := Result + #13 + Indent + '                ---------------------------------------------' ;

  for i := 1 to 16 do
  begin
    templine := '' ;
    tscore := '' ;
    endScale := '';
    scale := Piece(ScoreString, RECORD_END, i) ;
    tscore := padScore(Piece(scale,U,3)) ;
    tempLine := PadSpace(Piece(Scale,U,1),16);
    endScale := PadSpace('|', (46 - length(tscore))) ;
    if templine <> '' then  result := result + #13 + '  ' + tempLine + '|' + tscore +
                                   endScale + Piece(Scale,U,1) ;
  end;

  Result := Result + #13 +Indent + '                ---------------------------------------------' ;
  Result := Result + #13 + Indent + '                   -6  -4  -3  -2  -1   0   1   2   3   4   6' ;



  Result := Result + #13#13#13#13 + Indent + 'The following weighted satisfaction ratings indicate areas of dissatisfaction' +
                              #13 + Indent + 'for this person:' + #13;
  Result := Result + #13 + Indent + Indent + '                           Weighted ' ;
  Result := Result + #13 + Indent + Indent + 'Area                  Satisfaction Rating' ;
  Result := Result + #13 + Indent + Indent + '----                  -------------------' + #13;

  for i := 1 to High(dissat) do
  begin
    templine := '' ;
    if (dissat[i] <> '') then tempLine := AddSpace(Piece(dissat[i],U,1),32) ;
    if templine <> '' then result := result + Indent + Indent + tempLine + Piece(dissat[i],U,3) + #13;
  end;

  Result := Result + #13#13 + Indent + 'OMITTED ITEMS';
  if omitted = '' then
    Result := Result + #13 + Indent + Indent + 'None omitted'
  else
    Result := Result + #13 + Indent + Indent + omitted + #13#13 ;

  // Response summary, but first convert any response of -99 to ''
  result := result + #13#13 + Indent + 'ITEM RESPONSES' + #13;
  result := result + #13 + Indent + Indent ;
  for i := 1 to 9 do
    result := result + ' ' + Piece(items,'|',i) + Indent + ' ' ;
  result := result + Piece(items,'|',10) + #13 + Indent + Indent;
  for i := 11 to 20 do
    result := result + Piece(items,'|',i) + Indent + ' ' ;
  result := result + #13 + Indent + Indent;
  for i := 21 to 30 do
    result := result + Piece(items,'|',i) + Indent + ' ' ;
  result := result + #13 + Indent + Indent;
  for i := 31 to 32 do
    result := result + Piece(items,'|',i) + Indent + ' ' ;

end;


function sortSatisfaction(scoreString: string) : TDissatisfaction;
var
  i, cnt :    integer;

begin
  cnt := 1 ;

  for i := 1 to 16 do
  begin
    if Piece(Piece(ScoreString, RECORD_END, i),U,3) = '-6' then
    begin
      result[cnt] := Piece(ScoreString, RECORD_END, i) ;
      cnt := cnt + 1;
    end  ;
  end;

  for i := 1 to 16 do
  begin
    if Piece(Piece(ScoreString, RECORD_END, i),U,3) = '-4' then
    begin
      result[cnt] := Piece(ScoreString, RECORD_END, i) ;
      cnt := cnt + 1;
    end
  end;

  for i := 1 to 16 do
  begin
    if Piece(Piece(ScoreString, RECORD_END, i),U,3) = '-3' then
    begin
      result[cnt] := Piece(ScoreString, RECORD_END, i) ;
      cnt := cnt + 1;
    end
  end;

  for i := 1 to 16 do
  begin
    if Piece(Piece(ScoreString, RECORD_END, i),U,3) = '-2' then
    begin
      result[cnt] := Piece(ScoreString, RECORD_END, i) ;
      cnt := cnt + 1;
    end
  end;

  for i := 1 to 16 do
  begin
    if Piece(Piece(ScoreString, RECORD_END, i),U,3) = '-1' then
    begin
      result[cnt] := Piece(ScoreString, RECORD_END, i) ;
      cnt := cnt + 1;
    end
  end;
end;


end.

