unit uIA_Instrument_IO;

interface

uses
  Trpcb,
  classes,
  uIA_Instrument;

function InstrumentReadDescriptiveProperties(
  Broker: TRPCBroker;
  var aInstrument: TInstrument): string;

implementation

uses
  SysUtils,
  dialogs,
  uIA_Const,
  uIA_Utils;



function InstrumentReadDescriptiveProperties(
  Broker: TRPCBroker;
  var aInstrument: TInstrument): string;

var
  TempStr                     : string;

// Result of YTQ TSLIST1 is"
// [DATA]
// IEN=5
// NAME=AUDC
// PRINT TITLE=Alcohol Use Disorders Identification Test Consumption
// VERSION=
// AUTHOR=Bradley; McDonell; Bush; Kivlahan; Diehr; Fihn
// PUBLISHER=Alcoholism:Clinical & Experimental Research
// PUBLICATION DATE=1998
// REFERENCE=Alcoholism: Clinical & Experimental Research. 22(8):1842, November 1998
// A PRIVILEGE=
// R PRIVILEGE=
// OPERATIONAL=Yes
// HAS BEEN OPERATIONAL=Yes
// REQUIRES LICENSE=No
// PURPOSE=Alcohol Screening.  3 items from WHO AUDIT test dealing with consumption of alcohol.
// NORM SAMPLE=
// TARGET POPULATION=
// ENTERED BY=A FINKELSTEIN
// ENTRY DATE=FEB 26, 2004
// LAST EDITED BY=FINKELSTEIN,ALLAN
// LAST EDIT DATE=3090409
// IS NATIONAL TEST=Yes
// LICENSE CURRENT=No
// COPYRIGHT TEXT=
// REQUIRES SIGNATURE=No
// IS LEGACY=No
// SUBMIT TO NATIONAL DB=Yes
// IS COPYRIGHTED=No
// WRITE FULL TEXT=Yes
// DAYS TO RESTART=2
// GENERATE PNOTE=Yes
// TIU TITLE=MENTAL HEALTH DIAGNOSTIC STUDY NOTE
// Consult_Note=MENTAL HEALTH CONSULT NOTE
// YS_MHA_B.dll Version  -- not used here
// YS_MHA_B.dll Date     -- not used here
// DLL Version = 1.0.3.100
// DLL_Date = FileMan Date -- not used here

begin
  Result := '';

  with Broker, aInstrument do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ TSLIST1';
      Param[0].Mult['"CODE"'] := InstrumentName;
      Param[0].PType := List;
      Call;

      if (Results[0] = DATA) then
        begin
          IEN := Piece(Results[1], EQ, 2);
          InstrumentName := Piece(Results[2], EQ, 2); // Careful, there is a Broker.Name
          PrintTitle := Piece(Results[3], EQ, 2);
          Version := Piece(Results[4], EQ, 2);
          Author := Piece(Results[5], EQ, 2);
          Publisher := Piece(Results[6], EQ, 2);
          PublicationDate := Piece(Results[7], EQ, 2);
          Reference := Piece(Results[8], EQ, 2);
          A_Privilege := Piece(Results[9], EQ, 2);
          R_Privilege := Piece(Results[10], EQ, 2);
          IsOperational := 'N'; // Default

          TempStr := Piece(Results[11], EQ, 2);
          if (Length(TempStr) > 0) and
            (TempStr[1] in ['Y', 'N', 'U']) then
            IsOperational := TempStr[1]
          else
            IsOperational := 'N';

          IsUnderDevelopment := StrToBool(Piece(Results[12], EQ, 2));
          RequiresLicense := StrToBool(Piece(Results[13], EQ, 2));
          Purpose := Piece(Results[14], EQ, 2);
          NormSample := Piece(Results[15], EQ, 2);
          TargetPopulation := Piece(Results[16], EQ, 2);
          EnteredBy := Piece(Results[17], EQ, 2);
          EntryDate := Piece(Results[18], EQ, 2);
          LastEditedBy := Piece(Results[19], EQ, 2);
          LastEditDate := FMDateStrToDateStr(Piece(Results[20], EQ, 2));
          LastEditFMDate := Piece(Results[20], EQ, 2);
          IsNationalTest := StrToBool(Piece(Results[21], EQ, 2));
          LicenseIsCurrent := StrToBool(Piece(Results[22], EQ, 2));
          Copyright := Piece(Results[23], EQ, 2);
          RequiresSignature := StrToBool(Piece(Results[24], EQ, 2));
          Islegacy := StrToBool(Piece(Results[25], EQ, 2));
          SubmitToNationalDB := StrToBool(Piece(Results[26], EQ, 2));
          IsCopyrighted := StrToBool(Piece(Results[27], EQ, 2));
          WriteFullText := StrToBool(Piece(Results[28], EQ, 2));
          DaysToRestart := StrToIntDef(Piece(Results[29], EQ, 2), 2);
          GenerateProgressNote := StrToBool(Piece(Results[30], EQ, 2));
          TIU_Title := Piece(Results[31], EQ, 2);
          Consult_Title := Piece(Results[32], EQ, 2);
          // 33 YS_MHA_B.dll version  -- Not used here
          // 34 YS_MHA_B.dll date     -- Not used here
          DLL_Version := Piece(Results[35], EQ, 2); // Version of YS_MHA_AUX.dll
          DLL_Date := Piece(Results[36], EQ, 2);
        end
      else
        begin
          Result := BAD_DATA;
          Exit;
        end; //else
      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; //with Broker

end;

end.

